function [a,b,c,d]=cubic_coeff(x1,dx,u1,u2,u3,u4)
% CUBIC_COEFF.M - defines coefficients for cubic fit to determine shock
% formation. Maximum gradient of u located between u3=u(n)and u2=u(n-1),
% u1=u(n-2), u4=u(n+1), x1=x(n-2)
% Refer to Section 2.9 "Detection of Shock Formation"
%  A SHOCK FITTING PRIMER - CHAPTER 2, "A More Difficult Example"
%
%  Copyright 2005-2008 M.D.Salas.


d1=u1*u2*(u3-u4)-((u1+u2)*u3-(u1+u2+u3-u4)*u4)*u4;
d2=u1*u2*(u3-u4)+((u1+u2)*u4+(u3-u1-u2-u4)*u3)*u3;
d3=u1*u3*(u4-u2)-((u1+u3)*u4-(u1+u3+u4-u2)*u2)*u2;
d4=u2*u3*(u1-u4)+((u2+u3)*u4+(u1-u4-u3-u2)*u1)*u1;

e1=x1/d4;
e2=(x1+dx)/d3;
e3=(x1+2*dx)/d2;
e4=(x1+3*dx)/d1;

a=e1-e2+e3-e4;                                                              % coefficients of x=a*u^3+b*u^2+c*u+d
b=-(u2+u3+u4)*e1+(u1+u3+u4)*e2-(u1+u2+u4)*e3+(u1+u2+u3)*e4;
c=(u2*(u3+u4)+u4*u3)*e1-(u1*(u3+u4)+u4*u3)*e2+...
    (u1*(u2+u4)+u2*u4)*e3-(u1*(u2+u3)+u2*u3)*e4;
d=(u1*e2-u2*e1)*u3*u4+(u3*e4-u4*e3)*u1*u2;