%initial12_gauss.m
% INITIAL12_GAUSS.M -defines Gaussian initial conditions
%  A SHOCK FITTING PRIMER - CHAPTER 2
%
% Solves a non-conservative system of 2 PDE's
% dr/dt+(u+1)dr/dx=0, ds/dt+(u-1)ds/dt=0, r=u+g, s=u-g.
%
% INIT=4 for "r" gaussian distribution
% INIT=5 for "s" gaussian distribution
%
%  Copyright 2005-2008 M.D.Salas.


global eps0 dx I N NT nfig kmax kmax_save xl xr tlast t
global x u g r s R S INIT
disp('-----')
tlast=.8

xl=-3;                                                                      
xr=3;
t=0;
N=200;
NT=N+1;
kmax=0;                                                                     
kmax_save=kmax;                                                             % kmax_save is only used for plotting

eps0=10^-10;                                                                % small number
nfig=0                                                                     % figure counter
dx=(xr-xl)/N;                                                               % mesh interval size

for n=1:NT
    x(n)=xl+dx*(n-1);                                                       % physical coordinate [xs,xr]
end
                                                                            %      SHOCK INDEXES
for n=1:NT
    I(n,1)=0;
    I(n,2)=0;
end

if INIT == 4
    disp('Corresponding to r-gaussian initial conditions')
    s0=0;
    useed=0;
    sig=1;
    for n=1:NT
        s(n)=s0;
        r(n)=gauss_init(x(n),t,useed,sig,n);                                
        useed=r(n);
        S(n)=s(n);
        R(n)=r(n);
        u(n)=(r(n)+s(n))/2;
        g(n)=(r(n)-s(n))/2;
    end
elseif INIT == 5
    disp('Corresponding to s-gaussian initial conditions')
    r0=0;
    useed=0;
    sig=-1;
    for n=1:NT
        r(n)=r0;
        s(n)=gauss_init(x(n),t,useed,sig,n);                                
        useed=s(n);
        S(n)=s(n);
        R(n)=r(n);
        u(n)=(r(n)+s(n))/2;
        g(n)=(r(n)-s(n))/2;
    end
end

