function u=gauss_bc(x,t,useed,sig)
% GAUSS_BC.M  updates u at boundaries by MOC
% A SHOCK FITTING PRIMER - CHAPTER 2, "Application of Background Scheme".
% x & t are the coordinates of the boundary point
% useed is the current value of u(x,t)
% This function is for a Gaussian distribution at t=0
% of the form "sig*exp(-x0^2)". It updates u by finding x0.
%
% Copyright 2005-2008 M.D.Salas.

eps0=10^-12;
                                                                            % find initial guess to x0
x0=x-useed*t;
                                                                            % Newton iteration for x0
for k=1:10
    c=sig*exp(-x0^2)*t;
    f=x0+c-x;
    fx0=1-2*x0*c;
    err=f/fx0;
    x0=x0-err;
    if abs(err)<=eps0
        break
    end
end
u=sig*exp(-x0^2);
if abs(err) > eps0
    disp(['at t= ', num2str(t),' x= ', num2str(x),..
        ' bc error exceeded, err= ', num2str(err)])
end
