function plot_solution_cap01
% PLOT_SOLUTION_CAP01.M plot results
% A SHOCK FITTING PRIMER - CHAPTER 2, "Application of Background Scheme".
% works with SHOCKCAP01.M
%
% Copyright 2005-2008 M.D.Salas.

global t tshock nfig kn ts xs INIT x dx u N kout

disp('Output from SHOCKCAP01:')
disp(' ')
disp(['After ' num2str(kout) ' time steps'])

if t > tshock
    disp(['at time= ' num2str(ts(kn)) ' and xs= ' num2str(xs(kn))])
    nfig=nfig+1;
    figure(nfig)                                                            % plot shock-path computed and exact
    if INIT == 2
        plot(xs,ts,'ko',sqrt(2.*(1+ts))-1.,ts,'k-')                         % exact sock-path for "one-saw-tooth" problem
        axis([1.,2.5,1.,5.])
        xlabel('x','FontSize',16)
        ylabel('time','FontSize',16)
        legend('computed','exact')
    else                                                                    % exact shock-path data for Gaussian initial conditions
        xsdata=[1.4142 1.4349 1.4943 1.5234 1.5523 1.5809 1.6091 ...
                1.6371 1.6648 1.7421 1.7554 1.7730 1.8776 2.0038 ...
                2.1256 2.3576 2.5765 2.7843 2.9825 3.1722]; 
        tdata= [1.1658 1.2000 1.3000 1.3500 1.4000 1.4500 1.5000 ...
                1.5500 1.6000 1.7421 1.7670 1.8000 2.0000 2.2500 ... 
                2.5000 3.0000 3.5000 4.0000 4.5000 5.0000];
        plot(xs,ts,'ko',xsdata,tdata,'k-')
        axis([1.4,3.2,1.1,5.])
        xlabel('x','FontSize',16)
        ylabel('time','FontSize',16)
        legend('computed','exact')
        title('Shock Wave Path')
    end
end

nfig=nfig+1;
figure(nfig)	                    % plot u profile 
hold on
if t > tshock
    if INIT == 2
        xes=sqrt(2*(1+t))-1;         % exact shock location for "one-saw-tooth" problem
        for n=1:10
            xe(n)=-1.+(xes+1.)*n/10;
            ue(n)=(1.+xe(n))/(1.+t);
        end
        xe(11)=xe(10);
        ue(11)=0.;
    end
    if INIT == 1                      % exact solution at t=5 for Gaussian initial conditions
        ue=[.05 .1 .15 .2 .25 .3 .35 .4 .45 .5 .55 .6 .65 .7 .74336 4.2696e-005];
        xe=[-1.4808 -1.0174 -0.62736 -0.26864 0.07259 0.40274 0.72539 1.0428 1.3564...
                1.6674 1.9768 2.2853 2.5937 2.9028 3.1722 3.1722];
        xes=exactgauss_xs(t);
    end
    error=1.-xs(kn)/xes;
    disp(['the error in shock location is ' num2str(error)])
end

error=0;
if INIT==2  
    for n=1:N
        xplot(n)=x(n);
        uexact=0;
        if x(n) > -1 && x(n) < xes
            uexact=(1+x(n))/(1+t);
        end
        drs(n)=u(n)-uexact;
        error=error+abs(drs(n));
    end
    [drmax,nmax]=max(abs(drs)); 
    error=error/N;
    disp(' ')
    disp(['the average error in u is ' num2str(error)])
    disp(['The max error in u is ' num2str(drmax) ' at n= ' num2str(nmax)])
    
elseif INIT == 1
    for n=1:N
        xplot(n)=x(n);
        x1=x(n)-.5*dx;
        x2=x(n)+.5*dx;
        u1=gauss_init(x1,t,u(n),1,n);
        u2=gauss_init(x(n),t,u1,1,n);
        u3=gauss_init(x2,t,u2,1,n);
        uexact=.25*(2*u2+u1+u3);
        drs(n)=u(n)-uexact;
        error=error+abs(drs(n));
    end
    [drmax,nmax]=max(abs(drs)); 
    error=error/N;
    disp(' ')
    disp(['the average error in u is ' num2str(error)])
    disp(['The max error in u is ' num2str(drmax) ' at n= ' num2str(nmax)])
end

for n=1:N
    drs(n)=.5*drs(n)/drmax;
end
if t==5
    plot(x,u,'k-',xe, ue,'ko',xplot,drs,'k--')
else
    plot(x,u,'k-',xplot,drs,'k--')
end
xlabel('x','FontSize',16)
ylabel('u','FontSize',16)
legend('computed','exact','error')
hold off