function u=gauss_init(x,t,useed,sig,n)
% GAUSS_INIT.M - exact u at x,t for Gaussian: u(x,t)=sign*exp(-(x-ut)^2)
% A SHOCK FITTING PRIMER - CHAPTER 2, "Boundary Shock-Fitting"
%
% "useed" is a starting value of u for Newton iteration
% n is the mesh point index for u(m,n)
%
% Copyright 2005-2008 M.D.Salas.

eps0=10^-12;
u=useed;
for k=1:10
    f=u-sig*exp(-(x-u*t)^2);
    fu=1+2*(x-u*t)*t*(f-u);
    err=f/fu;
    if abs(err)<eps0
        break
    end
    u=u-err;
end
if abs(err) > eps0
    disp('WARNING: the error in the exact u profile')
    disp(['exceeded the set tolerance, u= ' num2str(u)...
        ' at n= ' num2str(n)])
    disp(['error= ' num2str(err)])
end

