function plot_solution(INDEX)
% PLOT_SOLUTION.M creates plots
% A SHOCK FITTING PRIMER - CHAPTER 2, "Boundary Shock-Fitting".
%
%
% Copyright 2005-2008 M.D.Salas.

global z zx drs drs2
global xLEFT tout xsout sig
global Nlast N INIT nfig
global x u xs usL usR t tlast xl xr

if INDEX == 1
    nfig=nfig+1;
    figure(nfig)                                                            % initial u-profile
    for n=2:Nlast(1)
        nn=n-1;
        xplot1(nn)=x(1,n);
        uplot1(nn)=u(1,n);
    end

    for n=2:Nlast(2)
        nn=n-1;
        xplot2(nn)=x(2,n);
        uplot2(nn)=u(2,n);
    end
    xsplot=[xs,xs];
    usplot=[usL,usR];
    plot(xplot1,uplot1,'k-',xplot2,uplot2,'k-',xsplot,usplot,'k-o')
    if INIT == 1 || INIT == 2
        axis([xl,xr, -.1, 1.1])
    else
        axis([xl,xr, -1.1, .1])
    end
    xlabel('x')
    ylabel('u')

else

    if INIT == 1 || INIT == 4
        xsexact=sig*exactgauss_xs(t);
        error=1.-xs/xsexact;
        disp(['The error in shock location is ' num2str(error)])
    end

    if INIT == 2 || INIT == 3
        xsexact=sig*(sqrt(2*(1+t))-1);
        error=1.-xs/xsexact;
        disp(['the error in shock location is ' num2str(error)])
    end

    for m=1:2
        for n=1:Nlast(m)
            x(m,n)=xLEFT(m)+z(m,n)/zx(m);
        end
    end

    for n=2:Nlast(1)
        nn=n-1;
        xplot1(nn)=x(1,n);
        uplot1(nn)=u(1,n);
    end

    for n=2:Nlast(2)
        nn=n-1;
        xplot2(nn)=x(2,n);
        uplot2(nn)=u(2,n);
    end

    xsplot=[xs,xs];
    usplot=[usL,usR];

    nfig=nfig+1;
    figure(nfig)                                                            % computed & exact u-profile and error plot
    hold on
    if (INIT == 1 || INIT == 4)
                                                                            % data for Gaussian at t=5
        ued=[0.0100  0.0500 0.1000 0.1500 0.2000 0.2500 0.3000...
            0.3500 0.4000 0.4500 0.5000 0.5500 0.6000 0.6500 0.7000...
            0.7434];
        xed=[-2.0960 -1.4808 -1.0174 -0.6274 -0.2686 0.0726 0.4027...
            0.7254 1.0428 1.3564 1.6674 1.9768 2.2853 2.5937 2.9028...
            3.1722];
        plot(xplot1,uplot1,'k-',sig*xed,sig*ued,'ko',xplot2,uplot2,...
            'k-',xsplot,usplot,'k-')
        xlabel('x')
        ylabel('u')
        legend('computed','exact')
    else
        for m=1:2
            for n=1:11
                xed(n)=xLEFT(m)+((n-1)/10)/zx(m);
                if m == 1 && INIT == 2
                    ued(n)=0;
                    if (xed(n) > -1)
                        ued(n)=(1+xed(n))/(1+t);
                    end
                elseif m == 1 && INIT == 3
                    ued(n)=0;
                elseif m == 2 && INIT == 2
                    ued(n)=0;
                elseif m == 2 && INIT == 3
                    ued(n)=0;
                    if (xed(n) < 1)
                        ued(n)=(-1+xed(n))/(1+t);
                    end
                end
            end
            plot(xplot1,uplot1,'k-',xed,ued,'ko',xplot2,uplot2,...
                'k-',xsplot,usplot,'k-')
            xlabel('x')
            ylabel('u')
            legend('computed','exact')
        end
    end

    erroru=0;
    drmax=0;
    for n=2:Nlast(1)                                                        % region 1 error
        nn=n-1;
        if  INIT == 1 || INIT == 4
            uexact=gauss_init(x(1,n),t,u(1,n),sig,n);
        elseif INIT == 2
            uexact=0;
            if (x(1,n) > -1)
                uexact=(1+x(1,n))/(1+t);
            end
        elseif INIT == 3
            uexact=0;
        end
        drs(nn)=u(1,n)-uexact;
        erroru=erroru+abs(drs(nn));
    end

    [drmax,nmax]=max(abs(drs));
    erroru=erroru/N(1);
    disp(' ')
    disp(['In region 1 the average error in u is ' num2str(erroru) ])
    if drmax >0
        disp(['The max error in u is ' num2str(drmax)...
            ' at n= ' num2str(nmax+1)])
    end
    for n=2:N(1)
        drs(n)=.5*drs(n)/(drmax+10^-12);
    end
    plot(xplot1,drs,'b--')

    erroru=0;
    drmax=0;
    nn=0;
    for n=2:Nlast(2)                                                        % region 2 error
        nn=n-1;
        if  INIT == 1 || INIT == 4
            uexact=gauss_init(x(2,n),t,u(2,n),sig,n);
        elseif INIT == 2
            uexact=0;
        elseif INIT == 3
            uexact=0;
            if (x(2,n) < 1)
                uexact=(-1+x(2,n))/(1+t);
            end
        end
        drs2(nn)=u(2,n)-uexact;
        erroru=erroru+abs(drs2(nn));
    end

    [drmax,nmax]=max(abs(drs2));
    erroru=erroru/N(2);
    disp(' ')
    disp(['In region 2 the average error in u is ' num2str(erroru)])
    if drmax > 0
        disp(['The max error in u is ' num2str(drmax)...
            ' at n= ' num2str(nmax+1)])
    end
    for n=1:N(2)
        drs2(n)=.5*drs2(n)/(drmax+10^-12);
    end
    plot(xplot2,drs2,'r--')
    hold off

    nfig=nfig+1;
    figure(nfig)                                                            %  computed & exact shock path
    if INIT == 2 || INIT == 3
        dt=(tlast-1)/10.;
        for n=1:11
            tsdata(n)=1.+(n-1)*dt;
            xsdata(n)=sig*(sqrt(2*(1+tsdata(n)))-1);
        end
        plot(xsout,tout,'ko',xsdata,tsdata,'k-')
        if INIT==2
            axis([1. 2.5 1. 5.1])
        else
            axis([-2.5 -1. 1. 5.1])
        end
        xlabel('x')
        ylabel('time')
        legend('computed','exact')
    end
    if INIT == 1 || INIT == 4
        tsdata=[1.1658 1.2 1.3 1.35 1.4 1.45 1.5 1.55 1.6 1.7421...
            1.8 2. 2.5 3. 3.5 4. 4.5 5.];
        xsdata=[1.4142 1.4349 1.4943 1.5234 1.5523 1.5809 1.6091...
            1.6371 1.6648 1.7421 1.7730 1.8776 2.1256 2.3576 2.5765...
            2.7843 2.9825 3.1722];
        plot(xsout,tout,'ko',sig*xsdata,tsdata,'k-')
        if INIT == 1
            axis([1. 3.5 1. 5.1])
        else
            axis([-3.5 -1. 1. 5.1])
        end
        xlabel('x')
        ylabel('time')
        legend('computed','exact')
    end
end