% SHOCKFIT04.M - Boundary Shock-Fitting
% A SHOCK FITTING PRIMER - CHAPTER 2, "Boundary Shock-Fitting".
% Numerical integration of inviscid Burgers equation with
% Gaussian initial conditions or "one-saw-tooth" initial conditions.
% Dependencies: This code makes function calls to:
%
%           function u=gauss_init(x,t,useed,sig,n)
%           function xs=exactgauss_xs(tstar)
%           function roe_runge_kutta
%           function mac_non_conservative
%           function plot_solution(INDEX)
%           function solve_for_ur
%
% These functions must reside in same directory as this code.
%
% Shock-fitted, using two regions.
% Code check: with INIT=2, tlast=5, CN=1, N(1)=100, N(2)=10, kappa=1/3
% xl=-1.5, xr=3., LIMITER=0 => After 93 time steps xs=2.47.
%
% m=1 region 1, x[xl,xs]; z(1)=(x-xl)/(xs(t)-xl)
% m=2 region 2, x[xs,xr]; z(2)=(x-xs(t))/(xr-xs(t))
% N(1)= number of mesh points in region 1 minus 2
% N(2)= number of mesh points in region 2 minus 2
%
% Copyright 2005-2009 M.D.Salas.

clear all

global kout LIMITER kappa eps0 z dz zx zt
global xLEFT kplot sig
global Nfirst Nlast N NT INIT CN nfig
global x u xs w usL usR t tlast LAST xl xr
                                                                            % PRE-ASSIGNED CASES:
                                                                            % CASE 1 Gaussian               Roe conservative
                                                                            % CASE 2 one-saw-tooth          Roe conservative
                                                                            % CASE 3 neg. one-saw-tooth     Roe conservative
                                                                            % CASE 4 neg. Gaussian          Roe conservative
                                                                            % CASE 5 Gaussian               MacCormack non-conservative
                                                                            % CASE 6 one-saw-tooth          MacCormack non-conservative
                                                                            % CASE 7 neg one-saw-tooth      MacCormack non-conservative
                                                                            % CASE 8 neg. Gaussian          MacCormack non-conservative
                                                                            %
                                                                            %               USER PROVIDED INPUTS

                                                                            % INIT=2;                     INIT=1 Gaussian; INIT=2 "one-saw-tooth";
                                                                            %                            INIT=3 -"one-saw-tooth"; INIT=4 -Gaussian
                                                                            % tlast=2;		             runs to a max time of t=tlast
                                                                            % CN=1;			             CN=Courant Number
                                                                            % N(1)=300;		             N(1)=number of mesh points -2 in region 1
                                                                            % N(2)=300;                   N(2)=number of mesh points -2 in region 2
                                                                            % xls=[-3. -1.5 -3. -4.];     left boundary:  xls(INIT)
                                                                            % xrs=[4. 3. 1.5 3.];         right boundary: xrs(INIT)
                                                                            % kplot=10;                   plot shock location every kplot steps
                                                                            % kappa=1/3;	                 kappa=Kappa interpolation parameter, for METHOD=0 only
                                                                            % LIMITER=0;                  LIMITER on/off; 1 -> on, 0 -> off
                                                                            % METHOD=0;                   METHOD=0 -> Roe/RK conservative,
                                                                            %                             METHOD=1 -> MacCormack non-conservative
disp('A SHOCK-FITTING PRIMER')
disp(' ')
disp('ICASE=1 Gaussian Roe conservative')
disp('ICASE=2 one-saw-tooth Roe conservative')
disp('ICASE=3 neg. one-saw-tooth Roe conservative')
disp('ICASE=4 neg. Gaussian Roe conservative')
disp('ICASE=5 Gaussian MacCormack non-conservative')
disp('ICASE=6 one-saw-tooth MacCormack non-conservative')
disp('ICASE=7 neg. one-saw-tooth MacCormack non-conservative')
disp('ICASE=8 neg.Gaussian MacCormack non-conservative')
disp(' ')
disp('In the next line enter the case to be executed with default values')
ICASE=input('Enter ICASE (1 to 8) ');

if ICASE == 1
    INIT=1;
    tlast=5;
    CN=1;
    N(1)=300;
    N(2)=300;
    xls=[-3. -1.5 -3. -4.];
    xrs=[4. 3. 1.5 3.];
    kplot=10;
    kappa=1/3;
    LIMITER=1;
    METHOD=0;
elseif ICASE == 2
    INIT=2;
    tlast=3;
    CN=1;
    N(1)=300;
    N(2)=300;
    xls=[-3. -1.5 -3. -4.];
    xrs=[4. 3. 1.5 3.];
    kplot=10;
    kappa=1/3;
    LIMITER=1;
    METHOD=0;
elseif ICASE == 3
    INIT=3;
    tlast=3;
    CN=1;
    N(1)=300;
    N(2)=300;
    xls=[-3. -1.5 -3. -4.];
    xrs=[4. 3. 1.5 3.];
    kplot=10;
    kappa=1/3;
    LIMITER=1;
    METHOD=0;
elseif ICASE == 4
    INIT=4;
    tlast=5;
    CN=1;
    N(1)=300;
    N(2)=300;
    xls=[-3. -1.5 -3. -4.];
    xrs=[4. 3. 1.5 3.];
    kplot=10;
    kappa=1/3;
    LIMITER=1;
    METHOD=0;
elseif ICASE == 5
    INIT=1;
    tlast=5;
    CN=1;
    N(1)=300;
    N(2)=300;
    xls=[-3. -1.5 -3. -4.];
    xrs=[4. 3. 1.5 3.];
    kplot=10;
    kappa=1/3;
    LIMITER=1;
    METHOD=1;
elseif ICASE == 6
    INIT=2;
    tlast=3;
    CN=1;
    N(1)=300;
    N(2)=300;
    xls=[-3. -1.5 -3. -4.];
    xrs=[4. 3. 1.5 3.];
    kplot=10;
    kappa=1/3;
    LIMITER=1;
    METHOD=1;
elseif ICASE == 7
    INIT=3;
    tlast=3;
    CN=1;
    N(1)=300;
    N(2)=300;
    xls=[-3. -1.5 -3. -4.];
    xrs=[4. 3. 1.5 3.];
    kplot=10;
    kappa=1/3;
    LIMITER=1;
    METHOD=1;
elseif ICASE == 8
    INIT=4;
    tlast=5;
    CN=1;
    N(1)=300;
    N(2)=300;
    xls=[-3. -1.5 -3. -4.];
    xrs=[4. 3. 1.5 3.];
    kplot=10;
    kappa=1/3;
    LIMITER=1;
    METHOD=1;
else
    disp('ICASE should be in the range 1 to 8; check input')
end
                                                                            %               NO CHANGES BELOW THIS LINE
                                                                            %               HOUSEKEEPING & INITIAL CONDITIONS
xss=[1.4349 1. -1. -1.4349];                                                % initial shock position: xss(INIT)
ts=[1.2 1. 1. 1.2];                                                         % initial time: ts(INIT)
usLs=[.7752 1. 0. -.4274];                                                  % initial u at shock (left side): usLs(INIT)
usRs=[.4274 0. -1. -.7752];                                                 % initial u at shock (right side): usRs(INIT)
xl=xls(INIT);
xr=xrs(INIT);
xs=xss(INIT);
t=ts(INIT);
usL=usLs(INIT);
usR=usRs(INIT);
w=.5*(usL+usR);                                                             % shock speed
sigs=[1 1 -1 -1];                                                           % sign of u depending on initial conditions
sig=sigs(INIT);
usmall=sig*0.0001;
useeds=[usmall usR];
xLEFT(1)=xl;                                                                % left boundary region 1
xLEFT(2)=xs;                                                                % left boundary region 2
zx(1)=1./(xs-xl);                                                           % region 1 Jacobian
zx(2)=1./(xr-xs);                                                           % region 2 Jacobian
for m=1:2
    Nlast(m)=N(m)+1;                                                        % various mesh indexes
    Nfirst(m)=3;
    NT(m)=N(m)+2;
    dz(m)=1./N(m);                                                          % mesh interval size in region m
end
eps0=10^-7;
nfig=0;                                                                     % figure counter

for m=1:2
    useed=useeds(m);
    for n=m:NT(m)-(2-m)                                                     % define initial conditions, region m
        z(m,n)=(n-m)*dz(m);                                                 % computational coordinate [0,1]
        zt(m,n)=(m-2/m)*(z(m,n)+(1-m))*zx(m)*w;
        x(m,n)=xLEFT(m)+z(m,n)/zx(m);                                       % physical coordinate [xs,xr]
        if INIT == 1 || INIT == 4                                           % Gaussian initial conditions
            u(m,n)=gauss_init(x(m,n),t,useed,sig,n);
            useed=u(m,n);
        elseif INIT == 2  || INIT == 3                                      % "one-saw-tooth" initial conditions
            u(m,n)=0.;
            if (x(m,n) > -1) && (x(m,n) <= 1)
                u(m,n)=(x(m,n)+sig)/(1.+t);
            end
        end
    end
end

u(1,Nlast(1))=usL;                                                          % shock point region 1
u(2,2)=usR;                                                                 % shock point region 2
u(1,NT(1))=2*u(1,Nlast(1))-u(1,N(1));                                       % shock ghost point, region 1
u(2,1)=2*u(2,2)-u(2,3);                                                     % shock ghost point, region 2
                                                                            %               OUTPUT
plot_solution(1)                                                            % plot initial u-profile
                                                                            %               MAIN INTEGRATION LOOP
LAST=0;
kout=0;
if METHOD == 0
    roe_runge_kutta
elseif METHOD == 1
    mac_non_conservative
end
                                                                            %               OUTPUT
disp('Output from SHOCKFIT04:')
disp(' ')
disp(['In region 1 N= ' num2str(N(1)) ', in region 2 N= ' num2str(N(2))])
disp(['LIMITER= ' num2str(LIMITER)...
    ', and inital conditions set for INIT= ' num2str(INIT)])
if INIT == 1
    disp('Corresponding to Positive Gaussian ')
elseif INIT==2
    disp('Corresponding to Positive One-Saw-Tooth ')
elseif INIT==3
    disp('Corresponding to Negative One-Saw-Tooth ')
elseif INIT==4
    disp('Corresponding to Negative Gaussian ')
end
if METHOD == 0
    disp('Roe_Runge_Kutta Method')
elseif METHOD == 1
    disp('MacCormack_non-conservative Method')
end
disp(' ')
disp(['After ' num2str(kout) ' time steps:'])
disp(['time= ' num2str(t) ' and xs= ' num2str(xs)])
plot_solution(2)                                                            % plot final results & output error estimates