function initial7
% INITIAL7.M - Initial conditions
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting" 
%
% INIT=1 Gaussian;
% INIT=2 "one-saw-tooth";
% INIT=3 negative "one-saw-tooth";
% INIT=4 negative Gaussian;
% INIT=5 "two-saw-tooth";
% INIT=6 staircase;
% INIT=7 Smooth Gaussian @ t=0;
% INIT=8 cos(pi*x) @ t=0;
%
% Copyright 2005-2008 M.D.Salas.

global eps0 dx sig I IS N NT INIT nfig kmax
global x u U xs w usL usR t tlast xl xr stop

xls=[-3. -1.5 -2.8 -4. -1.5 0 -3 -.5];                                      % left boundary:  xls(INIT)
xrs=[ 4.  2.8  1.5  3.  4.5 10 4 1.5];                                      % right boundary: xrs(INIT)
xss=[1.4349 -1. -1. -1.4349 -1. .2 0 0];                                    % initial shock position: xss(INIT)
ts=[1.2 1. 1. 1.2 1. 0 0 0];                                                % initial time: ts(INIT)
if tlast > ts(INIT)
    usLs=[.7752 0.  0. -.4274 0. 1. 0 0];                                   % initial u at shock (left side): usLs(INIT)
    usRs=[.4274 0. -1. -.7752 0. .8 0 0];                                   % initial u at shock (right side): usRs(INIT)
    xl=xls(INIT);
    xr=xrs(INIT);
    t=ts(INIT);
    sigs=[1 1 -1 -1 1 1 1 1];                                               % sign of u depending on initial conditions
    sig=sigs(INIT);
    xs(1)=xss(INIT);
    usL(1)=usLs(INIT);
    usR(1)=usRs(INIT);
    w(1)=.5*(usL(1)+usR(1));
    usmall=sig*0.0001;
    useeds=[usmall usR(1)];
    kmaxs=[1 2 2 1 4 12 0 0];                                               % number of discontinuities @ initial time

    kmax=kmaxs(INIT);
    NT=N+1;
    eps0=10^-10;                                                            % small number
    nfig=0;                                                                 % figure counter
    dx=(xr-xl)/N;                                                           % mesh interval size

    for n=1:NT
        x(n)=xl+dx*(n-1);                                                   % physical coordinate [xs,xr]
    end

    if INIT == 2
        xs(2)=1.;
        usL(2)=1.;
        usR(2)=0.;
        w(2)=.5;

    elseif INIT == 3
        xs(2)=1;
        usL(2)=0;
        usR(2)=0;
        w(2)=0;

    elseif INIT == 5
        xs(2)=1;
        usL(2)=1;
        usR(2)=0;
        w(2)=.5;

        xs(3)=2;
        usL(3)=0;
        usR(3)=-1;
        w(3)=-.5;

        xs(4)=4;
        usL(4)=0;
        usR(4)=0;
        w(4)=0;

    elseif INIT == 6
        for k=2:kmax
            xs(k)=xs(k-1)+(k+1)/10;
            usL(k)=usL(k-1)-.2;
            usR(k)=usR(k-1)-.2;
            w(k)=.5*(usL(k)+usR(k));
        end
    end
                                                                            %          SHOCK INDEXES
    for n=1:NT
        I(n,1)=0;
        I(n,2)=0;
    end
    if kmax > 0
        shock_index                                                         %  build shock indexes
    end
    
    useed=useeds(1);
    for n=1:NT
        if INIT == 1 || INIT == 4                                           % Gaussian initial conditions
            if (IS(1)+1) == n
                useed=useeds(2);
            end
            u(n)=gauss_init(x(n),t,useed,sig,n);
            useed=u(n);
        elseif INIT == 2  || INIT == 3                                      % "one-saw-tooth" initial conditions
            u(n)=0.;
            if (x(n) > -1) && (x(n) <= 1)
                u(n)=(x(n)+sig)/(1.+t);
            end
        elseif INIT == 5                                                    % "two-saw-tooth" initial conditions
            u(n)=0.;
            if (x(n) > -1) && (x(n) <= 1)
                u(n)=(x(n)+1)/(1.+t);
            elseif (x(n) > 2) && (x(n) <= 4)
                u(n)=(x(n)-4)/(1.+t);
            end
        elseif INIT == 6                                                    % staircase initial conditions
            if (x(n) >= xl) && (x(n) < xs(1))
                u(n)=usL(1);
            end
            for k=1:kmax-1
                if (x(n) > xs(k)) && (x(n) < xs(k+1))
                    u(n)=usL(k+1);
                end
            end
            if x(n) > xs(kmax)
                u(n)=usR(kmax);
            end
        elseif INIT == 7                                                    % smooth Gaussian initial conditions
            u(n)=gauss_init(x(n),t,useed,sig,n);
            useed=u(n);
        elseif INIT == 8                                                    % Cosine(x^2) initial conditions
            xa=1;
            if x(n) <= 0
                u(n)=1;
            end
            u(n)=1;
            if x(n) >= xa
                u(n)=-1;
            end
            if x(n)*(x(n)-xa) < 0
                u(n)=cos(pi*x(n));
            end
        end
        U(n)=u(n);
    end
else
    disp('   ')
    disp('"tlast" entered is less than starting time for case selected.')
    disp('Execution is stopped. Please check input parameters.')
    disp('   ')
    stop=1;
end