function inject_shock(ns)
% INJECT_SHOCK.M - shock injection function
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting". 
%
% Copyright 2005-2008 M.D.Salas.

global kout IS kmax x u xs w usL usR t

xsn=.5*(x(ns)+x(ns+1));
usLn=u(ns);
usRn=u(ns+1);
wn=.5*(usLn+usRn);

disp(['Shock detected at t= ' num2str(t) ' step = ' num2str(kout)])
disp(['xs= ' num2str(xsn) ' w= ' num2str(wn)])

if kmax == 0                                                                % no other shocks present
    kmax=1;
    xs(1)=xsn;
    w(1)=wn;
    usL(1)=usLn;
    usR(1)=usRn;
else                                                                        % other shocks presents
    nkmax=IS(kmax);
    n1=IS(1);
    if nkmax < ns ;                                                         % last shock to left of new shock
        kmax=kmax+1;
        xs(kmax)=xsn;
        w(kmax)=wn;
        usL(kmax)=usLn;
        usR(kmax)=usRn;
    elseif n1 > ns                                                          % first shock to right of new shock
        kmax=kmax+1;
        for k=2:kmax
            xs(k)=xs(k-1);
            w(k)=w(k-1);
            usL(k)=usL(k-1);
            usR(k)=usR(k-1);
        end
        xs(1)=xsn;
        w(1)=wn;
        usL(1)=usLn;
        usR(1)=usRn;
    else                                                                    % new shock in between other shocks
        for k=1:kmax                                                        % find shock index to left of new shock
            n=IS(k);
            if n < ns
                kleft=k;
            end
        end
        for k=kleft+1:kmax
            xs(k+1)=xs(k);
            w(k+1)=w(k);
            usL(k+1)=usL(k);
            usR(k+1)=usR(k);
        end
        kmax=kmax+1;
        xs(kleft+1)=xsn;
        w(kleft+1)=wn;
        usL(kleft+1)=usLn;
        usR(kleft+1)=usRn;
    end
end

