function mac_non_conservative7
% MAC_NON_CONSERVATIVE7.M - solver based on MacCormack's predictor
% -corrector method
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting". 
%
% Copyright 2005-2008 M.D.Salas.

global kout eps0 dx dt Uo dts FLAG IDETECT kplot tout xsout
global sig xso usLo usRo I IS N NT INIT CN kmax ICROSS kcross
global x u U xs w usL usR t tlast LAST xl xr

ICROSS(1)=1/eps0;                                                           % set to a large number, used in function find_shk_xing
FLAG=0;                                                                     % trigger for shock crossing, set to 1 in function find_shk_xing
SAFETY=1000;                                                                % stops calculation if over run condition

while LAST == 0                                                             % begin time stepping loop

    kout=kout+1;                                                            % step counter
    
    if kout > SAFETY                                                        % stop calculation if out-of-control condition
        LAST=1;
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value')
    end
    
    Amax=eps0;
    for n=1:NT                                                              % begin CFL condition
        Aabs=abs(u(n));
        if Aabs > Amax
            Amax=Aabs;
        end
    end

    dt=CN*dx/Amax;                                                          % end CFL condition

    if kmax >= 2                                                            % check for shock crossing, reset dt if etest < 1
        find_shk_xing
    end

    if (t+dt) >= tlast                                                      % check that time does not exceed tlast
        dt=tlast-t;
        LAST=1;                                                             % trigger to stop while loop
    end
    for n=1:NT                                                              % store time interval, needed if shock crosses mesh point
        dts(n)=dt;
    end

    Uo=u;                                                                   % save latest values
    if kmax > 0
        xso=xs;
        wo=w;
        usRo=usR;
        usLo=usL;
    end

    for j=1:2                                                               % predictor-corrector loop
        fac=j/2;
        np=2-j;
        nm=1-j;

        for n=2:N                                                           % sweep through mesh points
            sigma=dts(n)/dx;
            ux=u(n+np)-u(n+nm);
            if kmax > 0
                if j == 1 && I(n,1) ~= 0                                    % true if predictor stage & shock present in cell [n,n+1]
                    if I(n-1,1) == 0                                        % true if no shock in cell [n-1,n]
                        k=I(n,1);                                           % use shock index to right of n
                        eps1=(xs(k)-x(n))/dx;
                        ux=((eps1-2)*u(n-1)+usL(k)+(1-eps1)*u(n))/2;
                    else                                                    % shock in cells [n-1,n] & [n,n+1]
                        k=I(n,1);
                        ux=(usL(k)-usR(k-1))*dx/(xs(k)-xs(k-1));
                    end
                elseif j == 2 && I(n-1,1) ~= 0                              % true if corrector stage & shock present in cell [n-1,n]
                    if I(n,1) == 0                                          % true if no shock in cell [n,n+1]
                        k=I(n-1,2);                                         % use shock index to left of n
                        eps1=(x(n)-xs(k))/dx;
                        ux=-((eps1-2)*u(n+1)+usR(k)+(1-eps1)*u(n))/2;
                    else                                                    % shock in cells [n-1,n] & [n,n+1]
                        k=I(n,1);
                        ux=(usL(k)-usR(k-1))*dx/(xs(k)-xs(k-1));
                    end
                end
            end
            
            ut=-u(n)*ux;
            
            if j == 1
                U(n)=u(n)+sigma*ut;
            else
                U(n)=.5*(Uo(n)+u(n)+sigma*ut);
            end
        end

        if kmax > 0
            for k=1:kmax                                                    % begin shock point computation
                n=IS(k);                                                    % set mesh point to left of k-th shock
                                                                            % compute left side
                if I(n,1) == k                                              % true if k-shock is first shock in cell [n,n+1]
                    if I(n-1,1) == 0 && I(n-2,1) == 0                       % true if no shock between cells [n-2,n]
                        epsL=(xs(k)-x(n))/dx;
                        c1=2/(1+epsL);
                        uxL=(c1*(usL(k)-u(n-1))+.5*(u(n-2)-u(n)))/dx;
                    elseif I(n-1,1) == 0 && I(n-2,1) ~= 0                   % true if shock in cell [n-2,n-1]
                        epsL=(xs(k)-x(n))/dx;
                        c1=(2*epsL-1)/(1+epsL);
                        c2=3/(1+epsL);
                        uxL=(c1*u(n-1)+c2*usL(k)-2*u(n))/dx;
                    elseif I(n-1,1) ~=  0                                   % true if shock in cell [n-1,n]
                        epsL=xs(k)-xs(k-1);
                        if epsL >= dx
                            uxL=(usL(k)-usR(k-1))/epsL;
                        else
                            uxL=0;
                        end
                    end
                else                                                        % true if k-shock not first in cell [n,n+1]
                    uxL=0;                                                  % do not resolve cell microstructure
                end
                                                                            % compute right side
                if I(n,2) == k                                              % true is k-shock is last shock in cell [n,n+1]
                    if I(n+1,1) == 0 && I(n+2,1) == 0                       % true if no shock between cells [n+1,n+3]
                        epsR=(x(n+1)-xs(k))/dx;
                        c1=2/(1+epsR);
                        uxR=-(c1*(usR(k)-u(n+2))+.5*(u(n+3)-u(n+1)))/dx;
                    elseif I(n+1,1) == 0 && I(n+2,1) ~= 0                   % true if shock in cell [n+2,n+3]
                        epsR=(x(n+1)-xs(k))/dx;
                        c1=(2*epsR-1)/(1+epsR);
                        c2=3/(1+epsR);
                        uxR=-(c1*u(n+2)+c2*usR(k)-2*u(n+1))/dx;
                    elseif I(n+1,1) ~= 0                                    % true if shock in cell [n+1,n+2]
                        epsR=xs(k+1)-xs(k);
                        if epsR >= dx
                            uxR=(usL(k+1)-usR(k))/epsR;
                        else
                            uxR=0;
                        end
                    end
                else                                                        % true if k-shock not last shock in cell [n,n+1]
                    uxR=0;                                                  % do not resolve cell microstructure
                end
                
                RuL=-(usL(k)-w(k))*uxL;
                RuR=-(usR(k)-w(k))*uxR;
                R=.5*(RuL+RuR);
                if j == 1
                    Xs(k)=xso(k)+w(k)*dt;                                   % predictor for shock location
                    W(k)=wo(k)+R*dt;                                        % predictor for shock speed
                    if abs(RuL) < eps0
                        UsL(k)=usLo(k)+RuL*dt;
                        UsR(k)=2.*W(k)-UsL(k);
                    else
                        UsR(k)=usRo(k)+RuR*dt;
                        UsL(k)=2.*W(k)-UsR(k);
                    end
                else
                    Xs(k)=.5*(xso(k)+xs(k)+w(k)*dt);                        %  corrector for shock location
                    W(k)=.5*(wo(k)+w(k)+R*dt);                              %  corrector for shock speed
                    if abs(RuL) < eps0
                        UsL(k)=.5*(usLo(k)+usL(k)+RuL*dt);
                        UsR(k)=2.*W(k)-UsL(k);
                    else
                        UsR(k)=.5*(usRo(k)+usR(k)+RuR*dt);
                        UsL(k)=2.*W(k)-UsR(k);
                    end
                end
                if FLAG == 1                                                % if shock crossing taking place
                    xs(kcross)=xs(kcross-1);
                end
            end                                                             % end shock point computation
        end

        if INIT == 1                                                        % update boundary values for Gaussian initial conditions
            U(1)=gauss_init(xl,t+fac*dt,u(1),sig,1);                        % inflow, exact solution
            U(NT)=2*U(N)-U(N-1);                                            % extrapolate outflow
        elseif INIT == 4                                                    % update boundary values for Gaussian initial conditions
            U(NT)=gauss_init(xr,t+fac*dt,u(NT),sig,NT);                     % inflow, exact solution
            U(1)=2*U(2)-U(3);                                               % extrapolate outflow
        end
        
        if kmax > 0
            xs=Xs;
            w=W;
            usL=UsL;
            usR=UsR;
            mesh_xing                                                       % update mesh point crossed by a shock
        end

        u=U;                                                                % update u
        if kmax > 0
            for k=1:kmax-1
                if (IS(k)+1) == IS(k+1)                                     % if single mesh point lies between two shocks
                    ns=IS(k+1);                                             % update with shock averaged values
                    u(ns)=(usR(k)+usL(k+1))/2;
                end
            end
        end
        
    end                                                                     % end predictor-corrector loop

    if FLAG == 1                                                            % true if shock crossing occurs
        shock_cross
        FLAG=0;
    end

    t=t+dt;                                                                 % increment time

    if IDETECT == 1                                                         % shock detection
        detect_shock
    end

    test=rem(kout,kplot);                                                   % save values for shock locations for plotting every kplot steps
    if test==0 && kmax > 0
        for k=1:kmax
            tout(k,kout)=t;
            xsout(k,kout)=xs(k);
        end
    end

end                                                                         % end time stepping "while" loop
