function mesh_xing
% MESH_XING.M - performs update for mesh point crossed by a shock wave
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting".
%
% Copyright 2005-2008 M.D.Salas.

global dx dt ISO Uo dts xso usLo usRo I IS kmax x U xs usL usR

ISO=IS;                                                                     % save shock IS index, to detect shock crossing mesh point

shock_index                                                                 % re-build shock indexes

for k=1:kmax
    if ISO(k) < IS(k)                                                       % if shock index changed (+), recompute point crossed by right running shock
        n=IS(k);
        if I(n-1,1) == 0                                                    % no shocks in [n-1,n]
            kk=I(n,1);
            delta=(x(n)-xso(kk))/(xs(kk)-xso(kk));
            ua=Uo(n-1)+delta*(U(n-1)-Uo(n-1));
            ub=usLo(kk)+delta*(usL(kk)-usLo(kk));
            U(n)=ub-ub*(ub-ua)*(1-delta)*dt/dx;
            Uo(n)=ub;
            dts(n)=(1-delta)*dt;                                            % save time increment for use in corrector step
        else
            delta=(x(n)-xso(k))/(xs(k)-xso(k));
            U(n)=usL(k)*(x(n)-xs(k-1))/(xs(k)-xs(k-1))+...
                usR(k-1)*(x(n)-xs(k))/(xs(k-1)-xs(k));
            Uo(n)=usLo(k)+delta*(usL(k)-usLo(k));
            dts(n)=(1-delta)*dt;
        end
    elseif ISO(k) > IS(k)                                                   % if shock index changed (-), recompute point crossed by left running shock
        n=IS(k)+1;
        if I(n,1) == 0                                                      % no shocks in [n,n+1]
            kk=I(n-1,2);
            delta=(x(n)-xso(kk))/(xs(kk)-xso(kk));
            ua=Uo(n+1)+delta*(U(n+1)-Uo(n+1));
            ub=usRo(kk)+delta*(usR(kk)-usRo(kk));
            U(n)=ub-ub*(ua-ub)*(1-delta)*dt/dx;
            Uo(n)=ub;
            dts(n)=(1-delta)*dt;                                            % save time increment for use in corrector step
        else
            delta=(x(n)-xso(k))/(xs(k)-xso(k));
            U(n)=usL(k+1)*(x(n)-xs(k))/(xs(k+1)-xs(k))+...
                usR(k)*(x(n)-xs(k+1))/(xs(k)-xs(k+1));
            Uo(n)=usRo(k)+delta*(usR(k)-usRo(k));
            dts(n)=(1-delta)*dt;
        end
    end
end