% SHOCKFIT07.M - Floating shock-fitting main routine
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting".
%
% Dependencies. This code makes function calls to:
%
%           function u=gauss_init(x,t,useed,sig,n)
%           function xs=exactgauss_xs(tstar)
%           function mac_non_conservative7
%           function plot_solution7(INDEX)
%           function shock_index
%           function shock_cross
%           function initial7(INIT)
%           function find_shk_xing
%           function detect_shock
%           function inject_shock(ns)
%           function mesh_xing
%           function solve_for_ur
%
% These functions must reside in same directory as this code.
%
% Copyright 2005-2009 M.D.Salas.

clear all

global kout  IDETECT kplot tout xsout N INIT CN kmax xs t tlast LAST  stop
                                                                            %               USER PROVIDED INPUTS
                                                                            % INIT=1 Gaussian (requires tlast > 1.2);
                                                                            % INIT=2 "one-saw-tooth" (requires tlast > 1.);
                                                                            % INIT=3 negative "one-saw-tooth" (requires tlast > 1.);
                                                                            % INIT=4 negative Gaussian (requires tlast > 1.2);
                                                                            % INIT=5 "two-saw tooth" (requires tlast > 1.);
                                                                            % INIT=6 staircase, default to: N=112, tlast=8;
                                                                            % INIT=7 smooth Gaussian, start @ t=0, shock detection
                                                                            % INIT=8 cos(pi*x) @ t=0: N=400, tlast=.5;
                                                                            % tlast runs to a max time of t=tlast, see tlast requirements above
                                                                            % CN=Courant Number
                                                                            % N=number of mesh points -1
                                                                            % kplot=1 plot shock location every kplot steps
                                                                            % IDETECT=1 Flag to detect shocks; IDETECT=0 -> do not detect shocks 
disp('A SHOCK-FITTING PRIMER')
disp(' ')
disp('INIT=1 Gaussian');
disp('INIT=2 "one-saw-tooth"');
disp('INIT=3 negative "one-saw-tooth"');
disp('INIT=4 negative Gaussian');
disp('INIT=5 "two-saw tooth"');
disp('INIT=6 staircase');
disp('INIT=7 smooth Gaussian');
disp('INIT=8 cos(pi*x) @ t=0');
disp(' ');

disp('In the next line enter the case to be executed with default values')
INIT=input('Enter INIT (1 to 8) ');

if INIT == 1;
    tlast=4;
    CN=1;
    N=100;
    kplot=1;
    IDETECT=1; 
elseif INIT == 2;
    tlast=4;
    CN=1;
    N=100;
    kplot=1;
    IDETECT=1;
elseif INIT == 3;
    tlast=2;
    CN=1;
    N=100;
    kplot=1;
    IDETECT=1;
elseif INIT == 4;
    tlast=4;
    CN=1;
    N=100;
    kplot=1;
    IDETECT=1;
elseif INIT == 5;
    tlast=2.5;
    CN=1;
    N=100;
    kplot=1;
    IDETECT=1;
elseif INIT == 6;
    tlast=8;
    CN=1;
    N=112;
    kplot=1;
    IDETECT=1;
elseif INIT == 7;
    tlast=3;
    CN=1;
    N=100;
    kplot=1;
    IDETECT=1;
elseif INIT == 8;
    tlast=.5;
    CN=1;
    N=400;
    kplot=1;
    IDETECT=1;
else
    disp('INIT not in range, execution aborted')
    tlast=0;
    INIT=1;
end
                                                                            % k=1,2,...kmax, is the shock counter; if only one shock k=kmax=1
                                                                            % IS(k)=n, where n is the mesh point to the left of the k-th shock
                                                                            % I(n,1)=0, if no shock in cell [n,n+1]
                                                                            % I(n,1)=k, where k is the index of the first shock in cell [n,n+1]:
                                                                            %
                                                                            %                       n              n+1
                                                                            %                       o---\---\---\---o
                                                                            %                          k   k+1  k+2
                                                                            %
                                                                            % I(n,2)=k, where k is the index of last shock in cell [n,n+1]:
                                                                            %
                                                                            %                       n              n+1
                                                                            %                       o---\---\---\---o
                                                                            %                          k-2  k-1  k
                                                                            %
                                                                            %
                                                                            %               NO CHANGES BELOW THIS LINE
                                                                            %
                                                                            %               INITIAL CONDITIONS
stop=0;
initial7                                                                    % define initial conditions
if stop == 1
    return                                                                  % execution stopped if stop==1
else
                                                                            %               OUTPUT
    plot_solution7(1)                                                       % plot initial u-profile
                                                                            %               MAIN INTEGRATION LOOP
    LAST=0;
    kout=1;

    if kmax ~= 0
        for k=1:kmax
            tout(k,kout)=t;
            xsout(k,kout)=xs(k);
        end
    end
    
    mac_non_conservative7
    
                                                                            %               OUTPUT
    disp('Output from SHOCKFIT:')
    disp(' ')
    disp(['N= ' num2str(N)])
    disp(['Inital conditions set for INIT= ' num2str(INIT)])
    if INIT == 1
        disp('Corresponding to Positive Gaussian ')
    elseif INIT == 2
        disp('Corresponding to Positive One-Saw-Tooth ')
    elseif INIT == 3
        disp('Corresponding to Negative One-Saw-Tooth ')
    elseif INIT == 4
        disp('Corresponding to Negative Gaussian ')
    elseif INIT == 5
        disp('Corresponding to Two-Saw-Tooth ')
    elseif INIT == 6
        disp('Corresponding to Staircase Problem ')
    end
    disp(' ')
    disp(['After ' num2str(kout) ' time steps:'])
    disp(['time= ' num2str(t)])
    if kmax > 0
        for k=1:kmax
            disp(['xs(' num2str(k) ')= ' num2str(xs(k))])
        end
    end
    plot_solution7(2)                                                       % plot final results
end
