function initial12_s
% INITIAL12_S.M - Initial conditions s-shock
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting"
%
% Solves a non-conservative system of 2 PDE's
% dr/dt+(u-1)dr/dx=0, ds/dt+(u+1)ds/dt=0, r=u+g, s=u-g.
%
% INIT=1 "one-saw-tooth" for s-shock type
% The exact shock path is:
% x_s(t)=(s0/2+1)(3+2t)-sqrt((1+t)/2)*(3+2s0); x_s(1)=2+s0/2
%
% Copyright 2005-2009 M.D.Salas.

global eps0 dx I N NT nfig kmax kmax_save xl xr tlast t IDETECT
global x u g r s R S x_s w uL uR gL gR rL rR sL sR shock_type

tlast=3;

xl=-1.5;                                                                    
xr=6;
t=0;
r0=.2;
N=200;
IDETECT=0;

x_s(1)=-1;
rL(1)=r0;
rR(1)=r0;
sL(1)=0;
sR(1)=0;
uL(1)=(rL(1)+sL(1))/2;
uR(1)=(rR(1)+sR(1))/2;
gL(1)=(rL(1)-sL(1))/2;
gR(1)=(rR(1)-sR(1))/2;
w(1)=(sL(1)+sR(1)+rL(1)+rR(1))/4+1;
shock_type(1)='s';

x_s(2)=0;
rL(2)=r0;
rR(2)=r0;
sL(2)=2;
sR(2)=2;
uL(2)=(rL(2)+sL(2))/2;
uR(2)=(rR(2)+sR(2))/2;
gL(2)=(rL(2)-sL(2))/2;
gR(2)=(rR(2)-sR(2))/2;
w(2)=(sL(2)+sR(2)+rL(2)+rR(2))/4+1;
shock_type(2)='s';

x_s(3)=1;
rL(3)=r0;
rR(3)=r0;
sL(3)=0;
sR(3)=0;
uL(3)=(rL(3)+sL(3))/2;
uR(3)=(rR(3)+sR(3))/2;
gL(3)=(rL(3)-sL(3))/2;
gR(3)=(rR(3)-sR(3))/2;
w(3)=(sL(3)+sR(3)+rL(3)+rR(3))/4+1;
shock_type(3)='s';

kmax=3;                                                                     % start with 3 gradient discontinuities
kmax_save=kmax;                                                             % kmax_ssave is only used for plotting

NT=N+1;
eps0=10^-10;                                                                % small number
nfig=0;                                                                     % figure counter
dx=(xr-xl)/N;                                                               % mesh interval size

for n=1:NT
    x(n)=xl+dx*(n-1);                                                       % physical coordinate [xs,xr]
end
                                                                            %      SHOCK INDEXES
for n=1:NT
    I(n,1)=0;
    I(n,2)=0;
end
if kmax > 0
    shock_index                                                             %  build shock indexes
end

for n=1:NT
                                                                            % "one-saw-tooth" initial conditions
    s(n)=0;
    r(n)=r0;
    if (x(n) > -1) && (x(n) <= 0)
        s(n)=2*(1+x(n));
    end
    if (x(n) > 0) && (x(n) < 1)
        s(n)=2*(1-x(n));
    end
    S(n)=s(n);
    R(n)=r(n);
    u(n)=(r(n)+s(n))/2;
    g(n)=(r(n)-s(n))/2;
end