function plot_solution12(INDEX)
% PLOT_SOLUTION12.M creates plots
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting".
% Solves a non-conservative system of 2 PDE's
% dr/dt+(u+1)dr/dx=0, ds/dt+(u-1)ds/dt=0, r=u+g, s=u-g.
%
% Copyright 2005-2009 M.D.Salas.

global kout tout xout N nfig kmax x u  uL uR t kmax_save
global g r s gL gR rL rR sL sR x_s w shock_type 


if INDEX == 1

    nfig=nfig+1;
    figure(nfig)                                                            % initial r-profile

    nn=0;
    for n=1:N
        nn=nn+1;
        xplot(nn)=x(n);
        uplot(nn)=r(n);
        if kmax > 0
            for k=1:kmax
                if (x(n)-x_s(k))*(x(n+1)-x_s(k)) < 0
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=rL(k);
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=rR(k);
                end
            end
        end
    end

    plot(xplot,uplot,'k-')
    xmin=xplot(1);
    xmax=xplot(N);
    xlim([xmin xmax])
    xlabel('x at t=0')
    ylabel('r')
    f=['Initial Conditions, t=' num2str(t)];
    title(f)

    nfig=nfig+1;
    figure(nfig)                                                            % initial s-profile
    nn=0;
    for n=1:N
        nn=nn+1;
        xplot(nn)=x(n);
        uplot(nn)=s(n);
        if kmax > 0
            for k=1:kmax
                if (x(n)-x_s(k))*(x(n+1)-x_s(k)) < 0
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=sL(k);
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=sR(k);
                end
            end
        end
    end

    plot(xplot,uplot,'k-')
    xmin=xplot(1);
    xmax=xplot(N);
    xlim([xmin xmax])
    xlabel('x at t=0')
    ylabel('s')
    f=['Initial Conditions, t=' num2str(t)];
    title(f)

    nfig=nfig+1;
    figure(nfig)                                                            % initial u-profile
    nn=0;
    for n=1:N
        nn=nn+1;
        xplot(nn)=x(n);
        uplot(nn)=u(n);
        if kmax > 0
            for k=1:kmax
                if (x(n)-x_s(k))*(x(n+1)-x_s(k)) < 0
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=uL(k);
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=uR(k);
                end
            end
        end
    end
    plot(xplot,uplot,'k-')
    xmin=xplot(1);
    xmax=xplot(N);
    xlim([xmin xmax])
    xlabel('x at t=0')
    ylabel('u')
    f=['Initial Conditions, t=' num2str(t)];
    title(f)

    nfig=nfig+1;
    figure(nfig)                                                            % initial g-profile
    nn=0;
    for n=1:N
        nn=nn+1;
        xplot(nn)=x(n);
        uplot(nn)=g(n);
        if kmax > 0
            for k=1:kmax
                if (x(n)-x_s(k))*(x(n+1)-x_s(k)) < 0
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=gL(k);
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=gR(k);
                end
            end
        end
    end
    plot(xplot,-uplot,'k-')
    xmin=xplot(1);
    xmax=xplot(N);
    xlim([xmin xmax])
    xlabel('x at t=0')
    ylabel('-g')
    f=['Initial Conditions, t=' num2str(t)];
    title(f)

else                                                                        % INDEX == 2
    nn=0;
    for n=1:N
        nn=nn+1;
        xplot(nn)=x(n);
        uplot(nn)=r(n);
        if kmax > 0
            for k=1:kmax
                if (x(n)-x_s(k))*(x(n+1)-x_s(k)) < 0
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=rL(k);
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=rR(k);
                end
            end
        end
    end

    nfig=nfig+1;
    figure(nfig)                                                            % r-profile at tlast
    plot(xplot,uplot,'k-')
    xmin=xplot(1);
    xmax=xplot(N);
    xlim([xmin xmax])
    xlabel('x')
    ylabel('r')
    f=['r at t=' num2str(t)];
    title(f)

    nn=0;
    for n=1:N
        nn=nn+1;
        xplot(nn)=x(n);
        uplot(nn)=s(n);
        if kmax > 0
            for k=1:kmax
                if (x(n)-x_s(k))*(x(n+1)-x_s(k)) < 0
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=sL(k);
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=sR(k);
                end
            end
        end
    end

    nfig=nfig+1;
    figure(nfig)                                                            % s-profile at tlast
    plot(xplot,uplot,'k-')
    xmin=xplot(1);
    xmax=xplot(N);
    xlim([xmin xmax])
    xlabel('x')
    ylabel('s')
    f=['s at t=' num2str(t)];
    title(f)

    nn=0;
    for n=1:N
        nn=nn+1;
        xplot(nn)=x(n);
        uplot(nn)=u(n);
        if kmax > 0
            for k=1:kmax
                if (x(n)-x_s(k))*(x(n+1)-x_s(k)) < 0
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=uL(k);
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=uR(k);
                end
            end
        end
    end

    nfig=nfig+1;
    figure(nfig)                                                            % u-profile at tlast
    plot(xplot,uplot,'k-')
    xmin=xplot(1);
    xmax=xplot(N);
    xlim([xmin xmax])
    xlabel('x')
    ylabel('u')
    f=['u at t=' num2str(t)];
    title(f)

    nn=0;
    for n=1:N
        nn=nn+1;
        xplot(nn)=x(n);
        uplot(nn)=g(n);
        if kmax > 0
            for k=1:kmax
                if (x(n)-x_s(k))*(x(n+1)-x_s(k)) < 0
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=gL(k);
                    nn=nn+1;
                    xplot(nn)=x_s(k);
                    uplot(nn)=gR(k);
                end
            end
        end
    end

    nfig=nfig+1;
    figure(nfig)                                                            % g-profile at tlast
    plot(xplot,-uplot,'k-')
    xmin=xplot(1);
    xmax=xplot(N);
    xlim([xmin xmax])
    xlabel('x')
    ylabel('-g')
    f=['-g at t=' num2str(t)];
    title(f)

    if kmax > 0
        nfig=nfig+1;
        figure(nfig)                                                        % shock path
        for k=1:kmax_save
            for n=1:kout
                xsp=xout(k,n);
                tsp=tout(k,n);
                plot(xsp,tsp,'k.')
                hold on
            end
        end
        hold off
        xmin=xplot(1);
        xmax=xplot(N);
        xlim([xmin xmax])
        xlabel('x')
        ylabel('t')
        f='Shock Wave Path';
        title(f)
    end
    
    if kmax > 0
        for k=1:kmax
            disp(' ')
            disp(['for ' shock_type(k) '-shock x_s(' num2str(k) ')= '...
                num2str(x_s(k))])
            if shock_type(k) == 'r'
                lambLm=uL(k)-1;
                lambRm=uR(k)-1;
                disp([ 'lambda[Lm]= ' num2str(lambLm) ' > w= ' num2str(w(k))...
                    ' > lambda[Rm]= ' num2str(lambRm)])
            end
            if shock_type(k) == 's'
                lambLp=uL(k)+1;
                lambRp=uR(k)+1;
                disp([ 'lambda[Lp]= ' num2str(lambLp) ' > w= ' num2str(w(k))...
                    ' > lambda[Rp]= ' num2str(lambRp)])
            end
        end
    end
end
disp(' ')
        
        
