function remove_shock
% REMOVE_SHOCK.M - shock removal function
% checks to see if shock is consistent with:
% lambda[L-] >= w >= Lambda[R-] for s-shock
% lambda[L+] >= w >= Lambda[R+] for r-shock
% this problem might occur with gradient discontinuities
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting".
%
% Copyright 2005-2009 M.D.Salas.

global kout kmax x_s w uL uR t shock_type
global gL gR rL rR sL sR 

epszero=0;
kill_r=0;                                                                   % set flag

for k=1:kmax                                                                % check r-shocks
    if shock_type(k) == 'r'
        lambLm=uL(k)-1;
        lambRm=uR(k)-1;
        if (lambLm-w(k)) < epszero || (w(k)-lambRm) < epszero
            kill_r=k;                                                       % found r-shock to remove
            disp(' ')
            disp(['r-shock ' num2str(k) ' removed at t= '...
                num2str(t) ' step = ' num2str(kout)])
            disp('condition violated:')
            disp([ 'lambda[Lm]= ' num2str(lambLm) ' > w= ' num2str(w(k))...
                    ' > lambda[Rm]= ' num2str(lambRm)])
            disp(' ')
        end
    end
end

if kill_r ~= 0                                                              % remove r-shock
    for k=kill_r:kmax-1                                                     % shift index
        x_s(k)=x_s(k+1);
        w(k)=w(k+1);
        sL(k)=sL(k+1);
        sR(k)=sR(k+1);
        rL(k)=rL(k+1);
        rR(k)=rR(k+1);
        uL(k)=uL(k+1);
        gL(k)=gL(k+1);
        uR(k)=uR(k+1);
        gR(k)=gR(k+1);
        shock_type(k)=shock_type(k+1);
    end
    kmax=kmax-1;
    shock_index
end

kill_s=0;                                                                   % set flag

for k=1:kmax                                                                % check s-shocks
    if shock_type(k) == 's'
        lambLp=uL(k)+1;
        lambRp=uR(k)+1;
        if (lambLp-w(k)) < epszero || (w(k)-lambRp) < epszero
            kill_s=k;                                                       % found s-shock to remove
            disp(' ')
            disp(['s-shock ' num2str(k) ' removed at t= '...
                num2str(t) ' step = ' num2str(kout)])
            disp('condition violated:')
            disp([ 'lambda[Lp]= ' num2str(lambLp) ' > w= ' num2str(w(k))...
                    ' > lambda[Rp]= ' num2str(lambRp)])
            disp(' ')
        end
    end
end
if kill_s ~= 0                                                              % remove s-shock
    for k=kill_s:kmax-1                                                     % shift index
        x_s(k)=x_s(k+1);
        w(k)=w(k+1);
        sL(k)=sL(k+1);
        sR(k)=sR(k+1);
        rL(k)=rL(k+1);
        rR(k)=rR(k+1);
        uL(k)=uL(k+1);
        gL(k)=gL(k+1);
        uR(k)=uR(k+1);
        gR(k)=gR(k+1);
        shock_type(k)=shock_type(k+1);
    end
    kmax=kmax-1;
    shock_index
end


