function shock_cross
% SHOCK_CROSS.M - re-orders shocks after crossing
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting".
%
% Solves a non-conservative system of 2 PDE's
% dr/dt+(u-1)dr/dx=0, ds/dt+(u+1)ds/dt=0, r=u+g, s=u-g.
%
% performs crossing of k-1 & k shocks, where k=kcross
%
% Copyright 2005-2009 M.D.Salas.
%
global kmax kcross x_s w uL uR kmax_save
global gL gR rL rR sL sR t shock_type g u s r IS


disp([shock_type(kcross) '-shock ' num2str(kcross) ' crossing ' ...
    shock_type(kcross-1) '-shock ' num2str(kcross-1) ' at t= '...
    num2str(t) ' and x= ' num2str(x_s(kcross))])

km=kcross-1;
if shock_type(kcross) == shock_type(km)                                     % resolve crossing for shocks of same type
    uR(km)=uR(kcross);                                                      % two shocks in, one out
    gR(km)=gR(kcross);
    rR(km)=rR(kcross);
    sR(km)=sR(kcross);
    if shock_type(km) == 's'
        w(km)=(rL(km)+rR(km)+sL(km)+sR(km))/4+1;
    else
        w(km)=(rL(km)+rR(km)+sL(km)+sR(km))/4-1;
    end
    if kcross ~= kmax                                                       % shift shock index to left by one
        for k=kcross:kmax-1
            rL(k)=rL(k+1);
            rR(k)=rR(k+1);
            sL(k)=sL(k+1);
            sR(k)=sR(k+1);
            uL(k)=(rL(k)+sL(k))/2;
            gL(k)=(rL(k)-sL(k))/2;
            uR(k)=(rR(k)+sR(k))/2;
            gR(k)=(rR(k)-sR(k))/2;
            w(k)=w(k+1);
            x_s(k)=x_s(k+1);
        end
    end
    kmax_save=max(kmax_save,kmax);                                          % kmax_save is only used for plotting
    kmax=kmax-1;
    shock_index
else                                                                        % resolve crossing of shocks of different type
    if shock_type(km) == 'r'                                                % left shock is r-type, right shock is s-type
        shock_type(km)='s';
        shock_type(km+1)='r';
        rR(km)=rL(km);                                                      % left side of km unchanged; r(km) does not jump
        sR(km)=sR(km+1);                                                    % sR(km) takes value from km+1
        sL(km+1)=sR(km+1);                                                  % right side of km+1 unchanged; s(km+1) does not jump
        rL(km+1)=rL(km);                                                    % rL(km+1) takes value from km
        w(km)=(rL(km)+sL(km)+rR(km)+sR(km))/4+1;
        w(km+1)=(rL(km+1)+sL(km+1)+rR(km+1)+sR(km+1))/4-1;
    else                                                                    % left shock is s-type, right shock is r-type
        shock_type(km)='r';
        shock_type(km+1)='s';
        sR(km)=sL(km);                                                      % left side of km unchanged; s(km) does not jump
        rR(km)=rR(km+1);                                                    % rR(km) takes value from km+1
        rL(km+1)=rR(km+1);                                                  % right side of km+1 unchanged; r(km+1) does not jump
        sL(km+1)=sL(km);                                                    % sL(km+1) takes value from km
        w(km)=(rL(km)+sL(km)+rR(km)+sR(km))/4-1;
        w(km+1)=(rL(km+1)+sL(km+1)+rR(km+1)+sR(km+1))/4+1;
    end

    for k=km:km+1
        uL(k)=(rL(k)+sL(k))/2;
        gL(k)=(rL(k)-sL(k))/2;
        uR(k)=(rR(k)+sR(k))/2;
        gR(k)=(rR(k)-sR(k))/2;
    end
    if IS(km) < IS(km+1)                                                    % if true there is a mesh point between shock km and shock km+1
    n=IS(km+1);                                                             % r, s, g, u values at mesh point "n" need to be updated
    s(n)=(sR(km)+sL(km+1))/2;
    r(n)=(rR(km)+rL(km+1))/2;
    g(n)=(r(n)-s(n))/2;
    u(n)=(r(n)+s(n))/2;
    end
end

