% SHOCKFIT12.M - Floating shock-fitting main routine
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting".
% Solves a non-conservative system of 2 PDE's
% dr/dt+(u-1)dr/dx=0, ds/dt+(u+1)ds/dt=0, r=u+g, u=(r+s)/2    
%                                         s=u-g, g=(r-s)/2
% Computes r- and s-shocks.
% Dependencies. This code makes function calls to:
%
%           function mac_non_conservative12
%           function plot_solution12(INDEX)
%           function shock_index
%           function shock_cross
%           function initial12_s
%           function initial12_r
%           function initial12_rs
%           function initial12_gauss
%           function find_shk_xing
%           function detect_shock
%           function inject_shock(ns)
%           function mesh_xing
%           function remove_shock
%           function cubic_coeff(x1,u1,u2,u3,u4)
%
% These functions must reside in same directory as this code.
%
% Copyright 2005-2009 M.D.Salas.

clear all

global kout kplot tout xout N INIT CN kmax x_s 
global t LAST 

%               USER PROVIDED INPUTS
% INIT=1        s-shock "one-saw-tooth" 
% INIT=2        r-shock "one-saw-tooth"
% INIT=3        "The Mask of Batman"
% INIT=4        r-gaussian pulse
% INIT=5        s-gaussian pulse
% tlast         runs to a max time of t=tlast, set in "initial12" function
% CN=.8			Courant Number
% N=100		    number of mesh points -1, set in "initial12" function
% kplot=1       plot shock location every kplot steps
% IDETECT=1     Flag to detect s- and r-shocks; IDETECT=0 -> do not detect shocks
%                                               IDETECT=1 -> detect shocks

INIT=5;
CN=.8;
kplot=1;

% This code tracks r- and s-shocks. shock_type(k)='s' for s-shock,
% shock_type(k)='r' for r-shock, where
% k=1,2,...kmax, is the shock counter; if only one shock k=kmax=1
% IS(k)=n, where n is the mesh point to the left of the k-th shock
% I(n,1)=0, if no shock in cell [n,n+1]
% I(n,1)=k, where k is the index of the first shock in cell [n,n+1]:
%
%                       n              n+1
%                       o---\---\---\---o
%                          k   k+1  k+2
%
% I(n,2)=k, where k is the index of last shock in cell [n,n+1]:
%
%                       n              n+1
%                       o---\---\---\---o
%                          k-2  k-1  k
%
%                   NO CHANGES BELOW THIS LINE
%

disp('A SHOCK-FITTING PRIMER')
disp(' ')
disp('Output from SHOCKFIT12:')
disp(' ')
disp('Solution to dr/dt+(u-1)dr/dx=0,')
disp('            ds/dt+(u+1)ds/dx=0.')
disp(' ')
disp(['Inital conditions set for INIT= ' num2str(INIT)])
                                                                            %      INITIAL CONDITIONS
if INIT == 2
    initial12_r                                                             % define initial conditions
    disp('Corresponding to "One-Saw-Tooth", r-shock types ')
elseif INIT == 1
    initial12_s
    disp('Corresponding to "One-Saw-Tooth", s-shock types ')
elseif INIT == 3
    initial12_rs
    disp('Corresponding to the "Mask of Batman"')
elseif INIT == 4 || INIT == 5
    initial12_gauss
end
                                                                            %           OUTPUT
plot_solution12(1)                                                          % plot initial u-profile
                                                                            %         MAIN INTEGRATION LOOP
LAST=0;
kout=1;

if kmax ~= 0
    for k=1:kmax
        tout(k,kout)=t;
        xout(k,kout)=x_s(k);
    end
end

mac_non_conservative12
                                                                         %          OUTPUT
disp(' ')
disp([num2str(N) ' mesh points used in the computation'])
disp(['After ' num2str(kout) ' time steps:'])
disp(['time= ' num2str(t)])

plot_solution12(2)                                                          % plot final results
