function detect_shock
% DETECT_SHOCK.M - shock detection function
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting". 
%
% Copyright 2005-2009 M.D.Salas. 

global IS NT x kmax r s inject_type shock_type

det_trigger=-.5;                                                            % trigger to detect shock, based on gaussian pulse study
n_interval=2;

    dr(1)=1;                                                                % detect for r-shock formation
    dr(NT)=1;
    for n=2:NT-1                                                             
        dr(n)=r(n)-r(n-1);
    end
    
    if kmax > 0                                                             % avoid intervals near shocks of same type
        for k=1:kmax                                                        %             r-shock
            if shock_type(k) == 'r'                                         %                | 
                n=IS(k);                                                    %---o----o----o--|--o----o----o---
                if (n > n_interval+1) && (n < NT-(n_interval+1))            %  n-2  n-1   n  | n+1  n+2  n+3
                    for nn=n-n_interval:n+n_interval+1
                        dr(nn)=1;
                    end
                end
            end
        end
    end
    
    [drmin,ns]=min(dr);                                                     % find max |dr/dx| & dr <0

    if (ns > 3) && (ns < NT)
        x1=x(ns-2);
        r1=r(ns-2);
        r2=r(ns-1);
        r3=r(ns);
        r4=r(ns+1);

        [a,b,c,d]=cubic_coeff(x1,r1,r2,r3,r4);                              % compute cubic coefficients

        det=b^2-3*a*c;

        if det > det_trigger
            r_shock=-b/(3*a);                                               % r predicted by cubic fit
            x_shock=(((a*r_shock+b)*r_shock)+c)*r_shock+d;                  % shock location predicted by cubic fit
            if (x(ns-2) < x_shock) && (x(ns+1) > x_shock)                   % if x_shock not in [x(ns-2), x(ns+1)] don't insert shock
                inject_type='r';
                inject_shock(ns)
                shock_index
            end
        end
    end
    
    ds(1)=1;                                                                % detect for s-shock formation
    ds(NT)=1;
    for n=2:NT-1                                                             
        ds(n)=(s(n)-s(n-1));
    end
    if kmax > 0                                                             % avoid intervals near shocks of same type
        for k=1:kmax                                                        %             s-shock
            if shock_type(k) == 's'                                         %                | 
                n=IS(k);                                                    %---o----o----o--|--o----o----o---
                if (n > n_interval+1) && (n < NT-(n_interval+1))            %  n-2  n-1   n  | n+1  n+2  n+3
                    for nn=n-n_interval:n+n_interval+1
                        ds(nn)=1;
                    end
                end
            end
        end
    end

    [dsmin,ns]=min(ds);                                                     % find max |ds/dx| & ds <0

    if (ns > 3) && (ns < NT)
        x1=x(ns-2);
        s1=s(ns-2);
        s2=s(ns-1);
        s3=s(ns);
        s4=s(ns+1);

        [a,b,c,d]=cubic_coeff(x1,s1,s2,s3,s4);                              % compute cubic coefficients

        det=b^2-3*a*c;

        if det > det_trigger
            s_shock=-b/(3*a);                                               % s predicted by cubic fit
            x_shock=(((a*s_shock+b)*s_shock)+c)*s_shock+d;                  % shock location predicted by cubic fit
            if (x(ns-2) < x_shock) && (x(ns+1) > x_shock)                   % if x_shock not in [x(ns-2), x(ns+1)] don't insert shock
                inject_type='s';
                inject_shock(ns)
                shock_index
            end
        end
    end
    


