function initial13_gauss
% INITIAL13_GAUSS.M -defines Gaussian pulse initial conditions
%  A SHOCK FITTING PRIMER - CHAPTER 2
%
% Solves a non-conservative system of 2 PDE's
% dr/dt+(u+1)dr/dx=0, ds/dt+(u-1)ds/dt=0, r=u+g, s=u-g.
%
% INIT=4 for "r" gaussian distribution, s=s0
% INIT=5 for "s" gaussian distribution, r=r0
%
%  Copyright 2005-2009 M.D.Salas.

global eps0 dx I N NT nfig kmax kmax_save xl xr tlast t
global x u g r s R S INIT IDETECT

tlast=2.5;

t=0;
N=300;
IDETECT=1;

NT=N+1;
kmax=0; 
kmax_save=kmax;                                                             % kmax_save is only used for plotting

eps0=10^-10;                                                                % small number
nfig=0;                                                                     % figure counter
                                                                            %      SHOCK INDEXES
for n=1:NT
    I(n,1)=0;
    I(n,2)=0;
end

if INIT == 4
    xl=-3;                                                                      
    xr=6;
    dx=(xr-xl)/N;                                                           % mesh interval size
    disp('Corresponding to r-gaussian initial conditions')
    s0=0;
    for n=1:NT
        s(n)=s0;
        x(n)=xl+dx*(n-1);                                                   % physical coordinate [xs,xr]
        r(n)=exp(-x(n)^2);
    end

elseif INIT == 5
    xl=-6;                                                                      
    xr=3;
    dx=(xr-xl)/N;                                                           % mesh interval size
    disp('Corresponding to s-gaussian initial conditions')
    r0=0;
    for n=1:NT
        r(n)=r0; 
        x(n)=xl+dx*(n-1);                                                   % physical coordinate [xs,xr]
        s(n)=-exp(-x(n)^2);
    end
end

S=s;
R=r;
u=(r+s)/2;
g=(r-s)/2;
