function inject_shock(ns)
% INJECT_SHOCK.M - shock injection function
% initializes newly detected shock
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting".
%
% Copyright 2005-2009 M.D.Salas.

global kout IS kmax x_s w uL uR t shock_type inject_type
global gL gR rL rR sL sR kmax_save x r s

xn=(x(ns)+x(ns-1))/2.;                                                      % inject shock midpoint where max gradient was found
sLn=s(ns-1);                                                                % set shock values equal to mesh point values bounding shock
sRn=s(ns);
rLn=r(ns-1);
rRn=r(ns);

if inject_type == 's'
    disp('s-shock')
    wn=(sLn+sRn+rLn+rRn)/4-1;
else
    disp('r-shock')
    wn=(sLn+sRn+rLn+rRn)/4+1;
end
disp(['detected at t= ' num2str(t) ', step = ' num2str(kout)])
disp(['between n [' num2str(ns-1) ',' num2str(ns) ']'])
disp(['x_s= ' num2str(xn) ' w= ' num2str(wn)])

if kmax == 0                                                                % no other shocks present
    kin=1;
    kmax=1;
    x_s(1)=xn;
    w(1)=wn;
    shock_type(1)=inject_type;
    sL(1)=sLn;
    sR(1)=sRn;
    rL(1)=rLn;
    rR(1)=rRn;
    uL(1)=(rL(1)+sL(1))/2;
    gL(1)=(rL(1)-sL(1))/2;
    uR(1)=(rR(1)+sR(1))/2;
    gR(1)=(rR(1)-sR(1))/2;
else                                                                        % other shocks presents
    nkmax=IS(kmax);
    n1=IS(1);
    if nkmax < ns ;                                                         % last shock to left of new shock
        kmax=kmax+1;
        kin=kmax;
        x_s(kmax)=xn;
        w(kmax)=wn;
        shock_type(kmax)=inject_type;
        sL(kmax)=sLn;
        sR(kmax)=sRn;
        rL(kmax)=rLn;
        rR(kmax)=rRn;
        uL(kmax)=(rL(kmax)+sL(kmax))/2;
        gL(kmax)=(rL(kmax)-sL(kmax))/2;
        uR(kmax)=(rR(kmax)+sR(kmax))/2;
        gR(kmax)=(rR(kmax)-sR(kmax))/2;
    elseif n1 > ns                                                          % first shock to right of new shock
        kmax=kmax+1;
        for k=2:kmax
            x_s(k)=x_s(k-1);
            w(k)=w(k-1);
            sL(k)=sL(k-1);
            sR(k)=sR(k-1);
            rL(k)=rL(k-1);
            rR(k)=rR(k-1);
            uL(k)=(rL(k)+sL(k))/2;
            gL(k)=(rL(k)-sL(k))/2;
            uR(k)=(rR(k)+sR(k))/2;
            gR(k)=(rR(k)-sR(k))/2;
        end
        kin=1;
        x_s(1)=xn;
        w(1)=wn;
        shock_type(1)=inject_type;
        sL(1)=sLn;
        sR(1)=sRn;
        rL(1)=rLn;
        rR(1)=rRn;
        uL(1)=(rL(1)+sL(1))/2;
        gL(1)=(rL(1)-sL(1))/2;
        uR(1)=(rR(1)+sR(1))/2;
        gR(1)=(rR(1)-sR(1))/2;
    else                                                                    % new shock in between other shocks
        for k=1:kmax                                                        % find shock index to left of new shock
            n=IS(k);
            if n < ns
                kleft=k;
            end
        end
        for k=kleft+1:kmax
            x_s(k+1)=x_s(k);
            w(k+1)=w(k);
            sL(k+1)=sL(k);
            sR(k+1)=sR(k);
            rL(k+1)=rL(k);
            rR(k+1)=rR(k);
            uL(k+1)=(rL(k+1)+sL(k+1))/2;
            gL(k+1)=(rL(k+1)-sL(k+1))/2;
            uR(k+1)=(rR(k+1)+sR(k+1))/2;
            gR(k+1)=(rR(k+1)-sR(k+1))/2;
        end
        kmax=kmax+1;
        kin=kleft+1;
        x_s(kleft+1)=xn;
        w(kleft+1)=wn;
        shock_type(kleft+1)=inject_type;
        sL(kleft+1)=sLn;
        sR(kleft+1)=sRn;
        rL(kleft+1)=rLn;
        rR(kleft+1)=rRn;
        uL(kleft+1)=(rL(kleft+1)+sL(kleft+1))/2;
        gL(kleft+1)=(rL(kleft+1)-sL(kleft+1))/2;
        uR(kleft+1)=(rR(kleft+1)+sR(kleft+1))/2;
        gR(kleft+1)=(rR(kleft+1)-sR(kleft+1))/2;
    end
end
disp(['shock index k = ' num2str(kin) ])
disp(' ')
kmax_save=kmax;
