function mesh_xing
% MESH_XING.M - performs update for mesh point crossed by a shock wave
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting".
%
% Copyright 2005-2009 M.D.Salas.

global dx dt ISO dts xo_s rLo rRo sRo sLo I IS kmax x x_s 
global R S Ro So rL rR sL sR

ISO=IS;                                                                     % save shock IS index, to detect shock crossing mesh point

shock_index                                                                 % re-build shock indexes

for k=1:kmax
    if ISO(k) < IS(k)                                                       % if shock index changed (+), recompute point crossed by right running shock
        n=IS(k);
        if I(n-1,1) == 0                                                    % no shocks in [n-1,n]
            kk=I(n,1);
            delta=(x(n)-xo_s(kk))/(x_s(kk)-xo_s(kk));
            ra=Ro(n-1)+delta*(R(n-1)-Ro(n-1));
            rb=rLo(kk)+delta*(rL(kk)-rLo(kk));
            sa=So(n-1)+delta*(S(n-1)-So(n-1));
            sb=sLo(kk)+delta*(sL(kk)-sLo(kk));
            ub=.5*(rb+sb);
            R(n)=rb-(ub+1)*(rb-ra)*(1-delta)*dt/dx;
            Ro(n)=rb;
            S(n)=sb-(ub-1)*(sb-sa)*(1-delta)*dt/dx;
            So(n)=sb;
            dts(n)=(1-delta)*dt;                                            % save time increment for use in corrector step
        else
            delta=(x(n)-xo_s(k))/(x_s(k)-xo_s(k));
            R(n)=rL(k)*(x(n)-x_s(k-1))/(x_s(k)-x_s(k-1))+...
                rR(k-1)*(x(n)-x_s(k))/(x_s(k-1)-x_s(k));
            Ro(n)=rLo(k)+delta*(rL(k)-rLo(k));
            S(n)=sL(k)*(x(n)-x_s(k-1))/(x_s(k)-x_s(k-1))+...
                sR(k-1)*(x(n)-x_s(k))/(x_s(k-1)-x_s(k));
            So(n)=sLo(k)+delta*(sL(k)-sLo(k));
            dts(n)=(1-delta)*dt;
        end
    elseif ISO(k) > IS(k)                                                   % if shock index changed (-), recompute point crossed by left running shock
        n=IS(k)+1;
        if I(n,1) == 0                                                      % no shocks in [n,n+1]
            kk=I(n-1,2);
            delta=(x(n)-xo_s(kk))/(x_s(kk)-xo_s(kk));
            ra=Ro(n+1)+delta*(R(n+1)-Ro(n+1));
            rb=rRo(kk)+delta*(rR(kk)-rRo(kk));
            sa=So(n+1)+delta*(S(n+1)-So(n+1));
            sb=sRo(kk)+delta*(sR(kk)-sRo(kk));
            ub=.5*(rb+sb);
            R(n)=rb-(ub+1)*(ra-rb)*(1-delta)*dt/dx;
            Ro(n)=rb;
            S(n)=sb-(ub-1)*(sa-sb)*(1-delta)*dt/dx;
            So(n)=sb;
            dts(n)=(1-delta)*dt;                                            % save time increment for use in corrector step
        else
            delta=(x(n)-xo_s(k))/(x_s(k)-xo_s(k));
            R(n)=rL(k+1)*(x(n)-x_s(k))/(x_s(k+1)-x_s(k))+...
                rR(k)*(x(n)-x_s(k+1))/(x_s(k)-x_s(k+1));
            Ro(n)=rRo(k)+delta*(rR(k)-rRo(k));
            S(n)=sL(k+1)*(x(n)-x_s(k))/(x_s(k+1)-x_s(k))+...
                sR(k)*(x(n)-x_s(k+1))/(x_s(k)-x_s(k+1));
            So(n)=sRo(k)+delta*(sR(k)-sRo(k));
            dts(n)=(1-delta)*dt;
        end
    end
end