function initialpiston
% INITIALPISTON.M - Initial conditions, gas at rest
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows" 
%
%
% Copyright 2005-2008 M.D.Salas.

global eps0 dx N NT nfig x X t tlast stop a u P Z gamma Xx Xt U R Q sg gm
global uexact Pexact aexact

% starting from t=0.1; to start from t=0, comment call to exact below
% and change c0 to .05 in "exitboundary"
t=0.1;
gamma=1.4;
gm=gamma-1;
sg=sqrt(gamma);

if tlast > 0
    b=pistonlaw(t);                                                         % call piston routine
    bt=pistonlawt(t);
    c=exitboundary(t);                                                      % call exit boundary routine
    ct=exitboundaryt(t);
    NT=N+1;
    eps0=10^-10;                                                            % small number
    nfig=0;                                                                 % figure counter
    dx=1/N;
    Xx=1/(c-b);                                                             % mesh interval size
    X=zeros(1,NT);
    u=zeros(1,NT);
    P=zeros(1,NT);
    Z=ones(1,NT);
    a=zeros(1,NT);
    
    
    for n=1:NT
        X(n)=dx*(n-1);                                                      % computational coordinate
        a(n)=sg;
    end

%%{ 
% use exact solution if t start is different from zero, if t start = 0 
% comment these lines
    exact                                                                  
    for n=1:NT
        a(n)=aexact(n);
        P(n)=Pexact(n);
        u(n)=uexact(n);
        Z(n)=a(n)^2/gamma;
    end
%}
    
        Xt=-Xx*(bt+X*(ct-bt));
        x=b+(c-b)*X;                                                        % physical coordinate                                                                                                                                                                                         % speed of sound
        U=Xt(n)+u*Xx;                                                       % contravariant velocity
        R=2*a/gm+u;                                                         % Riemann variables
        Q=2*a/gm-u;
else
    disp('   ')
    disp('"tlast" entered is less than starting time for case selected.')
    disp('Execution is stopped. Please check input parameters.')
    disp('   ')
    stop=1;
end
