function lambda
% LAMBDA.M - solver based on Lambda Scheme, original version
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global kout dx dt U t tlast LAST N NT CN u un a X Xt Xx Q R gm
global uexact u_error x_error t_error

SAFETY=10000;                                                               % stops calculation if over run condition

while LAST == 0                                                             % begin time stepping loop

    kout=kout+1;                                                            % step counter

    if kout > SAFETY                                                        % stop calculation if out-of-control condition
        LAST=1;
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value in function mac_non_con_piston')
    end

    b=pistonlaw(t);
    bt=pistonlawt(t);
    c=exitboundary(t);
    ct=exitboundaryt(t);
    Xx=1/(c-b);
                                                               
    Aabs=abs(U)+a*Xx;                                                       % begin CFL conditio
    Amax=max(Aabs);
    dt=CN*dx/Amax;                                                          % end CFL condition

    if (t+dt) >= tlast                                                      % check that time does not exceed tlast
        dt=tlast-t;
        LAST=1;                                                             % trigger to stop while loop
    end

    sigma=dt/dx;

    Ro=R;                                                                   % save latest values
    Qo=Q;

    for j=1:2                                                               % predictor-corrector loop
        Xt=-Xx*(bt+X*(ct-bt));
        U=Xt+u*Xx;
        lambdaP=U+Xx*a;                                                     % characteristic slope in computational plane
        lambdaM=U-Xx*a;
        
        for n=2:N                                                           % sweep through mesh points         

            if j == 1                                                       % Predictor
                if n ~= 2
                    Rx=2*R(n)-3*R(n-1)+R(n-2);                              % lambdaP > 0
                    Qx=Q(n+1)-Q(n);                                         % lambdaM < 0
                    if lambdaP(n) < 0
                        Rx=R(n+1)-R(n);                                     % lower case 'x' is used for computational coordinate X
                    end
                    if lambdaM(n) > 0
                        Qx=2*Q(n)-3*Q(n)+Q(n-2);
                    end
                else                                                        % n = 2
                    Rx=R(n)-R(n-1);
                    Qx=Q(n+1)-Q(n);
                    if lambdaP(n) < 0
                        Rx=R(n+1)-R(n);                                        
                    end
                    if lambdaM(n) > 0
                        Qx=Q(n)-Q(n-1);
                    end
                end

            else                                                            % Corrector
                if n ~= N
                    Rx=R(n)-R(n-1);                                         % lambdaP > 0
                    Qx=-(2*Q(n)-3*Q(n+1)+Q(n+2));                           % lambdaM < 0
                    if lambdaP(n) < 0
                        Rx=-(2*R(n)-3*R(n+1)+R(n+2));
                    end
                    if lambdaM(n) > 0
                        Qx=Q(n)-Q(n-1);
                    end
                else                                                        % n = N
                    Rx=R(n)-R(n-1);
                    Qx=Q(n+1)-Q(n);
                    if lambdaP(n) < 0
                        Rx=R(n+1)-R(n);
                    end
                    if lambdaM(n) > 0
                        Qx=Q(n)-Q(n-1);
                    end
                end
            end
            
            Rt=-lambdaP(n)*Rx;
            Qt=-lambdaM(n)*Qx;

            Rn(n)=R(n)+sigma*Rt;
            Qn(n)=Q(n)+sigma*Qt;
            if j == 2
                Rn(n)=.5*(Ro(n)+Rn(n));
                Qn(n)=.5*(Qo(n)+Qn(n));
            end
        end

                                                                            % piston b.c.
        un(1)=pistonlawt(t+dt);
        Qx=Q(2)-Q(1);
        if j == 2
            Qx=-(2*Q(1)-3*Q(2)+Q(3));
        end
        Qt=-lambdaM(1)*Qx;
        Qn(1)=Q(1)+sigma*Qt;
        if j == 2
            Qn(1)=.5*(Qo(1)+Qn(1));
        end
        
        an1=gm*(Qn(1)+un(1))/2;                                             
        Rn(1)=2*an1/gm+un(1);
                                                                            % exit b.c. gas at rest
        Rx=R(NT)-R(N);
        if j == 2
            Rx=2*R(NT)-3*R(N)+R(N-1);
        end
        Rt=-lambdaP(NT)*Rx;
        Rn(NT)=R(NT)+sigma*Rt;
        if j == 2
            Rn(NT)=.5*(Ro(NT)+Rn(NT));
        end
        
        Qn(NT)=Q(NT);                                                       % Q given                                                                                                                                     
        
        R=Rn;
        Q=Qn;
        a=gm*(R+Q)/4;
        u=(R-Q)/2;

    end                                                                     % end predictor-corrector loop
    t=t+dt; 
    
    exact
    b=pistonlaw(t);
    c=exitboundary(t);
    for n=1:NT
        u_error(n,kout)=u(n)-uexact(n);
        x_error(n,kout)=b+X(n)*(c-b);
        t_error(n,kout)=t;
    end
end                                                                         % end time stepping "while" loop

