function lambda_gabutti_2
% LAMBDA_GABUTTI_2.M - solver based on Lambda Scheme, Gabutti variation
% dependent variable u, a
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global kout dx dt U t tlast LAST N NT CN u a X Xt Xx R Q gm
global uexact u_error x_error t_error

SAFETY=10000;                                                                % stops calculation if over run condition

while LAST == 0                                                             % begin time stepping loop

    kout=kout+1;                                                            % step counter

    if kout > SAFETY                                                        % stop calculation if out-of-control condition
        LAST=1;
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value in function mac_non_con_piston')
    end
    
    b=pistonlaw(t);
    bt=pistonlawt(t);
    c=exitboundary(t);
    ct=exitboundaryt(t);
    Xx=1/(c-b);

    Aabs=abs(U)+a*Xx;                                                       % begin CFL condition
    Amax=max(Aabs);
    dt=CN*dx/Amax;                                                          % end CFL condition

    if (t+dt) >= tlast                                                      % check that time does not exceed tlast
        dt=tlast-t;
        LAST=1;                                                             % trigger to stop while loop
    end

    sigma=dt/dx;

    for j=1:3
        Xt=-Xx*(bt+X*(ct-bt));
        U=Xt+u*Xx;
        lambdaP=U+Xx*a;                                                     % characteristic slope in computational plane
        lambdaM=U-Xx*a;
        signLP=sign(lambdaP);
        signLM=sign(lambdaM);

        for n=2:N
            sLP=signLP(n);
            sLM=signLM(n);
            if j == 2
                if n == 2                        
                    Rx1=-(2*R(1)-3*R(2)+R(3));
                    R0=R(1)-Rx1;
                    Rx=(2*R(2)-3*R(1)+R0);                        
                    Qx1=-(2*Q(1)-3*Q(2)+Q(3));
                    Q0=Q(1)-Qx1;
                    Qx=(2*Q(2)-3*Q(1)+Q0);
                    if sLP < 0
                        Rx=-(2*R(n)-3*R(n+1)+R(n+2));
                    end
                    if sLM < 0
                        Qx=-(2*Q(n)-3*Q(n+1)+Q(n+2));
                    end
                elseif n == N                        
                    Rx=sLP*(R(n)-R(n-sLP));
                    Qx=sLM*(Q(n)-Q(n-sLM));
                    if sLP > 0
                        Rx=2*R(n)-3*R(n-1)+R(n-2);
                    end
                    if sLM > 0
                        Qx=2*Q(n)-3*Q(n-1)+Q(n-2);     
                    end
                else
                    Rx=sLP*(2*R(n)-3*R(n-sLP)+R(n-2*sLP));
                    Qx=sLM*(2*Q(n)-3*Q(n-sLM)+Q(n-2*sLM));
                end
            else
                Rx=sLP*(R(n)-R(n-sLP));
                Qx=sLM*(Q(n)-Q(n-sLM));
            end

            at=-gm*(lambdaP(n)*Rx+lambdaM(n)*Qx)*sigma/4;
            ut=-(lambdaP(n)*Rx-lambdaM(n)*Qx)*sigma/2;

            if j == 1
                an(n)=a(n)+at;
                un(n)=u(n)+ut;
                ao(n)=a(n);
                uo(n)=u(n);
            elseif j == 2
                ats(n)=at;
                uts(n)=ut;
            else
                an(n)=ao(n)+.5*(ats(n)+at);
                un(n)=uo(n)+.5*(uts(n)+ut);
            end
        end
        
        % piston b.c.

        un(1)=pistonlawt(t+dt);
        if j == 1
            Qx=Q(2)-Q(1);
            Qt=-lambdaM(1)*Qx*sigma;
            Qn1=Q(1)+Qt;
            an(1)=gm*(Qn1+un(1))/2;
            Qo1=Q(1);
        elseif j == 2
            Qx=-(2*Q(1)-3*Q(2)+Q(3));
            Qt=-lambdaM(1)*Qx*sigma;
            Qts1=Qt;
        else
            Qx=Q(2)-Q(1);
            Qt=-lambdaM(1)*Qx*sigma;
            Qn1=Qo1+.5*(Qts1+Qt);
            an(1)=gm*(Qn1+un(1))/2;
        end

       % exit b.c. gas at rest
        if j == 1
            Rx=R(NT)-R(N);
            Rt=-lambdaP(NT)*Rx*sigma;
            RnNT=R(NT)+Rt;
            QnNT=Q(NT);
            RoNT=R(NT);
            an(NT)=gm*(RnNT+QnNT)/4;
            un(NT)=(RnNT-QnNT)/2;
        elseif j == 2
            Rx=2*R(NT)-3*R(NT-1)+R(NT-2);
            Rt=-lambdaP(NT)*Rx*sigma;
            RtsNT=Rt;
        else
            Rx=R(NT)-R(N);
            Rt=-lambdaP(NT)*Rx*sigma;
            RnNT=RoNT+.5*(RtsNT+Rt);
            QnNT=Q(NT);
            an(NT)=gm*(RnNT+QnNT)/4;
            un(NT)=(RnNT-QnNT)/2;
        end
        
        if j >= 2
            a=an;
            u=un;
            R=2*a/gm+u;
            Q=2*a/gm-u;
        end
    end
    t=t+dt;

    exact
    b=pistonlaw(t);
    c=exitboundary(t);
    for n=1:NT
        u_error(n,kout)=u(n)-uexact(n);
        x_error(n,kout)=b+X(n)*(c-b);
        t_error(n,kout)=t;
    end

end                                                                         % end time stepping "while" loop

