function mac_non_con_1
% MAC_NON_CON_1.M - solver based on MacCormack's predictor
% -corrector method, dependent variables u, P
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global kout dx dt U t tlast LAST N NT CN u P uo Po un Pn gamma a Z X Xt Xx 
global uexact u_error x_error t_error gm

SAFETY=10000;                                                               % stops calculation if over run condition

while LAST == 0                                                             % begin time stepping loop
   
    kout=kout+1;                                                            % step counter

    if kout > SAFETY                                                        % stop calculation if out-of-control condition
        LAST=1;
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value in function mac_non_con_piston')
    end
    
    b=pistonlaw(t);
    bt=pistonlawt(t);
    c=exitboundary(t);
    ct=exitboundaryt(t);
    Xx=1/(c-b);
    
    Aabs=abs(U)+a*Xx;                                                       % begin CFL condition
    Amax=max(Aabs);
    dt=CN*dx/Amax;                                                          % end CFL condition

    if (t+dt) >= tlast                                                      % check that time does not exceed tlast
        dt=tlast-t;
        LAST=1;                                                             % trigger to stop while loop
    end
    
    sigma=dt/dx;
    
    uo=u;
    Po=P;
    
    for j=1:2                                                               % predictor-corrector loop
        Xt=-Xx*(bt+X*(ct-bt));
        U=Xt+u*Xx;
        np=2-j;
        nm=1-j;

        for n=2:N                                                           % sweep through mesh points

            ux=u(n+np)-u(n+nm);                                             % lower case 'x' is used for computational coordinate X
            Px=P(n+np)-P(n+nm);

            ut=-(U(n)*ux+Z(n)*Xx*Px);
            Pt=-(U(n)*Px+gamma*Xx*ux);

            un(n)=u(n)+sigma*ut;
            Pn(n)=P(n)+sigma*Pt;
            
            if j == 2
                un(n)=.5*(uo(n)+un(n));
                Pn(n)=.5*(Po(n)+Pn(n));
            end

        end

        % piston b.c.
        
        un(1)=pistonlawt(t+dt);
        ux=u(2)-u(1);
        Px=P(2)-P(1);
        if j == 2
            ux=-(2*u(1)-3*u(2)+u(3));
            Px=-(2*P(1)-3*P(2)+P(3));
        end
        Pt=-(U(1)*Px+gamma*Xx*ux);
        Pn(1)=P(1)+sigma*Pt;
        if j == 2
            Pn(1)=.5*(Po(1)+Pn(1));
        end
        
        % exit b.c. gas at rest at n=NT
                                                         
        Pn(NT)=0.;
        ux=u(NT)-u(N);
        Px=P(NT)-P(N);
        if j == 2
            ux=2*u(NT)-3*u(N)+u(N-1);
            Px=2*P(NT)-3*P(N)+P(N-1);
        end
        ut=-(U(NT)*ux+Z(NT)*Xx*Px);
        un(NT)=u(NT)+sigma*ut;
        if j == 2
            un(NT)=.5*(uo(NT)+un(NT));
        end
        
        u=un;                                                               % update
        P=Pn;
        Z=exp(gm*P/gamma);
        a=sqrt(gamma*Z);

    end                                                                     % end predictor-corrector loop
    t=t+dt;

    exact
    b=pistonlaw(t);
    c=exitboundary(t);
    for n=1:NT
        u_error(n,kout)=u(n)-uexact(n);
        x_error(n,kout)=b+X(n)*(c-b);
        t_error(n,kout)=t;
    end

end                                                                         % end time stepping "while" loop


