function mac_non_con_2
% MAC_NON_CON_2.M - solver based on MacCormack's predictor
% -corrector method, dependent variables R, Q
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global kout dx dt U t tlast LAST N NT CN u a X Xt Xx R Q gm
global uexact u_error x_error t_error

SAFETY=10000;                                                               % stops calculation if over run condition

while LAST == 0                                                             % begin time stepping loop
   
    kout=kout+1;                                                            % step counter

    if kout > SAFETY                                                        % stop calculation if out-of-control condition
        LAST=1;
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value in function mac_non_con_piston')
    end

    b=pistonlaw(t);
    bt=pistonlawt(t);
    c=exitboundary(t);
    ct=exitboundaryt(t);
    Xx=1/(c-b);
    
    Aabs=abs(U)+a*Xx;                                                       % begin CFL condition
    Amax=max(Aabs);
    dt=CN*dx/Amax;                                                          % end CFL condition

    if (t+dt) >= tlast                                                      % check that time does not exceed tlast
        dt=tlast-t;
        LAST=1;                                                             % trigger to stop while loop
    end
    
    sigma=dt/dx;
    
    Qo=Q;                                                                   % save latest values
    Ro=R;

    for j=1:2                                                               % predictor-corrector loop
        Xt=-Xx*(bt+X*(ct-bt));
        U=Xt+u*Xx;
        lambdaP=U+Xx*a;                                                     % characteristic slope in computational plane
        lambdaM=U-Xx*a;
        np=2-j;
        nm=1-j;

        for n=2:N                                                           % sweep through mesh points

            Rx=R(n+np)-R(n+nm);                                             % lower case 'x' is used for computational coordinate X
            Qx=Q(n+np)-Q(n+nm);

            Rt=-lambdaP(n)*Rx;
            Qt=-lambdaM(n)*Qx;

            Rn(n)=R(n)+sigma*Rt;
            Qn(n)=Q(n)+sigma*Qt;
            if j == 2
                Rn(n)=.5*(Ro(n)+Rn(n));
                Qn(n)=.5*(Qo(n)+Qn(n));
            end
            
        end

        % piston b.c.
 
        un(1)=pistonlawt(t+dt);
        Qx=Q(2)-Q(1);
        if j == 2
            Qx=-(2*Q(1)-3*Q(2)+Q(3));
        end
        Qt=-lambdaM(1)*Qx;
        Qn(1)=Q(1)+sigma*Qt;
        if j == 2
            Qn(1)=.5*(Qo(1)+Qn(1));
        end     
        an1=gm*(Qn(1)+un(1))/2;                                             
        Rn(1)=2*an1/gm+un(1);

        % exit b.c. 

        Rx=R(NT)-R(N);
        if j == 2
            Rx=2*R(NT)-3*R(NT-1)+R(NT-2);
        end
        Rt=-lambdaP(NT)*Rx;
        Rn(NT)=R(NT)+sigma*Rt;
        if j == 2
            Rn(NT)=.5*(Ro(NT)+Rn(NT));
        end     
        Qn(NT)=Q(NT);
        
        R=Rn;                                                               % update
        Q=Qn;
        a=gm*(R+Q)/4;
        u=(R-Q)/2;
        
    end                                                                     % end predictor-corrector loop
        t=t+dt;                                                     


    exact
    b=pistonlaw(t);
    c=exitboundary(t);
    for n=1:NT
        u_error(n,kout)=u(n)-uexact(n);
        x_error(n,kout)=b+X(n)*(c-b);
        t_error(n,kout)=t;
    end
end                                                                         % end time stepping "while" loop

