% PISTON.M - Code for piston driven flow, no shocks, simple wave region
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
%{
% Dependencies. This code makes function calls to:
%
%           function mac_non_con
%           function mac_non_con_1
%           function lambda_gabutti_1
%           function lambda_gabutti_2
%           function lambda
%           function plot_solutionpiston(INDEX)
%           function initialpiston
%           function exitboundary(t)
%           function exitboundaryt(t)
%           function exitboundarytt(t)
%           function pistonlaw(t)
%           function pistonlawt(t)
%           function pistonlawtt(t)
%           function exact
%
% These functions must reside in same directory as this code.
%}
% Copyright 2005-2008 M.D.Salas.

clear all

global kout N INIT CN tlast LAST  stop

%{
%               USER PROVIDED INPUTS
% INIT=1 MacCormack Nonconservative, variables [u,P]
% INIT=2 MacCormack Nonconservative, variables [R,Q]
% INIT=3 Lambda-Gabutti, variables [R,Q]
% INIT=4 Lambda-Gabutti Nonconservative, variables [u,a]
% INIT=5 Lambda Scheme Nonconservative, variables [R,Q]
% INIT=6 Lambda Scheme 1 Nonconservative, variables [R,Q]
% tlast runs to a max time of t=tlast, see tlast requirements above
% CN=Courant Number
% N=number of mesh points -1
%}

disp(' ')
disp('A SHOCK-FITTING PRIMER')
disp(' ')
disp('INIT=1 MacCormack Nonconservative, variables [u,P]')
disp('INIT=2 MacCormack Nonconservative, variables [R,Q]')
disp('INIT=3 Lambda-Gabutti Nonconservative, variables [R,Q]')
disp('INIT=4 Lambda-Gabutti Nonconservative, variables [u,a]')
disp('INIT=5 Lambda Scheme Nonconservative, variables [R,Q]')
disp('INIT=6 Lambda Scheme 1 Nonconservative, variables [R,Q]')
disp(' ');
disp('In the next line enter the case to be executed with default values')
INIT=input('Enter INIT [1,2,3,4,5,6]: ');

tlast=.86;                                                                   % NOTE: shock forms at 0.8439 for b(t)=t^3
CN=.65;                                                                      % this code should not be used for t > t-of-shock-formation
N=100;

%               NO CHANGES BELOW THIS LINE
%
%               INITIAL CONDITIONS

stop=0;
initialpiston                                                               % define initial conditions

if stop == 1
    return                                                                  % execution stopped if stop==1
else
    %               OUTPUT
    plot_solutionpiston(1)                                                  % plot initial u-profile
    %               MAIN INTEGRATION LOOP
    LAST=0;
    kout=0;

    switch (INIT)
        case 1
            mac_non_con_1
        case 2
            mac_non_con_2
        case 3
            lambda_gabutti_1
        case 4
            lambda_gabutti_2
        case 5
            lambda
        case 6
            lambda_1
    end

    %               OUTPUT
    disp('Output from PISTON:')
    disp(' ')
    disp(['N= ' num2str(N)])
    switch (INIT)
        case 1
            disp('MacCormack Nonconservative, variables [u,P] ')
        case 2
            disp('MacCormack Nonconservative, variables [R,Q] ')
        case 3
            disp('Lambda-Gabutti Nonconservative, variables [R,Q]')
        case 4
            disp('Lambda-Gabutti Nonconservative, variables [u,a]')
        case 5
            disp('Lambda Scheme Nonconservative, variables [R,Q] ')
        case 6
            disp('Lambda Scheme 1 Nonconservative, variables [R,Q] ')
    end
    disp(' ')
    disp(['After ' num2str(kout) ' time steps:'])

    plot_solutionpiston(2)                                                  % plot final results
end
