function plot_solutionpiston(INDEX)
% PLOT_SOLUTIONPISTON.M creates plots
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global nfig u sg a X NT t uexact aexact u_error x_error t_error gm kout

map = [0 0 0]; 

b=pistonlaw(t);
bt=pistonlawt(t);
c=exitboundary(t);
ct=exitboundaryt(t);
R=2*a/gm+u;
Q=2*a/gm-u;
x=b+(c-b)*X;
Xx=1/(c-b);
    
if INDEX == 1
    nfig=nfig+1;
    figure(nfig)
    plot(x,u,'k-')
    xlabel('x')
    ylabel('u')
    f=['Initial Conditions, t=' num2str(t)];
    title(f)

else                                                                        % INDEX==2
    exact

    xbar=sg*t;
    nfig=nfig+1;
    figure(nfig)
    plot(x,u,'k-',x,uexact,'ko',xbar,0,'r+')

    xlim([x(1) x(NT)])
    xlabel('x')
    ylabel('u')
    legend('computed', 'exact')
    f=['u at t=' num2str(t)];
    title(f)
    
    nfig=nfig+1;
    figure(nfig)
    plot(x,a,'k-',x,aexact,'ko',xbar,0,'r+')
    xlabel('x')
    ylabel('a')
    ylim([1 1.6])
    legend('computed', 'exact')
    f=['a at t=' num2str(t)];
    title(f)
    
    nfig=nfig+1;
    figure(nfig)
    plot(x,R,'k-',xbar,0,'r+')
    xlabel('x')
    ylabel('R')
    ylim([5 10])
    f=['R at t=' num2str(t)];
    title(f)
    
    nfig=nfig+1;
    figure(nfig)
    plot(x,Q,'k-',xbar,0,'r+')
    xlabel('x')
    ylabel('Q')
    ylim([5 6])
    f=['Q at t=' num2str(t)];
    title(f)
    
    %for n=1:NT
        Xt=-Xx*(bt+X*(ct-bt));
        U=Xt+u*Xx;
        lambdaP=U+Xx*a;
        lambdaM=U-Xx*a;
    %end
    
    nfig=nfig+1;
    figure(nfig)
    plot(x,lambdaP,'k-')
    xlabel('x')
    ylabel('lambda+')
    grid on
    f=['lambda+ at t=' num2str(t)];
    title(f)
    
    nfig=nfig+1;
    figure(nfig)
    plot(x,lambdaM,'k-')
    xlabel('x')
    ylabel('lambda-')
    grid on
    f=['lambda- at t=' num2str(t)];
    title(f)
    
    sum=0;
    maxe=0;
    for n=1:NT
        error(n)=u(n)-uexact(n);
        sum=sum+error(n)^2;     
    end
    maxe=max(abs(error));
    sum=sqrt(sum/NT);
    disp(['time= ' num2str(t) ' L2(u)= ' num2str(sum) ' Linf(u)= '...
        num2str(maxe) ' N= ' num2str(NT-1) ])
    
    nfig=nfig+1;
    figure(nfig)
    plot(x,error,'ko-',xbar,0,'r+')
    xlabel('x')
    ylabel('u-error')
    f=['error at t=' num2str(t)];
    title(f)
    
    nfig=nfig+1;
    figure(nfig)
    for k=1:kout
        for n=1:NT
            uperror(n,k)=1/log(abs(u_error(n,k)));
        end
    end
    [C,h]=contour(x_error,t_error,uperror);
    clabel(C,h);
    colormap(map)
    hold on
    xlabel('x')
    ylabel('t')
    f='error=1/log(|u -uexact|)';
    title(f)
    
    for m=1:101
        tp(m)=(m-1)*t/100;
        bplot(m)=pistonlaw(tp(m));
    end
    xp1=[0 xbar];
    tp1=[0 t];
    plot(bplot,tp,'b-',xp1,tp1,'b-')
    hold off
    
end

