% Q1D.M - Main code for quasi-one-dimensional flows
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%{ 
 NEW PROBLEM SET UP GUIDELINES:
 IN "Q1D.M":
    add a case number to the options of the input parameter INIT.
    add a corresponding case number to the "switch" function to defines 
    tlast, CN, N, etc.
    add output line describing this case.
 IN "PISTONLAW.M", "PISTONLAWT.M", AND "PISTONLAWTT.M":
    define piston law as a function of time, and its first and second 
    derivative.
 IN "EXITBOUNDARY.M", EXITBOUNDARYT.M" AND EXITBOUDARYTT.M":
    define exit boundary law as a function of time, and its first and 
    second derivative.
 IN "LAMBDA_Q1D.M":
    if you want pressure contour plots, add a case number to line 74.
    set up appropriate boundary conditions in functions "BCATNEQUAL1" and
    "BCATNEQUALNT" within "LAMBDA_Q1D.M", see examples.
 IN "INITIALPISTON.M":
    write initial conditions corresponding to this INIT case, see examples.
 IN "PLOT_SOLUTIONPISTON.M":
    if you want pressure contours plots, add case number INIT to line 97.
 IN "AREA_F.M", "AREAX_F.M", "AREAT_F.M" define the area variation as a 
    function of x and t, see examples.
%} 
% Copyright 2005-2009 M.D.Salas.

clear all

global kout N INIT CN tlast LAST IDETECT kplot shift t event_count
global area_index over_pressure osci_eps pulse_loc SAFETY bc_type
global tout X xout NT kmaxout sg x1mach x2mach typeout pressure movie

%{
%         USER PROVIDED INPUTS:
% tlast: runs to a max time t=tlast
% CN = Courant Number
% N = number of mesh points -1
% kplot= steps between plot of shock paths (typical value=1)
% IDETECT= 1 to detect shock formation, 0 otherwise
% SAFETY= maximum number of steps, used to prevent over-run condition if
% dt->0
% movie = 0, no movie, movie = 1 save data in "temporary.mat" for making
% movie. To see movie, run "movie_maker.m". not all cases can be used to
% make movies.

%         THE FOLLOWING DEPEND ON THE CASE:
% bc_type(1)= boundary condition type left end; bc_type(2)= boundary 
% condtion type right end. Required if shock interacts with boundary.
% bc_type=0 solid wall boundary; bc_type=1 given pressure boundary
% condition, default is 0.
% shift= time delay in starting c(t) piston, used in cases 15 & 16.
% area_index= index to select area=area(x,t), default is 0 -> area=1.
% over_pressure= rise in exit pressure, used in case 22 
% osci_eps= oscillation magnitude, used in case 23
% pulse_loc= location of pulse, used in case 26
%}
                                                                            % a file "piston_diary" will be created to record input & outputs for this run
diary('piston_diary')                                                       % results from multiple runs are appended to the file
str = date; clock1=clock; H12=0; AM=' AM';
if clock1(4) > 12
    H12=12; AM=' PM';   
end

disp( ['A SHOCK-FITTING PRIMER ' ,str, ' ' , num2str(clock1(4)-H12) , ':'...
    , num2str(clock1(5)) , ':' , num2str(round(clock1(6))), AM])
disp(' ')
disp('           UNIT PROBLEMS:')
disp('INIT= 1 b(t)=t^3, c(t)=.05+sqrt(gamma)*t')
disp('INIT= 2 single shock moving right')
disp('INIT= 3 single shock moving left')
disp('INIT= 4 b(t)=0.95-sqrt(gamma)*t, c(t)=1-t^3')
disp('INIT= 5 two shocks colliding')
disp('INIT= 6 two l-shocks coalescing')
disp('INIT= 7 two r-shocks coalescing')
disp('INIT= 8 l-shock interacts with contact, resulting in reflected shock')
disp('INIT= 8.5 l-shock interacts with contact, resulting in reflected expansion')
disp('INIT= 9 r-shock interacts with contact, resulting in reflected shock')
disp('INIT= 9.5 r-shock interacts with contact, resulting in reflected expansion')
disp('INIT=10 l-shock reflection from c-wall')
disp('INIT=11 r-shock reflection from b-wall')
disp('           MORE COMPLEX PROBLEMS:')
disp('INIT=12 tests shock detection and reflection; b(t)=1-cos(pi*t/2), c(t)=2')
disp('INIT=13 mirror of case 12')
disp('INIT=14 cases 12 and 13 combined')
disp('INIT=15 asymmetric version of case 14, involves opposite-type shock interactions')
disp('INIT=16 another asymmetric version of case 14, involves same-type shock interactions')
disp('           VARIABLE AREA PROBLEMS:')
disp('INIT=17 shock moving right in variable area duct, contracting area')
disp('INIT=18 shock moving right in variable area duct, expanding area')
disp('INIT=19 Laval nozzle, no shocks')
disp('INIT=20 Laval nozzle, with shock')
disp('INIT=21 Laval nozzle start (this case runs for approximately 5400 steps)')
disp('INIT=22 Laval nozzle surge; explore effect of "over_pressure"')
disp('           TIME-OSCILLATING AREA PROBLEMS:')
disp('INIT=23 Oscillating Laval nozzle: area=x+1/(x+e*sin(t)^2)')
disp('INIT=24 Oscillating Laval nozzle: area=x+e*sin(t)+1/x, approximately 6000 steps')
disp('INIT=25 Oscillating Laval nozzle: area=x(1+e*sin(t))+1/x')
disp('INIT=26 Oscillating Laval nozzle: area=x+1/x+exp(-(x-1)^2)*sin(t)')
disp('           DOUBLE THROAT NOZZLE PROBLEMS:')
disp('INIT=27 Double throat nozzle, initial solution perturbed')
disp('INIT=28 Double throat nozzle, nozzle start')
disp(' ');

disp('In the next line enter the case to be executed with default values')
INIT=input('Enter INIT [1 through 28]: ');
disp(' ')

area_index=0;
bc_type=[0 0];                                                              % 0 -> solid wall boundary conditon at both ends is default
SAFETY=2000;                                                                % stops calculation if over run condition
movie=0;                                                                    % set to 1 to save data for movie
kplot=1;

switch (INIT)
    case {1, 4}
        tlast=.8;                                                            % NOTE: shock forms at t=0.8439 for b(t)=t^3
        CN=.7;
        N=50;
        IDETECT=1;
        if INIT == 4
            movie=0;                                                        % no movie for INIT=4
        end
    case {2, 3}
        tlast=.2;
        CN=.7;
        N=50;
        IDETECT=0;
        movie=0;                                                            % no movie for INIT=2, 3
    case {5, 6, 7}
        tlast=.2;
        CN=.7;
        N=50;
        IDETECT=0;
        movie=0;                                                            % no movie for INIT=5, 6, 7
    case {10, 11}
        tlast=.7;
        CN=.7;
        N=50;
        IDETECT=0;
        movie=0;                                                            % no movie for INIT=10, 11
    case {8, 8.5}
        tlast=.3;
        CN=.7;
        N=50;
        IDETECT=0;
        movie=0;                                                            % no movie for INIT=8, 8.5
    case {9, 9.5}
        tlast=.3;
        CN=.7;
        N=50;
        IDETECT=0;
        moview=0;                                                           % no movie for INIT=9, 9.5
    case 12
        tlast=2.5;
        CN=.7;
        N=50;
        IDETECT=1;
        movie=0;                                                            % no movie for INIT=12
    case 13
        tlast=2.5;                                                           
        CN=.7;
        N=50;
        IDETECT=1;
        movie=0;                                                            % no movie for INIT=13
    case 14
        tlast=10;
        CN=.7;
        N=100;
        kplot=1;
        IDETECT=1;
    case 15
        tlast=10;
        CN=.7;
        N=100;
        shift=.5;
        IDETECT=1;
    case 16
        tlast=10;
        CN=.7;
        N=100;
        shift=1.5;
        IDETECT=1;
    case 17                                                                 % region of ambiguity for shock interaction with abrupt area change
        tlast=.8;
        CN=.6;
        N=100;
        IDETECT=1;                                                          % "area_index" used in area_f, areax_f, and areat_f to select case area variation
        area_index=2;                                                       % area_index=2 -> contracting area
    case 18
        tlast=2;
        CN=.6;
        N=100;
        IDETECT=1;
        area_index=3;                                                       % area_index=3 -> expanding area
    case 19
        tlast=1;
        CN=.7;
        N=100;
        IDETECT=0;
        area_index=4;
        movie=0;
    case 20                                                                 % area_index=4 -> area=x+1/x
        tlast=1;
        CN=.7;
        N=100;
        IDETECT=0;
        area_index=4;
        bc_type(2)=1;
        movie=0;
    case 21                                                                 % area_index=4 -> area=x+1/x
        tlast=35;
        CN=.7;
        N=100; 
        IDETECT=1;
        area_index=4;
        bc_type(2)=1;
        SAFETY=7000;
    case 22                                                                 % area_index=4 -> area=x+1/x
        tlast=20;
        CN=.7;
        N=100;
        kplot=5;
        IDETECT=1;
        bc_type(2)=1;
        area_index=4;                                                       % "over_pressure" used in "function bcatnequalNT"
        over_pressure=0.350;                                                % explore the effect of "over_pressure" in the range [0.300, 0.350]
                                                                            % explore the effect of "over_pressure" in the range [-0.1, -0.4]
        %{
       % try the following (about 3500 steps each):
       %over_pressure=0.325;
       %over_pressure=0.335;
       %over_pressure=0.339;
       %over_pressure=0.33905;
       %over_pressure=0.340;
       %over_pressure=0.350;
        %}
        SAFETY=6000;
    case 23                                                                 % area_index=5 -> area=x+1/(x+osci_eps*sin(t)^2)
        tlast=15;
        CN=.7;
        N=100;
        kplot=5;
        IDETECT=1;
        area_index=5;
        SAFETY=6000;
        bc_type(2)=1;
        osci_eps=0.2;                                                       % explore osci_eps [0.12, 0.2]
    case 24                                                                 % area_index=6 -> area=x+e*sin(t)+1/x
        tlast=25;
        CN=.7;
        N=100;
        kplot=5;
        IDETECT=1;
        SAFETY=6000;
        bc_type(2)=1;
        area_index=6;        
    case 25                                                                 % area_index=7 -> area=x(1+e*sin(t))+1/x
        tlast=20;
        CN=.7;
        N=100;
        kplot=5;
        SAFETY=6000;
        bc_type(2)=1;
        IDETECT=1;
        area_index=7;        
   case 26                                                                  % area_index=8 -> area=x+1/x+e*exp(-(x-pulse_loc)^2)*sin(t)
        tlast=25;
        CN=.7;
        N=100;
        kplot=5;
        IDETECT=1;
        bc_type(2)=1;
        area_index=8;
        SAFETY=6000;
        pulse_loc= 1.0;                                                     % explore "pulse_loc" at [0.75, 1, 1.25, 1.5, 1.75]
   case 27                                                                  % two throats, initial steady solution perturbed                                                             
        tlast=30;
        CN=.6;
        N=400;
        kplot=5;
        IDETECT=1;
        area_index=9;
        bc_type(2)=1;
        SAFETY=10000;
        over_pressure=.2;
    case 28                                                                 % two throats, nozzle start
        CN=.7;
        IDETECT=1;
        N=150;
        tlast=70;
        kplot=7;
        area_index=1;
        bc_type(2)=1;
        SAFETY=20000;
    otherwise
        disp('INIT not in range, execution aborted')
        tlast=0;
        INIT=1;
        SAFETY=1;
end

%               NO CHANGES BELOW THIS LINE
%
%               INITIAL CONDITIONS
event_count=0;                                                              % counter to keep tract of major events
initialpiston                                                               % define initial conditions

%               OUTPUT
if movie == 0
    plot_solutionpiston(1)                                                  % plot initial profiles
end
%               MAIN INTEGRATION LOOP

LAST=0;                                                                     % trigger to stop computation in "mac_non_con_piston"
kout=1;                                                                     % time step counter

lambda_Q1D                                                                  % call integration routine

%----------------movie info-----------------
if movie == 1
    % SAVES DATA ON FILE TEMPORARY.MAT FOR USE WITH MOVIE_MAKER.M
    save temporary.mat tout X area_index xout INIT NT osci_eps pulse_loc...
        kmaxout shift sg x1mach x2mach typeout pressure
    disp(' ')
    disp('movie data file saved in "temporary.mat"')
end
%----------------movie info-----------------

%               OUTPUT

disp(' ')
disp('Output from Q1D:')
disp(' ')
disp(['N= ' num2str(N)])
disp(['Inital conditions set for INIT= ' num2str(INIT)])
switch (INIT)
    case 1
        disp('Corresponding to b(t)=t^3, c(t)=.05+sqrt(gamma)*t ')
    case 2
        disp('Corresponding to single shock moving right')
    case 3
        disp('Corresponding to single shock moving left ')
    case 4
        disp('Corresponding to b(t)=0.95-sqrt(gamma)*t, c(t)=1-t^3')
    case 5
        disp('Corresponding to two shocks colliding')
    case 6
        disp('Corresponding to two l-shocks coalescing')
    case 7
        disp('Corresponding to two r-shocks coalescing')
    case 8
        disp('Corresponding to l-shock interaction with contact, resulting in reflected shock')
    case 8.5
        disp('Corresponding to l-shock interaction with contact, resulting in reflected expansion')
    case 9
        disp('Corresponding to r-shock interaction with contact, resulting in reflected shock')
    case 10
        disp('Corresponding to l-shock reflection from c-wall')
    case 11
        disp('Corresponding to r-shock reflection from b-wall')
    case 12
        disp('Corresponding to b(t)=1-cos(pi*t/2), c(t)=2)')
    case 13
        disp('Corresponding to mirror of case 12; c(t)=2-{1-cos(pi*t/2)}, b(t)=0')
    case 14
        disp('Corresponding to cases 12 and 13 combined')
    case 15
        disp('Corresponding to axisymmetric version of case 14')
    case 16
        disp('Corresponding to axisymmetric version of case 14')
    case 17
        disp('Corresponding to shock moving right in variable area duct, contracting area')
    case 18
        disp('Corresponding to shock moving right in variable area duct, expanding area')
    case 19
        disp('Corresponding to Laval nozzle, no shocks. To show that exact solution is satisfied')
    case 20
        disp('Corresponding to Laval nozzle, nwith shock. To show that exact solution is satisfied')
     case 21
        disp('Corresponding to Laval nozzle start')
     case 22
        disp('Corresponding to Laval nozzle surge')
     case {23, 24, 25, 26}
        disp('Corresponding to Oscillating Laval nozzle')
     case 27
        disp('Corresponding to a double throat, initial solution perturbed')
     case 28
        disp('Corresponding to a double throat, nozzle start')
end
disp(' ')
disp(['After ' num2str(kout) ' time steps, time= ' num2str(t)])

if movie == 0
    plot_solutionpiston(2)                                                  % plot final results
end
diary off                                                                   % close diary

