function area=area_f(x, t, area_index)
% AREA_F.M - defines area as a function of x for quasi-one-dimensional flows
% A SHOCK FITTING PRfMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global osci_eps pulse_loc

switch area_index
    case 0                                                                  % no area variation
        area=1;
    case 1                                                                  % double throat nozzle
        if x <= 1
            area=x+1/x;
        elseif x < 2
            area=2+8*((x-1)*(x-2))^2;
        else
            area=x-1+1/(x-1);
        end
        
        %{
        % problem 4.7(a)
        delta=1;
        if x <= 0.8289
            area=(1/x-1/(x-3))+1.16667*exp(-2.8*(x-1.5)^2)+0.002402690332595;
        elseif (x > 0.8289) && (x < 2.1708)
            area=2+2.479101181284402*((x-0.8289)^2*(x-2.1708)^2);
        elseif x >= 2.1708
            area=delta*(x-1.1708)+1/(x-1.1708)+1-delta;
        end
        %}

        %{
        % problem 4.7(b)
        delta=1;
        area=(1/x-1/(x-3))+1.16667*exp(-2.8*(x-1.5)^2)+0.002402690332595;
        if x >= 2.1708
            area=delta*(x-1.1708)+1/(x-1.1708)+1-delta;
        end
        %}

        %{
        % problem 4.7(c)'
        delta=1;
        if x < 0.8289
            area=(1/x-1/(x-3))+1.16667*exp(-2.8*(x-1.5)^2)+0.002402690332595;
        elseif (x >= 0.8289) && (x < 2.1708)
            area=2+1.1*(x-0.8289)*(2.1708-x);
        elseif x >= 2.1708
            area=delta*(x-1.1708)+1/(x-1.1708)+1-delta;
        end
        %}
        
    case 2                                                                  % contracting area
        L1=1;                                                               % L1 length of straight section
        L2=.1;                                                              % L2 length of contracting section
        al=0.10714285;                                                      % final area=1-al; alpha=1/(1-al)
        if x < L1                                                           %
            area=1;                                                         %___________
        elseif x < L1+L2                                                    %            .
            area=1+al*((cos(pi*(x-L1)/L2)+1)/2-1);                          %              .___________________
        else                                                                %<---L1----><-->
            area=1-al;                                                      %            L2
        end
    case 3                                                                  % expanding area
        L1=1;                                                               % length of straight section
        L2=.5;                                                              % length of expanding section
        al=2;                                                               % final area=1+al
        if x < L1
            area=1;
        elseif x < L1+L2
            area=1-al*((cos(pi*(x-L1)/L2)+1)/2-1);
        else
            area=1+al;                                                      % alpha=1/(1+al)          
        end
    case 4
        area=x+1/x;
    case 5                                                                  % time oscillating area
        t1=4;
        area=x+1/(x+osci_eps*sin(pi*t/t1)^2);
    case 6                                                                  % time oscillating area
        eps1=.5;
        t1=4;
        area=x+eps1*sin(pi*t/t1)+1/x;
    case 7                                                                  % time oscillating area
        eps1=.1;
        t1=4;
        area=x*(1+eps1*sin(pi*t/t1))+1/x;
    case 8
        eps1=.2;
        t1=4;
        f=eps1*exp(-30*(x-pulse_loc)^2)*sin(pi*t/t1);
        area=x+1/x+f;
    case 9
        if x <= 2
            area=x+1/x;
        elseif x <= 4
            area=13-45*x/4+3*x^2;
            if area > 5
                area=5;
            end
        else
            area=x-3.92+1/(x-4);
            if area > 5
                area=5;
            end
        end
end