function areax=areax_f(x, t, area_index)
% AREA.M - defines d(area)/dx as a function of x for quasi-one-dimensional flows
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global osci_eps pulse_loc

switch area_index
    case 0
        areax=0;
    case 1
        if x <= 1
            areax=1-1/x^2;
        elseif x < 2
            areax= 16*((x-1)*(x-2)^2+(x-2)*(x-1)^2);
        else
            areax=1-1/(x-1)^2;
        end
        
        %{
        % problem 4.7 (a)
        delta=1;
        if x <= 0.8289
            areax=(-1/x^2+1/(x-3)^2)-5.6*1.16667*(x-1.5)*exp(-2.8*(x-1.5)^2);
        elseif (x > 0.8289) && (x < 2.1708)
            areax=2*2.479101181284402*((x-0.8289)*(x-2.1708)^2+(x-2.1708)*(x-0.8289)^2);
        elseif x >= 2.1708
            areax=delta-1/(x-1.1708)^2;
        end
        %}

        %{
        % problem 4.7(b)
        delta=1;
        areax=(-1/x^2+1/(x-3)^2)-5.6*1.16667*(x-1.5)*exp(-2.8*(x-1.5)^2);
        if x >= 2.1708
             areax=delta-1/(x-1.1708)^2;
        end
        %}

        %{
        % problem 4.7 (c)
        delta=1;
        if x < 0.8289
            areax=(-1/x^2+1/(x-3)^2)-5.6*1.16667*(x-1.5)*exp(-2.8*(x-1.5)^2);
        elseif (x > 0.8289) && (x < 2.1708)
            areax=2*1.1*((2.1708-x)-(x-0.8289));
        elseif x >= 2.1708
            areax=delta-1/(x-1.1708)^2;
        end
        %}
        
    case 2
        L1=1;
        L2=.1;
        alpha=0.1071;
        if x < L1
            areax=0;
        elseif x < L1+L2
            areax=-pi*alpha*sin(pi*(x-L1)/L2)/(4*L2);
        else
            areax=0;
        end
    case 3
        L1=1;
        L2=.5;
        alpha=2;
        if x < L1
            areax=0;
        elseif x < L1+L2
            areax=pi*alpha*sin(pi*(x-L1)/L2)/(4*L2);
        else
            areax=0;
        end
    case {4, 6}                                                             % case 6: area=x+eps1*sin(pi*t/t1)+1/x;
       areax=1-1/x^2; 
    case 5
        t1=4;
        areax=1-1/(x+osci_eps*sin(pi*t/t1)^2)^2;
    case 7                                                                  % case 7: area=x*(1+eps1*sin(pi*t/t1))+1/x;
        eps1=.1;
        t1=4;
        areax=1+eps1*sin(pi*t/t1)-1/x^2;
    case 8
        eps1=.2;
        t1=4;
        f=eps1*exp(-30*(x-pulse_loc)^2)*sin(pi*t/t1);
        fx=-60*f*(x-pulse_loc);
        areax=1-1/x^2+fx;
    case 9
        if x <= 2
            areax=1-1/x^2;
        elseif x <= 4
            areax=-45/4+6*x;
            area=13-45*x/4+3*x^2;
            if area > 5
                areax=0;
            end
        else
            area=x-3.92+1/(x-4);
            areax=1-1/(x-4)^2;
            if area > 5
                areax=0;
            end
        end
        
end