function exact
% EXACT.M - exact solution to piston flow before shock formation
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% works for b(t)=t^3
% x & t given, find tau, u(x,t)=bt(tau)
% Copyright 2005-2008 M.D.Salas.

global t gamma uexact NT X Pexact sg INIT Sexact Rexact Qexact gp gm

xbar=sg*t;                                                                  % location of first signal at t
tol=10^-7;

b=pistonlaw(t);
c=exitboundary(t);

if INIT == 4
  b=1-exitboundary(t);
  c=1-pistonlaw(t);
end

xe=b+(c-b)*X;

tau=t;
for n=1:NT
    if xe(n) < xbar
        for m=1:20
            %__________________________________________ piston law
            B=tau^3;
            Bt=3*tau^2;
            Btt=6*tau;
            %__________________________________________
            F=xe(n)-(B+(sg+gp*Bt/2)*(t-tau));                               % Newton iteration
            Ftau=-(Bt+gp*Btt*(t-tau)/2-(sg+gp*Bt/2));
            taun=tau-F/Ftau;
            err=abs(taun-tau);
            if err < tol
                break
            end
            tau=taun;
        end
        if err > tol
            disp(['In exact.m, function exact: err= ' num2str(err) ' n= ' num2str(n)])
        end
        %____________________________
        uexact(n)=3*tau^2;
        %____________________________
    else
        uexact(n)=0;
    end
    ae=sg+gm*uexact(n)/2;
    Ze=ae^2/gamma;
    Pexact(n)=gamma*log(Ze)/gm;
end

if INIT == 4  
    usave=uexact;
    Psave=Pexact;
    for n=1:NT
        m=n-1;
        uexact(n)=-usave(NT-m);
        Pexact(n)=Psave(NT-m);
    end
end
Sexact=zeros(1,NT);
Zexact=exp((gm*Pexact)/gamma);
aexact=sqrt(gamma*Zexact);
Rexact=2*aexact/gm+uexact;
Qexact=2*aexact/gm-uexact;