function exact_s
% EXACT_S.M - approximate solution to piston flow after shock formation
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% x & t given, find tau, u(x,t)=bt(tau)
% Copyright 2005-2008 M.D.Salas.

global t gamma uexact NT x X Pexact x_s gp gm sg

xbar=sg*t;                                                                  % location of first signal at t
b=pistonlaw(t);
c=exitboundary(t);

x=b+(c-b)*X; 

tau=t;
for n=1:NT
    if x(n) < x_s(1)                                                        % find solution on left side of shock
        for m=1:20
            B=pistonlaw(tau);
            Bt=pistonlawt(tau);
            Btt=pistonlawtt(tau);
            F=x(n)-(B+(sg+gp*Bt/2)*(t-tau));                                % Newton iteration
            Ftau=-(Bt+gp*Btt*(t-tau)/2-(sg+gp*Bt/2));
            taun=tau-F/Ftau;
            err=abs(taun-tau);
            if err < 10^-7
                break
            end
            tau=taun;
        end
        if err > 10^-7
            disp(['In "exact_s.m" err= ' num2str(err) ' n= ' num2str(n)])
        end
        uexact(n)=pistonlawt(tau);
    else
        uexact(n)=0;
    end
    ae=sg+gm*uexact(n)/2;
    Ze=ae^2/gamma;
    Pexact(n)=gamma*log(Ze)/gm;
end

tau=0;
for nn=1:NT
    n=NT+1-nn;
    if x_s(1) < x(n) <= xbar                                                % find solution to right of shock and left of first signal
        for m=1:20
            B=pistonlaw(tau);
            Bt=pistonlawt(tau);
            Btt=pistonlawtt(tau);
            F=x(n)-(B+(sg+gp*Bt/2)*(t-tau));                                % Newton iteration
            Ftau=-(Bt+gp*Btt*(t-tau)/2-(sg+gp*Bt/2));
            taun=tau-F/Ftau;
            err=abs(taun-tau);
            if err < 10^-7
                break
            end
            tau=taun;
        end
        if err > 10^-7
            disp(['In exact_s.m err= ' num2str(err) ' n= ' num2str(n)])
        end
        uexact(n)=pistonlawt(tau);
    end
    ae=sg+gm*uexact(n)/2;
    Ze=ae^2/gamma;
    Pexact(n)=gamma*log(Ze)/gm;
end