function find_shk_xing
% FIND_SHK_XING.M - detects shock crossing and resets dt
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global eps0 dt FLAG kmax ICROSS kcross x_s w  

FLAG=0;                                                                     % trigger for shock crossing
for k=1:kmax-1                                                              % check for shock crossing, reset dt if etest < 1
    ICROSS(k)=1/eps0;                                                       % set ICROSS to a large value
    if w(k) > w(k+1)                                                        % no crossing if w(k-1) < w(k)
        etest=(x_s(k)-x_s(k+1))/(w(k+1)-w(k))/dt;                           % etest=(t*-t)/dt, number of dt's to crossing
        if (etest > 0) && (etest <= 1.1)
            ICROSS(k)=min([etest, 1]);                                      % set trigger on
            FLAG=1;
        end
    end
end

if FLAG == 1                                                                % kcross (the lower k index of two shocks crossing) is defined here
    [newdt, kcross]=min(ICROSS);                                            % of all possible crossings, in next t interval, choose the one occuring first
    dt=newdt*dt;                                                            % reset dt
end

