function lambda_Q1D
% LAMBDA_Q1D.M - solver based on Lambda Scheme predictor
% -corrector method
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global kout dx dt IDETECT x kplot tout xout w kmax_save x_s xo_s kcross
global U t tlast LAST N CN kmax FLAG WALL I IS RnR RnL QnL QnR Rn Qn 
global u P S So Sn gamma a Z X Xt Xx X_s NT INIT RoL RoR QoL QoR LAt
global uL uR PL PR SL SR ZL ZR aL aR SoR SoL eps0 R Q Ro Qo wo kout_plot 
global XC YC ZC wn unR PnR SnR unL PnL SnL xn_s RR RL QR QL LAx area_index
global t1mach x1mach typeout shock_type SAFETY psaveR psaveL gm
global kmaxout x2mach pressure movie  
                                                   
ggm=gamma*gm;
WALL=[0 0];                                                                 % trigger for wall reflection
FLAG=0;                                                                     % trigger for shock crossing, set to 1 in function find_shk_xing                                                                    % trigger for shock reflection from wall, set to 1 in function find_wall_xing
kc=0;                                                                       % countour plot counter
km=0;                                                                       % Mach line counter for output purpose
kout_plot=0;                                                                % shock path plot counter
YC(:,1)=zeros(NT,1);                                                        % saves time for plotting

while LAST == 0                                                             % begin time stepping loop

    kout=kout+1;                                                            % step counter
    
    if kout > SAFETY                                                        % stop calculation if out-of-control condition
        LAST=1;
        print_event
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value, function lambda_Q1D')
    end

    b=pistonlaw(t);
    bt=pistonlawt(t);
    c=exitboundary(t);
    ct=exitboundaryt(t);
    Xx=1/(c-b);

    Abs=abs(U)+a*Xx;                                                        % CFL condition
    Amax=max(Abs);
    dt=CN*dx/Amax;
    if (t+dt) >= tlast                                                      % check that time does not exceed tlast
        dt=tlast-t;
        LAST=1;                                                             % set trigger to stop while loop
    end

    sigma=dt/dx;
    
    if kmax > 0                                                             % check for shock reflection from wall
        find_wall_xing
        shock_elimination                                                   % check strength of shocks, eliminate weak shocks
        X_s=Xx*(x_s-b);                                                     % if shocks, create computational coordinate of shock
        if kmax >= 2                                                        % check for shock crossing, reset dt if etest < 1
            dtold=dt;
            find_shk_xing
            dt=min(dt,dtold);
        end
        kmax_save=max(kmax, kmax_save);                                     % kmax_save is used for plotting shock paths in "plot_solutionpiston" routine
    end   

    test=rem(kout,kplot);
    if (movie == 0) && (test == 0)
        if (INIT == 1) || (INIT == 4) || (INIT >= 12)                       % save values to create contour plots
            kc=kc+1;
            XC(:,kc)=X/Xx+b;
            YC(:,kc)=t;
            ZC(:,kc)=P;
        end
    end
    
    for j=1:2                                                               % begin predictor-corrector loop
        
        Xt=-Xx*(bt+X*(ct-bt));
        U=Xt+u*Xx;
        A=(U-abs(U))/2;                                                     % A, B are either +-U or zero
        B=(U+abs(U))/2;
        lambdaP=U+Xx*a;
        lambdaM=U-Xx*a;
        lPm=(lambdaP-abs(lambdaP))/2;                                       % the following functions are either +-lambda or zero
        lPp=(lambdaP+abs(lambdaP))/2;
        lMm=(lambdaM-abs(lambdaM))/2;
        lMp=(lambdaM+abs(lambdaM))/2;                                 
        area_term=a.*(u.*LAx+LAt)*dx;                                       % Note: multiply by dx because sigma devides by dx
        
        for n=2:N                                                           % sweep through mesh points
            
            Sxp=S(n+1)-S(n);
            Rxp=R(n+1)-R(n);
            Qxp=Q(n+1)-Q(n);
            Sxm=S(n)-S(n-1);
            Rxm=R(n)-R(n-1);
            Qxm=Q(n)-Q(n-1);
                      
            if I(n,1) ~= 0
                Sxp=Sxm;
                Rxp=Rxm;
                Qxp=Qxm;
            elseif I(n-1,1)~= 0
                Sxm=Sxp;
                Rxm=Rxp;
                Qxm=Qxp;
            end

            if j == 2
                if (n ~= N) && ((I(n,1)+I(n+1,1)) == 0)
                    Sxp=-(2*S(n)-3*S(n+1)+S(n+2));
                    Rxp=-(2*R(n)-3*R(n+1)+R(n+2));
                    Qxp=-(2*Q(n)-3*Q(n+1)+Q(n+2));
                end
                if (n ~= 2) && ((I(n-2,1)+I(n-1,1)) == 0)
                    Sxm=2*S(n)-3*S(n-1)+S(n-2);
                    Rxm=2*R(n)-3*R(n-1)+R(n-2);
                    Qxm=2*Q(n)-3*Q(n-1)+Q(n-2);
                end
            end        
            
            if kmax > 0                                                     % compute Rx, Qx, Sx near shock
                
                if (j == 1) && (I(n,1) ~= 0)                                % true if predictor stage & shock present in cell [n,n+1]

                    if I(n-1,1) == 0                                        % true if no shock in cell [n-1,n]
                        k=I(n,1);                                           % use shock index to right of n
                        eps1=(X_s(k)-X(n))/dx;
                        Sxp=((eps1-2)*S(n-1)+SL(k)+(1-eps1)*S(n))/2;
                        Rxp=((eps1-2)*R(n-1)+RL(k)+(1-eps1)*R(n))/2;
                        Qxp=((eps1-2)*Q(n-1)+QL(k)+(1-eps1)*Q(n))/2;
                    else                                                    % shock in cells [n-1,n] & [n,n+1]
                        k=I(n,1);
                        del=dx/(X_s(k)-X_s(k-1));
                        Sxm=(SL(k)-SR(k-1))*del;                            
                        Sxp=Sxm;
                        Rxm=(RL(k)-RR(k-1))*del;
                        Rxp=Rxm;
                        Qxm=(QL(k)-QR(k-1))*del;
                        Qxp=Qxm;
                    end

                elseif (j == 2) && (I(n-1,1) ~= 0)                          % true if corrector stage & shock present in cell [n-1,n]

                    if I(n,1) == 0                                          % true if no shock in cell [n,n+1]
                        k=I(n-1,2);                                         % use shock index to left of n
                        eps1=(X(n)-X_s(k))/dx;
                        Sxm=-((eps1-2)*S(n+1)+SR(k)+(1-eps1)*S(n))/2;
                        Rxm=-((eps1-2)*R(n+1)+RR(k)+(1-eps1)*R(n))/2;
                        Qxm=-((eps1-2)*Q(n+1)+QR(k)+(1-eps1)*Q(n))/2;
                    else                                                    % shock in cells [n-1,n] & [n,n+1]
                        k=I(n,1);
                        del=dx/(X_s(k)-X_s(k-1));
                        Sxp=(SL(k)-SR(k-1))*del;
                        Sxm=Sxp;
                        Rxp=(RL(k)-RR(k-1))*del;
                        Rxm=Rxp;
                        Qxp=(QL(k)-QR(k-1))*del;
                        Qxm=Qxp;
                    end
                end
            end                                                             % end computation of Rx, Qx, Sx near shock

            St=-(A(n)*Sxp+B(n)*Sxm);
            aggm=a(n)/ggm;
            Rt=-(lPm(n)*(Rxp-aggm*Sxp)+lPp(n)*(Rxm-aggm*Sxm))...
                +aggm*St-area_term(n);
            Qt=-(lMm(n)*(Qxp-aggm*Sxp)+lMp(n)*(Qxm-aggm*Sxm))...
                +aggm*St-area_term(n);
            
            Sn(n)=S(n)+sigma*St;
            Rn(n)=R(n)+sigma*Rt;
            Qn(n)=Q(n)+sigma*Qt;
            
            if j == 2
                Sn(n)=.5*(So(n)+Sn(n));
                Rn(n)=.5*(Ro(n)+Rn(n));
                Qn(n)=.5*(Qo(n)+Qn(n));
            end
        end                                                                 % end sweep through mesh points loop

        bcatnequal1(j,sigma)                                                % apply boundary conditions at n=1
        bcatnequalNT(j,sigma)                                               % apply boundary conditions at n=NT

        if kmax > 0                                                         % BEGIN SHOCK COMPUTATION; compute Rx, Qx, Sx at shock, left & right
            for k=1:kmax                                                    % sweep through shocks

                n=IS(k);                                                    % set mesh point to left of k-th shock
                if n > 2                                                    % COMPUTE LEFT SIDE DERIVATIVES AT DISCONTINUITY
                    [RxL, QxL, SxL] = fxL1(k,n);
                elseif n > 1
                    [RxL, QxL, SxL] = fxL2(k,n);
                else
                    RxL=0;
                    QxL=0;
                    SxL=0;
                end

                if n < N-1                                                  % COMPUTE RIGHT SIDE DERIVATIVES AT DISCONTINUITY
                    [RxR, QxR, SxR] = fxR1(k,n);
                elseif n < N
                    [RxR, QxR, SxR] = fxR2(k,n);
                else
                    RxR=0;
                    QxR=0;
                    SxR=0;
                end

                discontinuity(k,j,RxL,QxL,SxL,RxR,QxR,SxR)                  % compute discontinuities

            end
        end                                                                 % end all discontinuity loop

        if kmax > 0                                                         % save & update shock values
            xo_s=x_s;
            wo=w;
            SoR=SR;
            SoL=SL;
            QoL=QL;
            QoR=QR;
            RoL=RL;
            RoR=RR;
            x_s=xn_s;
            w=wn;
            uL=unL;
            uR=unR;
            PL=PnL;
            PR=PnR;
            SL=SnL;
            SR=SnR;            
            RL=RnL;
            QL=QnL;
            RR=RnR;
            QR=QnR;
            aL=gm*(RL+QL)/4;
            aR=gm*(RR+QR)/4;
            ZL=aL.^2/gamma;
            ZR=aR.^2/gamma;
        end
        
        So=S;
        Ro=R;                                                              
        Qo=Q;      
        R=Rn;
        Q=Qn;
        S=Sn;               
        u=(R-Q)/2;
        a=gm*(R+Q)/4;
        Z=a.^2/gamma;
        P=(gamma*log(Z)-S)/gm;       
    end                                                                     % end predictor-corrector loop
    t=t+dt;                                                                 % increment time 
    
    b=pistonlaw(t);
    c=exitboundary(t);
    x=b+(c-b)*X;
    for n=1:NT
        area=area_f(x(n), t, area_index);
        LAx(n)=areax_f(x(n), t, area_index)/area;                           % define area terms
        LAt(n)=areat_f(x(n), t, area_index)/area;
    end
    
    if kmax > 0
        mesh_xing                                                           % update mesh point crossed by a shock
        if (WALL(1) == 1) || (WALL(2) == 1)                                 % true if shock reflection from either end
            shock_wall
        end
        if FLAG == 1                                                        % true if shock crossing occurs; trigger set in find_shk_xing
            x_s(kcross)=x_s(kcross+1)-eps0;                                 % this is to avoid shock crossing before crossing is done
            shock_cross
            FLAG=0;
        end
    end
    if IDETECT == 1                                                         % shock detection
        detect_shock
    end
    
    if movie == 0                                                           % no movie
        test=rem(kout,kplot);                                               % save values of shock locations  and shock pressure for plotting every kplot steps
        if (test == 0) && (kmax > 0)
            kout_plot=kout_plot+1;
            for k=1:kmax
                psaveR(k,kout_plot)=exp(PR(k));
                psaveL(k,kout_plot)=exp(PL(k));
                tout(k,kout_plot)=t;
                xout(k,kout_plot)=x_s(k);
                typeout(k,kout_plot)=shock_type(k);
            end
        end

        if INIT >= 21                                                       % saving sonic line for plotting
            mach=u./a-1;
            for n=2:N
                if mach(n)*mach(n+1) < 0
                    if I(n,1) ~= 0                                          % skip the shock
                        continue
                    end
                    km=km+1;
                    x1mach(km)=(-x(n)*mach(n+1)+x(n+1)*mach(n))...
                        /(mach(n)-mach(n+1));
                    t1mach(km)=t;
                end
            end
        end

    else                                                                    % movie = 1
        %----------------start movie info-----------------
        kout_plot=kout_plot+1;
        tout(kout_plot)=t;
        kmaxout(kout_plot)=0;
        pressure(:,kout_plot)=exp(P(:));
        if kmax > 0
            kmaxout(kout_plot)=kmax;
            for k=1:kmax
                xout(k,kout_plot)=x_s(k);
                typeout(k,kout_plot)=shock_type(k);
            end
        end
        if INIT >= 17                                                       % saving sonic line for plotting
            x1mach(kout_plot)=-10;                                          % set to a value outside plot window
            x2mach(kout_plot)=-10;
            mach=u./a-1;
            ncou=0;
            for n=2:N
                if mach(n)*mach(n+1) < 0
                    if I(n,1) ~= 0                                          % skip the shock
                        continue
                    elseif ncou == 0
                        x1mach(kout_plot)=(-x(n)*mach(n+1)+x(n+1)*mach(n))...
                            /(mach(n)-mach(n+1));
                        ncou=1;
                    elseif ncou == 1
                        x2mach(kout_plot)=(-x(n)*mach(n+1)+x(n+1)*mach(n))...
                            /(mach(n)-mach(n+1));
                    else
                        break
                    end
                end
            end
        end
        %----------------end movie info-----------------
    end
end                                                                         % end time stepping "while" loop

function discontinuity(k,j,RxL,QxL,SxL,RxR,QxR,SxR)
% DISCONTINUITY.M - computes shock, contact and gradient discontinuities
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.
  
global x_s xo_s xn_s dt w shock_type gamma Xx wn unR PnR SnR unL PnL SnL 
global uL uR PL PR SL SR aL aR SoR SoL wo t gm gp
global IS I RnL RnR QnL QnR RoR RoL QoL QoR RR RL QR QL area_index

ggm=gamma*gm;

pR=exp(PR(k));                                                              % other notation used L=side 1, R=side 2
pL=exp(PL(k));
Xxs=Xx;
Xts=-Xxs*w(k);
URs=Xts+uR(k)*Xxs;
ULs=Xts+uL(k)*Xxs;
MRrel=(uR(k)-w(k))/aR(k);
MLrel=(uL(k)-w(k))/aL(k);
area=area_f(x_s(k), t, area_index);
areax=areax_f(x_s(k), t, area_index);
areat=areat_f(x_s(k), t, area_index);
LAxs=areax/area;
LAts=areat/area;
area_termR=aR(k)*(uR(k)*LAxs+LAts);
area_termL=aL(k)*(uL(k)*LAxs+LAts);

switch (shock_type(k))
    
    case 'l'                                                                % R-H L-SHOCK COMPUTATION (HIGH PRESSURE ON LEFT)
        L2p=ULs+Xxs*aL(k);
        L1p=URs+Xxs*aR(k);
        L1m=URs-Xxs*aR(k);
        StR=-URs*SxR;
        RtR=-(L1p*(RxR-aR(k)*SxR/ggm))+aR(k)*StR/ggm-area_termR;
        QtR=-(L1m*(QxR-aR(k)*SxR/ggm))+aR(k)*StR/ggm-area_termR;
        utR=(RtR-QtR)/2;
        atR=gm*(RtR+QtR)/4;
        PtR=(2*atR/aR(k)-StR)/gm;       
        D=4*((pR*aL(k)/(pL*aR(k)))*MRrel-1/MRrel^2)/gp;
        E=MLrel*aL(k)/(MRrel*aR(k));
        A=gamma*(1-E-D);
        axL=gm*(RxL+QxL)/4;
        PxL=(2*axL/aL(k)-SxL)/gm;
        uxL=(RxL-QxL)/2;
        B=-aL(k)*(PtR+L2p*PxL)-area_termL-gamma*(E*utR+L2p*uxL);
        C=-gamma*D*(utR-MRrel*atR);
        wt=(B+C)/A;

            xn_s(k)=x_s(k)+w(k)*dt;                                         % predictor for shock location
            wn(k)=w(k)+wt*dt;                                               % predictor for shock speed
            RnR(k)=RR(k)+RtR*dt;                                            
            QnR(k)=QR(k)+QtR*dt;
            SnR(k)=SR(k)+StR*dt;
        if j == 2
            xn_s(k)=.5*(xo_s(k)+xn_s(k));                                   % corrector for shock location
            wn(k)=.5*(wo(k)+wn(k));                                         % corrector for shock speed
            RnR(k)=.5*(RoR(k)+RnR(k));                                
            QnR(k)=.5*(QoR(k)+QnR(k));
            SnR(k)=.5*(SoR(k)+SnR(k));
        end
        unR(k)=(RnR(k)-QnR(k))/2;
        a1=gm*(RnR(k)+QnR(k))/4;
        Z1=a1^2/gamma;                                                      % Rankine-Hugoniot
        SM1rel=((unR(k)-wn(k))/a1)^2;
        p2p1=(2*gamma*SM1rel-gm)/gp;
        r2r1=gp*SM1rel/(gm*SM1rel+2);
        PnR(k)=(gamma*log(Z1)-SnR(k))/gm;
        p1=exp(PnR(k));
        r1=p1/Z1;
        unL(k)=wn(k)+(unR(k)-wn(k))/r2r1;
        PnL(k)=log(p2p1*p1);
        SnL(k)=PnL(k)-gamma*log(r2r1*r1);
        Z2=exp((SnL(k)+gm*PnL(k))/gamma);
        a2=sqrt(gamma*Z2);
        RnL(k)=2*a2/gm+unL(k);
        QnL(k)=2*a2/gm-unL(k);

    case 'r'                                                                % R-H R-SHOCK COMPUTATION (HIGH PRESSURE ON RIGHT)
        L2m=URs-Xxs*aR(k);
        L1p=ULs+Xxs*aL(k);
        L1m=ULs-Xxs*aL(k);
        StL=-ULs*SxL;
        RtL=-(L1p*(RxL-aL(k)*SxL/ggm))+aL(k)*StL/ggm-area_termL;
        QtL=-(L1m*(QxL-aL(k)*SxL/ggm))+aL(k)*StL/ggm-area_termL;
        utL=(RtL-QtL)/2;
        atL=gm*(RtL+QtL)/4;
        PtL=(2*atL/aL(k)-StL)/gm;
        D=4*((pL*aR(k)/(pR*aL(k)))*MLrel+1/MLrel^2)/gp;
        E=MRrel*aR(k)/(MLrel*aL(k));
        A=gamma*(-1+E-D);
        axR=gm*(RxR+QxR)/4;
        PxR=(2*axR/aL(k)-SxR)/gm;
        uxR=(RxR-QxR)/2;
        B=-aR(k)*(PtL+L2m*PxR)-area_termR+gamma*(E*utL+L2m*uxR);
        C=-gamma*D*(utL-MLrel*atL);
        wt=(B+C)/A;

            xn_s(k)=x_s(k)+w(k)*dt;                                         % predictor for shock location
            wn(k)=w(k)+wt*dt;                                               % predictor for shock speed
            RnL(k)=RL(k)+RtL*dt;                                            % predictor for u, P and S
            QnL(k)=QL(k)+QtL*dt;
            SnL(k)=SL(k)+StL*dt;
        if j == 2
            xn_s(k)=.5*(xo_s(k)+xn_s(k));                                   % corrector for shock location
            wn(k)=.5*(wo(k)+wn(k));                                         % corrector for shock speed
            RnL(k)=.5*(RoL(k)+RnL(k));                                      % corrector for u, P, S
            QnL(k)=.5*(QoL(k)+QnL(k));
            SnL(k)=.5*(SoL(k)+SnL(k));
        end
        unL(k)=(RnL(k)-QnL(k))/2;
        a1=gm*(RnL(k)+QnL(k))/4;
        Z1=a1^2/gamma; 
        SM1rel=((unL(k)-wn(k))/a1)^2;
        p2p1=(2*gamma*SM1rel-gm)/gp;
        r2r1=gp*SM1rel/(gm*SM1rel+2);
        PnL(k)=(gamma*log(Z1)-SnL(k))/gm;
        p1=exp(PnL(k));
        r1=p1/Z1;
        unR(k)=wn(k)+(unL(k)-wn(k))/r2r1;
        PnR(k)=log(p2p1*p1);
        SnR(k)=PnR(k)-gamma*log(r2r1*r1);
        Z2=exp((SnR(k)+gm*PnR(k))/gamma);
        a2=sqrt(gamma*Z2);
        RnR(k)=2*a2/gm+unR(k);
        QnR(k)=2*a2/gm-unR(k);
    
    case 'c'                                                                % CONTACT DISCONTINUITY
        ns=IS(k);
        L1m=URs-Xxs*aR(k);
        L1p=ULs+Xxs*aL(k);
        QtR=-(L1m*(QxR-aR(k)*SxR/ggm))-area_termR;
        RtL=-(L1p*(RxL-aL(k)*SxL/ggm))-area_termL;

            xn_s(k)=x_s(k)+w(k)*dt;                                         % predictor for contact
            RnL(k)=RL(k)+RtL*dt;                                            
            QnR(k)=QR(k)+QtR*dt;
            SnL(k)=SL(k);
            SnR(k)=SR(k);
        if j == 2
            xn_s(k)=.5*(xo_s(k)+xn_s(k));                                   % corrector for contact            
            RnL(k)=.5*(RoL(k)+RnL(k));                                
            QnR(k)=.5*(QoR(k)+QnR(k));
            SnL(k)=SL(k);
            SnR(k)=SR(k);
        end

        if (I(ns,1) == k) && (I(ns,2) == k)
            aLaR=exp((SnL(k)-SnR(k))/(2*gamma));
            aRn=gm*(RnL(k)+QnR(k))/(2*(1+aLaR));
            aLn=aRn*aLaR;
            unL(k)=(RnL(k)-QnR(k))/2+(aRn-aLn)/gm;
            unR(k)=unL(k);
            wn(k)=unL(k);
            PnL(k)=(gamma*log(aLn^2/gamma)-SnL(k))/gm;
            PnR(k)=PnL(k);
            RnR(k)=2*aRn/gm+unR(k);
            QnL(k)=2*aLn/gm-unL(k);

        else
            wn(k)=w(k);
            unL(k)=uL(k);
            unR(k)=unL(k);
            PnL(k)=PL(k);
            PnR(k)=PR(k);
            SnL(k)=SL(k);
            SnR(k)=SR(k);
            RnR(k)=RR(k);
            RnL(k)=RL(k);
            QnR(k)=QR(k);
            QnL(k)=QL(k);
        end  
end
            
function bcatnequal1(j,sigma)
% BCATNEQUAL1.M - boundary condition at n=1
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global t dt R Q u S un Sn INIT Rn Qn gm sg

switch (INIT)
    case 1                                                                  % simple wave, no shock
        un(1)=pistonlawt(t+dt);
        an1=sg+gm*un(1)/2;                                                  % Q constant=2*sg/gm
        Sn(1)=S(1);
        Rn(1)=2*an1/gm+un(1);
        Qn(1)=2*an1/gm-un(1);
    case {19, 20, 21, 22, 23, 24, 25, 26, 27, 28}                           % infinite stagnation reservoir
        un(1)=u(1);
        Sn(1)=S(1);
        Rn(1)=R(1);
        Qn(1)=Q(1);
     case {2, 3, 4, 5, 6, 7, 8, 8.5, 9, 9.5, 10, 17, 18}                    % R constant, evaluate Q
        Rn(1)=R(1);
        Sn(1)=S(1);
        
        find_Q(j,sigma)
        
        un(1)=(Rn(1)-Qn(1))/2;
    case {11, 12, 13, 14, 15, 16}                                           % piston speed given, evaluate Q
        un(1)=pistonlawt(t+dt);
        Sn(1)=S(1);
        
        find_Q(j,sigma)
        
        an1=gm*(Qn(1)+un(1))/2;
        Rn(1)=2*an1/gm+un(1);
end

function bcatnequalNT(j,sigma)
% BCATNEQUALNT.M - boundary condition at n=NT
%
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global U t N kmax IS dt u S un Sn gamma Xx NT INIT Rn Qn R Q Ro a RR LAx
global P dx pexit_final over_pressure LAt gm sg

ggm=gamma*gm;

switch (INIT)
    case {3, 5, 7, 8, 8.5, 9, 9.5, 11, 19}                                  % outflow, Q given
        Qn(NT)=Q(NT);
        
        find_R(j,sigma)
        
        un(NT)=(Rn(NT)-Qn(NT))/2;
    case 4                                                                  % simple wave, no shock
        un(NT)=exitboundaryt(t+dt);
        anNT=sg-gm*un(NT)/2;
        Sn(NT)=S(NT);
        Rn(NT)=2*anNT/gm+un(NT);
        Qn(NT)=2*anNT/gm-un(NT);
    case {10, 12, 13, 14, 15, 16}                                           % piston speed give, evaluate R
        un(NT)=exitboundaryt(t+dt);
        Sn(NT)=S(NT);
        L1p=U(NT)+Xx*a(NT);
        area_term=a(NT)*(u(NT)*LAx(NT)+LAt(NT))*dx;
        if (kmax > 0) && (IS(kmax) == N)
            Rn(NT)=RR(kmax);
        else
            if j == 1
                Rx=(R(NT)-R(N));
                Sx=(S(NT)-S(N));
                Rt=-L1p*(Rx-a(NT)*Sx/ggm)-area_term;
                Rn(NT)=R(NT)+sigma*Rt;
            else
                Rx=(2*R(NT)-3*R(N)+R(N-1));
                Sx=(2*S(NT)-3*S(N)+S(N-1));
                if (kmax > 0) && (IS(kmax) == N-1)
                    Rx=(R(NT)-R(N));
                    Sx=(S(NT)-S(N));
                end
                Rt=-L1p*(Rx-a(NT)*Sx/ggm)-area_term;
                Rn(NT)=.5*(Ro(NT)+R(NT)+sigma*Rt);
            end
        end
        anNT=gm*(Rn(NT)-un(NT))/2;
        Qn(NT)=2*anNT/gm-un(NT);
    case {20, 23, 24, 25, 26}                                               % contant pressure prescribed, evaluate R
        if (kmax > 0) && (IS(kmax) == NT-1)
            kmax=kmax-1;
        end
        Pnew=P(NT);
        
        find_R(j,sigma)
        
        Znew=exp((gm*Pnew+Sn(NT))/gamma);
        anew=sqrt(gamma*Znew);
        un(NT)=Rn(NT)-2*anew/gm;
        Qn(NT)=2*anew/gm-un(NT);
    case {21, 28}                                                           % changing pressure prescribed, evaluate R
        if (kmax > 0) && (IS(kmax) == NT-1)
            kmax=kmax-1;
        end
        pnew=pexit_final;
        Pnew=log(pnew);
        
        %%{
        % smooth start
        t2=1;
        if t < t2
            pnew=(pexit_final+1)/2+...
                (1-pexit_final)*cos(pi*t/t2)/2;
            Pnew=log(pnew);
        end
        %}
        %{
        if (t > 50) && (t < 52)
            pexit_final2=0.72;
            pnew=(pexit_final+pexit_final2)/2+...
                (pexit_final-pexit_final2)*cos(pi*(t-50)/2)/2;
            Pnew=log(pnew);
        end
        if t >= 52
            pnew=pexit_final2;
            Pnew=log(pnew);
        end
        %}
        
        find_R(j,sigma)
        
        Znew=exp((gm*Pnew+Sn(NT))/gamma);
        anew=sqrt(gamma*Znew);
        un(NT)=Rn(NT)-2*anew/gm;
        Qn(NT)=2*anew/gm-un(NT);
    case {22, 27}   
        pnew=pexit_final;                                                   % changing pressure prescribed, evaluate R
        if t < 1
            pnew=pexit_final+over_pressure*sin(t);   
        end
        Pnew=log(pnew);
        
        find_R(j,sigma)
        
        Znew=exp((gm*Pnew+Sn(NT))/gamma);
        anew=sqrt(gamma*Znew);
        un(NT)=Rn(NT)-2*anew/gm;
        Qn(NT)=2*anew/gm-un(NT);
    otherwise
        un(NT)=0.;
        Sn(NT)=0.;
        Rn(NT)=R(NT);
        Qn(NT)=Q(NT);
end

function [RxR, QxR, SxR] = fxR1(k,n)
% fxR1.M - computes derivatives at right side of shock k when n < N-1
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global dx I S X X_s R Q SL SR RR QR RL QL 

if I(n,2) == k                                                              % true is k-shock is last shock in cell [n,n+1]
    if (I(n+1,1) == 0) && (I(n+2,1) == 0)                                   % true if no shock between cells [n+1,n+3]
        epsR=(X(n+1)-X_s(k))/dx;
        c1=2/(1+epsR);
        RxR=-(c1*(RR(k)-R(n+2))+.5*(R(n+3)-R(n+1)))/dx;
        QxR=-(c1*(QR(k)-Q(n+2))+.5*(Q(n+3)-Q(n+1)))/dx;
        SxR=-(c1*(SR(k)-S(n+2))+.5*(S(n+3)-S(n+1)))/dx;
    elseif (I(n+1,1) == 0) && (I(n+2,1) ~= 0)                               % true if shock in cell [n+2,n+3]
        epsR=(X(n+1)-X_s(k))/dx;
        c1=(2*epsR-1)/(1+epsR);
        c2=3/(1+epsR);
        RxR=-(c1*R(n+2)+c2*RR(k)-2*R(n+1))/dx;
        QxR=-(c1*Q(n+2)+c2*QR(k)-2*Q(n+1))/dx;
        SxR=-(c1*S(n+2)+c2*SR(k)-2*S(n+1))/dx;
    elseif I(n+1,1) ~= 0                                                    % true if shock in cell [n+1,n+2]
        epsR=X_s(k+1)-X_s(k);
        if epsR >= dx
            RxR=(RL(k+1)-RR(k))/epsR;
            QxR=(QL(k+1)-QR(k))/epsR;
            SxR=(SL(k+1)-SR(k))/epsR;
        else
            RxR=0;
            QxR=0;
            SxR=0;
        end
    end
else                                                                        % true if k-shock not last shock in cell [n,n+1]
    RxR=0;                                                                  % do not resolve cell microstructure
    QxR=0;
    SxR=0;
end                                                                         % end of right side derivative computation

function [RxR, QxR, SxR] = fxR2(k,n)
% fxR2.M - computes derivatives at right side of shock k when N-1 < n < N
%
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global dx S X X_s SR R Q RR QR 

epsR=(X(n+1)-X_s(k))/dx;
c1=(2*epsR-1)/(1+epsR);
c2=3/(1+epsR);
RxR=-(c1*R(n+2)+c2*RR(k)-2*R(n+1))/dx;
QxR=-(c1*Q(n+2)+c2*QR(k)-2*Q(n+1))/dx;
SxR=-(c1*S(n+2)+c2*SR(k)-2*S(n+1))/dx;

function [RxL, QxL, SxL] = fxL1(k,n)
% fxL1.M - computes derivatives at left side of shock k when n > 2
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global dx I S X X_s SL SR R Q RR RL QR QL 

if I(n,1) == k                                                              % true if k-shock is first shock in cell [n,n+1]
    if (I(n-1,1) == 0) && (I(n-2,1) == 0)                                   % true if no shock between cells [n-2,n]
        epsL=(X_s(k)-X(n))/dx;
        c1=2/(1+epsL);
        RxL=(c1*(RL(k)-R(n-1))+.5*(R(n-2)-R(n)))/dx;
        QxL=(c1*(QL(k)-Q(n-1))+.5*(Q(n-2)-Q(n)))/dx;
        SxL=(c1*(SL(k)-S(n-1))+.5*(S(n-2)-S(n)))/dx;
    elseif (I(n-1,1) == 0) && (I(n-2,1) ~= 0)                               % true if shock in cell [n-2,n-1]
        epsL=(X_s(k)-X(n))/dx;
        c1=(2*epsL-1)/(1+epsL);
        c2=3/(1+epsL);
        RxL=(c1*R(n-1)+c2*RL(k)-2*R(n))/dx;
        QxL=(c1*Q(n-1)+c2*QL(k)-2*Q(n))/dx;
        SxL=(c1*S(n-1)+c2*SL(k)-2*S(n))/dx;
    elseif I(n-1,1) ~=  0                                                   % true if shock in cell [n-1,n]
        epsL=X_s(k)-X_s(k-1);
        if epsL >= dx
            RxL=(RL(k)-RR(k-1))/epsL;
            QxL=(QL(k)-QR(k-1))/epsL;
            SxL=(SL(k)-SR(k-1))/epsL;
        else
            RxL=0;
            QxL=0;
            SxL=0;
        end
    end
else
    RxL=0;
    QxL=0;
    SxL=0;
end

function [RxL, QxL, SxL] = fxL2(k,n)
% fxL2.M - computes derivatives at left side of shock k when 2 > n > 1
%
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global X_s X S dx SL R Q RL QL

epsL=(X_s(k)-X(n))/dx;
c1=(2*epsL-1)/(1+epsL);
c2=3/(1+epsL);
RxL=(c1*R(n-1)+c2*RL(k)-2*R(n))/dx;
QxL=(c1*Q(n-1)+c2*QL(k)-2*Q(n))/dx;
SxL=(c1*S(n-1)+c2*SL(k)-2*S(n))/dx;

function find_R(j,sigma)
% FIND_R.M - find Riemann variable R at n=NT, pressure given
%
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global U N kmax IS u S Sn gamma Xx NT Rn R Ro a LAx So dx LAt gm  

ggm=gamma*gm;
LNTp=U(NT)+Xx*a(NT);
area_term=a(NT)*(u(NT)*LAx(NT)+LAt(NT))*dx;
if j == 1
    Rx=(R(NT)-R(N));
    Sx=(S(NT)-S(N));
    St=-U(NT)*Sx;
    Rt=-LNTp*(Rx-a(NT)*Sx/ggm)-area_term;
    Rn(NT)=R(NT)+sigma*Rt;
    Sn(NT)=S(NT)+sigma*St;
else
    Rx=(2*R(NT)-3*R(N)+R(N-1));
    Sx=(2*S(NT)-3*S(N)+S(N-1));
    if (kmax > 0) && (IS(kmax) == N-1)
        Rx=(R(NT)-R(N));
        Sx=(S(NT)-S(N));
    end
    St=-U(NT)*Sx;
    Rt=-LNTp*(Rx-a(NT)*Sx/ggm)-area_term;
    Rn(NT)=.5*(Ro(NT)+R(NT)+sigma*Rt);
    Sn(NT)=.5*(So(NT)+S(NT)+sigma*St);
end

function find_Q(j,sigma)
% FIND_Q.M - find Riemann variable Q at n=1
%
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global U kmax IS u S Xx Qn a LAx dx LAt gamma gm QL Q Qo

ggm=gamma*gm;
L1m=U(1)-Xx*a(1);
area_term=a(1)*(u(1)*LAx(1)+LAt(1))*dx;
if (kmax > 0) && (IS(1) == 1)
    Qn(1)=QL(1);
else
    if j == 1
        Qx=(Q(2)-Q(1));
        Sx=(S(2)-S(1));
        Qt=-L1m*(Qx-a(1)*Sx/ggm)-area_term;
        Qn(1)=Q(1)+sigma*Qt;
    else
        Qx=-(2*Q(1)-3*Q(2)+Q(3));
        Sx=-(2*S(1)-3*S(2)+S(3));
        if (kmax > 0) && (IS(1) == 2)
            Qx=(Q(2)-Q(1));
            Sx=(S(2)-S(1));
        end
        Qt=-L1m*(Qx-a(1)*Sx/ggm)-area_term;
        Qn(1)=.5*(Qo(1)+Q(1)+sigma*Qt);
    end
end
        