function mesh_xing
% MESH_XING.M - performs update for mesh point crossed by a shock wave
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piton Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global ISO xo_s I IS kmax x x_s Xx dx u P S So kout a Z gamma
global SL SR SoR SoL R Q Ro Qo QL RL QR RR t RoL RoR QoL QoR 

gm=gamma-1;
ndone=0;

ISO=IS;                                                                      % save shock IS index, to detect shock crossing mesh point

shock_index                                                                 % re-build shock indexes (re-defines IS)

for kk=2:kmax+1                                                             % find points that are crossed by shocks; will be used to avoid 
    nx(kk)=0;                                                               % using point (n-1) or (n+1) if that point is crossed by shock at this time step.
    k=kk-1;                                                                 % because we need to check k-1 & k+1 and we want to avoid going
    if IS(k) > ISO(k)                                                       % outside array bounds [1,kmax] we are creating a buffer at [0,kmax+1]
        nx(kk)=IS(k);
    elseif IS(k) < ISO(k)
        nx(kk)=ISO(k);
    end
end
nx(1)=0; nx(kmax+2)=0;

for k=1:kmax

    if ISO(k) < IS(k)                                                       % if shock index changed (+), recompute point crossed by right running shock
        n=IS(k);
        kk=I(n,1);
        denom=x_s(kk)-xo_s(kk);
        if abs(denom) > Xx*dx/100
            delta=(x(n)-xo_s(kk))/denom;
        else
            delta=0;
        end
        Ro(n)=RoL(kk)+delta*(RL(kk)-RoL(kk));
        Qo(n)=QoL(kk)+delta*(QL(kk)-QoL(kk));
        So(n)=SoL(kk)+delta*(SL(kk)-SoL(kk));
        if (I(n-1,1) == 0) && (n-1 ~= nx(k)) && (ndone ~= n)
            %{
            disp(['(a) mesh crossing at kout= ' num2str(kout)...
                ', n= ' num2str(n) ', t= ' num2str(t)...
                ', k= ' num2str(kk) ' -- moving right'])
            %}
            ndone=n;
             d1=(x(n)-x_s(kk))/(x(n-1)-x_s(kk));
             d2=(x(n)-x(n-1))/(x_s(kk)-x(n-1));
             Q(n)=Q(n-1)*d1+QL(kk)*d2;
             R(n)=R(n-1)*d1+RL(kk)*d2;
             S(n)=S(n-1)*d1+SL(kk)*d2;
             u(n)=(R(n)-Q(n))/2;
             a(n)=gm*(R(n)+Q(n))/4;
             Z(n)=a(n)^2/gamma;
             P(n)=(gamma*log(Z(n))-S(n))/gm; 
        elseif ndone ~= n
            ndone=n;                                                        % if two or more shocks cross the same mesh point, we want 
            %{
            disp(['(c) mesh crossing at kout= ' num2str(kout)...            % the interpolation done with values from lowest k, this is accomplished with "ndone"
                ', n= ' num2str(n) ', t= ' num2str(t)...
                ', k= ' num2str(k) ' -- moving right'])
            %}
            d1=x_s(k)-x_s(k-1);
            S(n)=(SL(k)*(x(n)-x_s(k-1))-SR(k-1)*(x(n)-x_s(k)))/d1;
            R(n)=(RL(k)*(x(n)-x_s(k-1))-RR(k-1)*(x(n)-x_s(k)))/d1;
            Q(n)=(QL(k)*(x(n)-x_s(k-1))-QR(k-1)*(x(n)-x_s(k)))/d1;
            u(n)=(R(n)-Q(n))/2;
            a(n)=gm*(R(n)+Q(n))/4;
            Z(n)=a(n)^2/gamma;
            P(n)=(gamma*log(Z(n))-S(n))/gm;
        end
    elseif ISO(k) > IS(k)                                                   % if shock index changed (-), recompute point crossed by left running shock
        n=IS(k)+1;
        kk=I(n-1,2);
        denom=x_s(k)-xo_s(k);
        if abs(denom) > Xx*dx/100
            delta=(x(n)-xo_s(k))/denom;
        else
            delta=0;
        end
        Ro(n)=RoR(k)+delta*(RR(k)-RoR(k));
        Qo(n)=QoR(k)+delta*(QR(k)-QoR(k));
        So(n)=SoR(k)+delta*(SR(k)-SoR(k));
        if (I(n,1) == 0 ) && (n+1 ~= nx(k+2))                           % no shocks in [n,n+1], also check for rare event when (n+1) is crossed
            %{
            disp(['(e) mesh crossing at kout= ' num2str(kout)...
                ', n= ' num2str(n) ', t= ' num2str(t)...
                ', k= ' num2str(kk) ' -- moving left'])
            %}
            d1=(x(n)-x_s(kk))/(x(n+1)-x_s(kk));
            d2=(x(n)-x(n+1))/(x_s(kk)-x(n+1));
            S(n)=S(n+1)*d1+SR(kk)*d2;
            R(n)=R(n+1)*d1+RR(kk)*d2;
            Q(n)=Q(n+1)*d1+QR(kk)*d2;
            u(n)=(R(n)-Q(n))/2;
            a(n)=gm*(R(n)+Q(n))/4;
            Z(n)=a(n)^2/gamma;
            P(n)=(gamma*log(Z(n))-S(n))/gm;
        else
            %{
            disp(['(f) mesh crossing at kout= ' num2str(kout)...
                ', n= ' num2str(n) ', t= ' num2str(t)...
                ', k= ' num2str(k) ' -- moving left'])
            %}
            d1=x_s(k+1)-x_s(k);
            S(n)=(SL(k+1)*(x(n)-x_s(k))-SR(k)*(x(n)-x_s(k+1)))/d1;
            R(n)=(RL(k+1)*(x(n)-x_s(k))-RR(k)*(x(n)-x_s(k+1)))/d1;
            Q(n)=(QL(k+1)*(x(n)-x_s(k))-QR(k)*(x(n)-x_s(k+1)))/d1;
            u(n)=(R(n)-Q(n))/2;
            a(n)=gm*(R(n)+Q(n))/4;
            Z(n)=a(n)^2/gamma;
            P(n)=(gamma*log(Z(n))-S(n))/gm;
        end
    end
end


