% MOVIE_MAKER.M makes movie
% reads data from temporary.mat and makes movie
%
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

clear all

load temporary.mat tout X area_index xout INIT NT osci_eps pulse_loc...
    kmaxout shift sg x1mach x2mach typeout pressure

global sg osci_eps pulse_loc shift

movie_figure=figure('position', [100 250 600 400]);

Mcount=length(tout);

switch INIT
    
    case 1
        pmax1=max(pressure);
        pmax=max(pmax1);
        for frame_number=1:Mcount
            t=tout(frame_number);
            b=pistonlaw(t);
            c=exitboundary(t); 
            area_top=4;
            area_bottom=2;
            area_bound=[area_top area_bottom];                    
            pist=area_bound;
            pistloc=[b b];
            rod_end=b-.1;
            xarm=[rod_end b];
            area_mid=(area_top+area_bottom)/2;
            yarm=[area_mid area_mid];
            x1(1)=0;
            x1(2)=2;
            area(1)=area_top;
            area(2)=area_top;
            area1(1)=area_bottom;
            area1(2)=area_bottom;
            kmax=kmaxout(frame_number);
            x=b+(c-b)*X;
            p(1:NT)=2*pressure(1:NT,frame_number)/pmax;
            plot(pistloc,pist,'k-',xarm,yarm,'k-',x,p,'b-')
            hold on
            if kmax > 0
                for k=1:kmax
                    x_shock=xout(k,frame_number);
                    x_s1=[x_shock x_shock];
                    plot(x_s1,area_bound,'k-');
                    hold on
                end
            end
            plot(x1,area,'k-',x1,area1,'k-');
            tstr=sprintf('%5.2f', t);
            text(1.5,4.5,tstr)
            xlim([-.2 2])
            ylim([0 5])
            figure(movie_figure);
            f=['CASE ' num2str(INIT)];
            text(.5,1,'pressure')
            title(f)
            xlabel('x')
            ylabel('area')
            hold off
        end
        
        case {14, 15, 16}
        pmax1=max(pressure);
        pmax=max(pmax1);
        x1(1)=pistonlaw(0);
        x1(2)=exitboundary(0);
        xmin=x1(1)-.2;
        xmax=x1(2)+.2;
        for frame_number=1:Mcount
            t=tout(frame_number);
            b=pistonlaw(t);
            c=exitboundary(t); 
            area_top=4; 
            area_bottom=2;
            area=[area_bottom area_top];                    
            pist=area;
            pistloc_left=[b b];
            pistloc_right=[c c];
            rod_end_left=b-.1;
            rod_end_right=c+.1;
            xarm_left=[rod_end_left b];
            xarm_right=[rod_end_right c];
            yarm1=(area_top+area_bottom)/2;
            yarm=[yarm1 yarm1];
            area_t(1)=area_top;
            area_t(2)=area_top;
            area_b(1)=area_bottom;
            area_b(2)=area_bottom;
            kmax=kmaxout(frame_number);
            x=b+(c-b)*X;
            p(1:NT)=2*pressure(1:NT,frame_number)/pmax;
            plot(pistloc_left,pist,'k-',xarm_left,yarm,'k-',...
                pistloc_right,pist,'k-',xarm_right,yarm,'k-',x,p,'b-')
            hold on
            if kmax > 0
                for k=1:kmax
                    x_shock=xout(k,frame_number);
                    x_s1=[x_shock x_shock];
                    type=typeout(k,frame_number);
                    if type ~= 99
                        plot(x_s1,area,'k-');
                    else
                        plot(x_s1,area,'g-');
                    end
                    hold on
                end
            end
            plot(x1,area_t,'k-',x1,area_b,'k-');
            tstr=sprintf('%5.2f', t);
            text(3.5,4.5,tstr)
            xlim([xmin xmax])
            ylim([0 5])
            figure(movie_figure);
            f=['CASE ' num2str(INIT)];
            text(.3,1,'pressure')
            title(f)
            xlabel('x')
            ylabel('area')
            hold off
        end
        
    case {17, 18, 21, 22, 23, 24, 25, 26, 27, 28}
        % M=moviein(Mcount);
        pmax1=max(pressure);
        pmax=max(pmax1);
        pmin1=min(pressure);
        pmin=min(pmin1);
        ymax=10;
        D=4;
        if INIT == 17
            ymax=3;
            D=2;
        elseif INIT == 18
            ymax=6;
            D=3;
        end
        ymax1=ymax-1;
        for frame_number=1:Mcount
            t=tout(frame_number);
            b=pistonlaw(t);
            c=exitboundary(t);
            x=b+(c-b)*X;
            for n=1:NT
                area(n)=area_f(x(n), t, area_index);
            end                               
            p(1:NT)=D*(pressure(1:NT,frame_number)-pmin)/(pmax-pmin)+ymax-D;
            xm=x1mach(frame_number);
            xm1=[xm xm];
            areas2=area_f(xm, t, area_index);
            xm=x2mach(frame_number);
            xm2=[xm xm];
            areas4=area_f(xm, t, area_index);
            areas3=[0 areas2];
            areas5=[0 areas4];
            kmax=kmaxout(frame_number);
            plot(x,p,'b-',xm1,areas3,'b:',xm2,areas5,'b:')
            hold on
            if kmax > 0
                for k=1:kmax
                    x_shock=xout(k,frame_number);
                    areas=area_f(x_shock, t, area_index);
                    x_s1=[x_shock x_shock];
                    areas1=[0 areas];                    
                    type=typeout(k,frame_number);
                    if type ~= 99
                        plot(x_s1,areas1,'k-');
                    else
                        plot(x_s1,areas1,'g-');
                    end
                    hold on
                end
            end
            plot(x,area,'k-');
            tstr=sprintf('%5.2f', t);
            text(c-1,ymax1,tstr)
            xlim([b c])
            ylim([0 ymax])
            figure(movie_figure);
            f=['CASE ' num2str(INIT)];
            title(f)
            text(.2,ymax1,'pressure')
            xlabel('x')
            ylabel('area')
            % M(:,frame_number)=getframe;
            hold off
        end
        % movie(M);
end


