function plot_solutionpiston(INDEX)
% PLOT_SOLUTIONPISTON.M creates plots
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global nfig u X NT t uexact INIT sg XC YC ZC dx 
global kmax kmax_save kout tout kout_plot LAx area_index
global t1mach x1mach psaveR psaveL typeout 

b=pistonlaw(t);
c=exitboundary(t);
map = [0 0 0];                                                              % set contour color map to black

x=b+(c-b)*X;
threshhold=2*dx*(c-b);

switch (INDEX)
    case 1                                                                  % INDEX
        switch (INIT)
            case {1, 4}
                nfig=nfig+1;
                figure(nfig)
                plot(x,u,'k-')
                xlabel('x')
                ylabel('u')
                f=['Initial Conditions, t=' num2str(t)];
                grid on
                title(f)
            otherwise

                plots

                nfig=nfig+1;
                figure(nfig)
                for n=1:NT
                    area(n)=area_f(x(n), t, area_index);
                end
                plot(x,area,'k-',0,0,'.')
                xlabel('x')
                ylabel('area')
                ylim([0 20])
                xlim([b c])
                grid on
                f='area';
                title(f)

                nfig=nfig+1;
                figure(nfig)
                plot(x,LAx,'k-')
                xlabel('x')
                ylabel('d(area)/dx/area')
                grid on
                f='derivative of log(area) wrt x';
                title(f)
        end

        if kmax > 0
            disp(['time= ' num2str(t)])
            print_shock
            disp(' ')
        end

    case 2                                                                  % INDEX

        if (kmax == 0) && (INIT == 1)
            exact
        end

        plots

        if INIT == 19                                                       % plot P contours although no shocks are present
            nfig=nfig+1;
            figure(nfig)
            [C,h]=contour(XC,YC,abs(ZC));
            clabel(C,h);
            colormap(map)
        end

        if kmax > 0
            nfig=nfig+1;
            figure(nfig)
            if INIT >= 21
                plot(x1mach,t1mach,'y.')
                hold on
            end
            switch(INIT)
                case {1, 12}
                    [C,h]=contour(XC,YC,abs(ZC));
                    colormap(map)
                    hold on
                    x1=[0,1.1832];
                    t1=[0,1];
                    plot(x1,t1,'k--')
                    hold on
                case {4, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,...
                        24, 25, 26, 27, 28}
                    
                    [C,h]=contour(XC,YC,abs(ZC));
                    %{
                                        % turn on to add labels to contours
                                        % clabel(C,h);
                    %}
                    colormap(map)
                    hold on
            end
            
            path_white(threshhold)                                          % white-out contours at shock before plotting shock wave paths
            path(threshhold)                                                % plot shock wave paths

            mc=kout;
            delta_t=t/mc;
            for m=1:mc+1                                                    % plot piston paths
                tplot(m)=(m-1)*delta_t;
                bplot(m)=pistonlaw(tplot(m));
                cplot(m)=exitboundary(tplot(m));
                fsplot(m)=sg*tplot(m);
                if INIT == 4
                    fsplot(m)=1-fsplot(m);
                end

            end

            if (INIT == 1) || (INIT == 4)                                   % plot location of shock formation for INIT [1,4]
                plot(bplot,tplot,'k-','LineWidth',2)
                hold on
                plot(fsplot,tplot,'k--')
                hold on
                plot(cplot,tplot,'k-','LineWidth',2)
                hold on
                if t > 0.8439
                    x0=0.8449;
                    if INIT == 4
                        x0=1-x0;
                    end
                    plot(x0,0.8439,'r+')
                end
            else
                plot(bplot,tplot,'k-','LineWidth',2)
                hold on
                plot(cplot,tplot,'k-','LineWidth',2)
            end

            hold off
            xlabel('x')
            ylabel('t')
            f='Shock Wave Path';
            title(f)

            nfig=nfig+1;                                                    % plot shock pressure
            figure(nfig)
            for k=1:kmax_save
                for n=1:kout_plot
                    type=typeout(k,n);
                    if type == 99                                           % skip contact
                        continue
                    end
                    pl=psaveL(k,n);
                    pr=psaveR(k,n);
                    tsp1=tout(k,n);
                    plot(tsp1,pl,'b.',tsp1,pr,'k.')
                    hold on
                end
            end
            grid on
            xlabel('t')
            ylabel('pressure')
            f='Shock Wave Pressure';
            title(f)
            hold off

            print_shock
        end

        if  (INIT == 1) &&  (kmax == 0)                                     % compare to exact solution for simple wave
            udif=(u-uexact).^2;
            sumdif=sum(udif);
            maxe1=abs(u-uexact);
            maxe=max(maxe1);
            sumdif=sqrt(sumdif);
            disp(['time= ' num2str(t) ' L2(u)= ' num2str(sumdif)...
                ' Linf(u)= ' num2str(maxe)])
        end
        
end

function path_white(threshhold)
% PATH.M white-out contours at shock
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global kmax_save xout tout typeout kout_plot 

for k=1:kmax_save                                                           % because the shock counter changes if new shocks are inserted or removed
    m=0;                                                                    % we need some logic to identify shock points and connect them with a line
    clear xsp tsp type                                                      % the test we use here is that if two shock points are close (within threshhold)
    xout(k,kout_plot+1)=xout(k,kout_plot);                                  % padd the arrays
    tout(k,kout_plot+1)=tout(k,kout_plot);
    typeout(k,kout_plot+1)=typeout(k,kout_plot);
    for n=2:kout_plot+1
        delta=abs(xout(k,n)-xout(k,n-1));
        if (delta < threshhold) && (typeout(k,n) ~= 99)                     % typeout = 99 corresponds to a contact, we skip the contacts
            m=m+1;
            xsp(m)=xout(k,n-1);
            tsp(m)=tout(k,n-1);
            type=typeout(k,n-1);
        elseif m > 0
            plot(xsp,tsp,'w-','LineWidth',8)                                % white-out the concentration of contours at the shock
            hold on
            m=0;
            clear xsp tsp type
        end
    end
    if m > 0
        plot(xsp,tsp,'w-','LineWidth',8)                                    % white-out the concentration of contours at the shock
        hold on
    end
end

function path(threshhold)
% PATH.M plots shock path
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global kmax_save xout tout typeout kout_plot

for k=1:kmax_save                                                           % because the shock counter changes if new shocks are inserted or removed
    m=0;                                                                    % we need some logic to identify shock points and connect them with a line
    clear xsp tsp type                                                      % the test we use here is that if two shock points are close (within threshhold)
    xout(k,kout_plot+1)=xout(k,kout_plot);
    tout(k,kout_plot+1)=tout(k,kout_plot);
    typeout(k,kout_plot+1)=typeout(k,kout_plot);
    for n=2:kout_plot+1
        delta=abs(xout(k,n)-xout(k,n-1));
        if (delta < threshhold) && (typeout(k,n) == typeout(k,n-1))
            m=m+1;
            xsp(m)=xout(k,n-1);
            tsp(m)=tout(k,n-1);
            type=typeout(k,n-1);
        elseif m > 0
            if type ~= 99                                                   % type = 99 is a contact
                plot(xsp,tsp,'k-','LineWidth',2)
            else
                plot(xsp,tsp,'g--','LineWidth',2)
            end
            hold on
            m=0;
            clear xsp tsp type
        end
    end
    if m > 0
        if type ~= 99                                                       % type = 99 is a contact
            plot(xsp,tsp,'k-','LineWidth',2)
        else
            plot(xsp,tsp,'g--','LineWidth',2)
        end
        hold on
    end
end

function plots
% PLOTS.M creates plots
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global u P S t uexact Pexact Sexact uL uR PL PR SR SL
global R Q RL RR QL QR Rexact Qexact kmax a aL aR

f=['u at t=' num2str(t)];                                                   % plot u, P, S, R, Q, M
str='u';
plot_f(u,uL,uR,uexact,f,str)

f=['P at t=' num2str(t)];
str='P';
plot_f(P,PL,PR,Pexact,f,str)

f=['S at t=' num2str(t)];
str='S';
plot_f(S,SL,SR,Sexact,f,str)

f=['R at t=' num2str(t)];
str='R';
plot_f(R,RL,RR,Rexact,f,str)

f=['Q at t=' num2str(t)];
str='Q';
plot_f(Q,QL,QR,Qexact,f,str)

M=u./a;
ML(1)=0;
MR(1)=0;
if kmax > 0
    ML=uL./aL;
    MR=uR./aR;
end

f=['Mach at t=' num2str(t)];
str='M';
plot_f(M,ML,MR,M,f,str)



