function print_shock
% PRINT_SHOCK.M creates output of shock data
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global kmax shock_type x_s w IS uL PL SL uR PR SR ZL ZR aL aR RR RL QR QL t

b=pistonlaw(t);
bt=pistonlawt(t);
c=exitboundary(t);
ct=exitboundaryt(t);

disp([' at t= ' num2str(t) ' b= ' num2str(b) ', bt= ' num2str(bt)...
    ', c= ' num2str(c) ', ct= ' num2str(ct)])
for k=1:kmax
    switch shock_type(k)
        case 'l'
            disp(' k   x_s     w      ns     type    p-ratio   Mrel    [a]      [R]     [Q]')
            p_rat=exp(PL(k))/exp(PR(k));
            da=abs(aR(k)-aL(k));
            Mrel=(uR(k)-w(k))/aR(k);
            dR=RL(k)-RR(k);
            dQ=QL(k)-QR(k);
            ML=uL(k)/aL(k);
            MR=uR(k)/aR(k);
            fprintf('%2.0f %7.4f %7.4f %3.0f    %s %7.4f   %7.4f %7.4f  %7.4f %7.4f\n'...
                ,k,x_s(k),w(k),IS(k),'l-shock',p_rat,Mrel,da,dR,dQ)
            disp(' uL      PL      SL      ZL      RL      QL      ML')
            fprintf('%7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f\n',...
                uL(k),PL(k),SL(k),ZL(k),RL(k),QL(k),ML)
            disp(' uR      PR      SR      ZR      RR      QR      MR')
            fprintf('%7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f\n',...
                uR(k),PR(k),SR(k),ZR(k),RR(k),QR(k),MR)
        case 'r'
            disp(' k   x_s     w      ns     type    p-ratio   Mrel    [a]      [R]     [Q]')
            p_rat=exp(PR(k))/exp(PL(k));
            da=abs(aR(k)-aL(k));
            Mrel=(uL(k)-w(k))/aL(k);
            dR=RR(k)-RL(k);
            dQ=QR(k)-QL(k);
                        ML=uL(k)/aL(k);
            MR=uR(k)/aR(k);
            fprintf('%2.0f %7.4f %7.4f %3.0f    %s %7.4f   %7.4f %7.4f  %7.4f %7.4f\n'...
                ,k,x_s(k),w(k),IS(k),'r-shock',p_rat,Mrel,da,dR,dQ)
            disp(' uL      PL      SL      ZL      RL      QL      ML')
            fprintf('%7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f\n',...
                uL(k),PL(k),SL(k),ZL(k),RL(k),QL(k),ML)
            disp(' uR      PR      SR      ZR      RR      QR      MR')
            fprintf('%7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f\n',...
                uR(k),PR(k),SR(k),ZR(k),RR(k),QR(k),MR)
        case 'c'
            disp(' k   x_s     w      ns     type    p-ratio   Mrel   [a]      [R]     [Q]')
            p_rat=exp(PL(k))/exp(PR(k));
            da=abs(aR(k)-aL(k));
            dR=abs(RR(k)-RL(k));
            dQ=abs(QR(k)-QL(k));
            ML=uL(k)/aL(k);
            MR=uR(k)/aR(k);
            fprintf('%2.0f %7.4f %7.4f %3.0f    %s %7.4f    %s   %7.4f  %7.4f %7.4f\n'...
                ,k,x_s(k),w(k),IS(k),'contact',p_rat,'---',da,dR,dQ)
            disp(' uL      PL      SL      ZL      RL      QL      ML')
            fprintf('%7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f\n',...
                uL(k),PL(k),SL(k),ZL(k),RL(k),QL(k),ML)
            disp(' uR      PR      SR      ZR      RR      QR      MR')
            fprintf('%7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f\n',...
                uR(k),PR(k),SR(k),ZR(k),RR(k),QR(k),MR)
    end
end

