function shock_cross
% SHOCK_CROSS.M - performs computation for shock interactions
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% performs crossing of k-1 & k shocks, where k=kcross
%
% Copyright 2005-2008 M.D.Salas.

global kcross x_s w t LAST shock_type kout uL uR aL aR RR RL QR QL gm gp

km=kcross;
kp=km+1;
tol=10^-7;

print_event
disp([ 'shock crossing detected at kout= ' num2str(kout)...
    ', t= ' num2str(t) ', k= [' num2str(km) ',' num2str(kp) ']'])
print_shock
disp([shock_type(km) '-shock (k= ' num2str(km) ') crossing ' ...
    shock_type(kp) '-shock (k= ' num2str(kp) ') executed at t= '...
    num2str(t) ' and x= ' num2str(x_s(km))])
        
if (shock_type(km) == 'c') || (shock_type(kp) == 'c')                       % IF TRUE, SHOCK/CONTACT INTERACTION    
    if shock_type(km) == 'l'                                                % CASE 1, l-shock k=km, contact k=kp
        if (aL(kp) >= aR(kp))                                               % case 1a, shock reflected
            tryb=error1(km,tol);
            shock_contact_shock(km,tryb)
        else                                                                % case 1b, expansion reflected
            [tryb,rat]=error2(km,kp,tol);
            if rat < 1
                fan_contact_shock(km,tryb)
            else
                tryb=error1(km,tol);
                shock_contact_shock(km,tryb)
            end
        end
    elseif shock_type(kp) == 'r'                                            % CASE 2, r-shock k=kp, contact k=km
        if aR(km) >= aL(km)                                                 % case 2a, shock reflected
            wkm=error4(kp,tol);
            shock_contact_shock(km,wkm)
        else                                                                % case 2b, expansion reflected
            [tryb,rat]=error3(km,kp,tol);
            if rat < 1
                shock_contact_fan(km,tryb)
            else
                wkm=error4(kp,tol);
                shock_contact_shock(km,wkm)
            end
        end
    end

elseif shock_type(km) == shock_type(kp)                                     % resolves crossing for shocks of same family
    if shock_type(km) == 'l'                                                % L-SHOCKS COALESCING
        try0(1)=(w(km)+w(kp))/2;                                            % start by guessing w for out going shock
        err(1)=shock_shock_2(try0(1));
        try0(2)=0.95*try0(1);
        err(2)=shock_shock_2(try0(2));
        for kip=1:20
            if abs(err(2)) > tol
                tryb=try0(1)-err(1)*(try0(2)-try0(1))/(err(2)-err(1));
                try0(1)=try0(2);
                try0(2)=tryb;
                err(1)=err(2);
                err(2)=shock_shock_2(try0(2));
                %{
                disp(['kip= ' num2str(kip) ', err2= ' num2str(err(2))...
                    ', try2= ' num2str(try0(2))])
                %}
            else
                break
            end
        end
        if abs(err(2)) > tol
            disp('shock-shock iteration failed, job aborted. ')
            disp('check shock_cross routine at (a). ')
            LAST=1;
        end
        
        fan_contact_shock(km,try0(2))

    else                                                                    % R-SHOCKS COALESCING
        try0(1)=(w(km)+w(kp))/2;                                            % start by guessing w for out going shock
        err(1)=shock_shock_3(try0(1));
        try0(2)=0.95*try0(1);
        err(2)=shock_shock_3(try0(2));
        for kip=1:20
            if abs(err(2)) > tol
                tryb=try0(1)-err(1)*(try0(2)-try0(1))/(err(2)-err(1));
                try0(1)=try0(2);
                try0(2)=tryb;
                err(1)=err(2);
                err(2)=shock_shock_3(try0(2));
                %{
                disp(['kip= ' num2str(kip) ', err2= ' num2str(err(2))...
                    ', try2= ' num2str(try0(2))])
                %}
            else
                break
            end
        end
        if abs(err(2)) > tol
            disp('shock-shock iteration failed, job aborted. ')
            disp('check shock_cross routine at (b). ')
            LAST=1;
        end
        
        shock_contact_fan(km,try0(2))

    end
                                                                            % HEAD-ON COLLISION
elseif (shock_type(km) ~= 'c') &&  (shock_type(kp) ~= 'c')                  % resolves crossing for shocks of different family (not contacts)
    u0=uR(km);
    a0=aR(km);
    M0rel=(u0-w(km))/a0;
    SM0rel=M0rel^2;
    factor=1-2*gm*(1-1/SM0rel)/gp;                                          % this provides a good guess
    try0(1)=factor*w(kp);                                                   % start by guessing w for left shock; use factor*w(kp)as first guess
    err(1)=shock_shock_1(try0(1));
    try0(2)=.95*try0(1);
    err(2)=shock_shock_1(try0(2));
    for kip=1:20
        if abs(err(2)) > tol
            tryb=try0(1)-err(1)*(try0(2)-try0(1))/(err(2)-err(1));
            try0(1)=try0(2);
            try0(2)=tryb;
            err(1)=err(2);
            err(2)=shock_shock_1(try0(2));
            %{
            disp(['kip= ' num2str(kip) ', err2= ' num2str(err(2))...
                ', try2= ' num2str(try0(2))])
            %}
        else
            break
        end
    end
    if abs(err(2)) > tol
        disp('shock-shock iteration failed, job aborted. ')
        disp('check shock_cross routine at (c). ')
        LAST=1;
    end
    
    shock_contact_shock(km,try0(2))
    
end
RR=2*aR/gm+uR;
RL=2*aL/gm+uL;
QR=2*aR/gm-uR;
QL=2*aL/gm-uL;
print_shock
shock_elimination

function error = shock_shock_1(w_guess)
% SHOCK_SHOCK_1.M - solves l-shock/r-shock interaction
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global gamma uL uR PL PR aR aL kcross gm gp
                                                                           %         contact
km=kcross;                                                                  %  r-shock   kp  l-shock
kp=km+1;                                                                    %   km \    |    /kp+1
                                                                            %       \ 3 | 4 /
p1=exp(PL(km));                                                             %        \  |  /
u1=uL(km);                                                                  %     1   \ | /   2
a1=aL(km);                                                                  %------------------------>
M1rel=(u1-w_guess)/a1;                                                      %     1   /   \   2       x
SM1rel=M1rel^2;                                                             %        /  0  \
                                                                            %      km       kp
p3=p1*(2*gamma*SM1rel-gm)/gp;                                               %    l-shock   r-shock
rho3rho1=gp*SM1rel/(gm*SM1rel+2);
u3=w_guess+(u1-w_guess)/rho3rho1;

u4=u3;
p4=p3;

p2=exp(PR(kp));
u2=uR(kp);
a2=aR(kp);

p4p2=p4/p2;
SM2rel=(p4p2*gp+gm)/(2*gamma);
rho4rho2=gp*SM2rel/(gm*SM2rel+2);
wkp2=(u4*rho4rho2-u2)/(rho4rho2-1);                                         % resulting w for right shock (km+2)
SM2rel_bar=((u2-wkp2)/a2)^2;
error=SM2rel_bar-SM2rel;

function [error,rat] = shock_shock_2(w_guess)
% SHOCK_SHOCK_2.M - solves l-shock/l-shock interaction
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global gamma uR PL PR aR kcross SL RL gm gp
                                                                            % exp     contact
km=kcross;                                                                  %  fan     km    l-shock
kp=km+1;                                                                    %   ....    |    /kp
                                                                            %     ... 3 | 4 /
p1=exp(PL(km));                                                             %      ...  |  /
                                                                            %     1  .. | /   2
p2=exp(PR(kp));                                                             %------------------------>
u2=uR(kp);                                                                  %     1   / |  2          x
a2=aR(kp);                                                                  %        /  |   
SM2rel=((u2-w_guess)/a2)^2;                                                 %       / 0 |
                                                                            %      /    |
p4p2=(2*gamma*SM2rel-gm)/gp;                                                %    km     kp
p4=p4p2*p2;                                                                 %l-shock l-shock
rho4rho2=gp*SM2rel/(gm*SM2rel+2);
u4=w_guess+(u2-w_guess)/rho4rho2;

p3=p4;
rat=p3/p1;
u3=u4;
S3=SL(km);
Z3=exp((S3+gm*log(p3))/gamma);
a3=sqrt(gamma*Z3);

R3=2*a3/gm+u3;
error=R3-RL(km);
    
function [error,rat] = shock_shock_3(w_guess)
% SHOCK_SHOCK_3.M - solves r-shock/r-shock interaction
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global gamma uL PL PR aL kcross SR QR gm gp                                                                
                                                                            %       r-shock  contact
km=kcross;                                                                  %         km     kp
kp=km+1;                                                                    %           |    /  ... exp
                                                                            %        1  | 4 /3 ...  fan
p2=exp(PR(kp));                                                             %           |  / ..
                                                                            %           | /.  
p1=exp(PL(km));                                                             %------------------------>
u1=uL(km);                                                                  %     1   / |  2          x
a1=aL(km);                                                                  %        /  |   
SM1rel=((u1-w_guess)/a1)^2;                                                 %       / 0 |
                                                                            %      /    |
p4p1=(2*gamma*SM1rel-gm)/gp;                                                %    km     kp
p4=p4p1*p1;                                                                 %  r-shock   r-shock
rho4rho1=gp*SM1rel/(gm*SM1rel+2);
u4=w_guess+(u1-w_guess)/rho4rho1;

p3=p4;
rat=p3/p2;
u3=u4;
S3=SR(kp);
Z3=exp((S3+gm*log(p3))/gamma);
a3=sqrt(gamma*Z3);

Q3=2*a3/gm-u3;
error=Q3-QR(kp);

function [error, wkm] = shock_shock_4(w_guess)
% SHOCK_SHOCK_4.M - solves l-shock/r-shock interaction
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global gamma uL uR PL PR aR aL kcross gm gp
                                                                           %         contact
km=kcross;                                                                  %  r-shock   kp  l-shock
kp=km+1;                                                                    %   km \    |    /kp+1
                                                                            %       \ 3 | 4 /
p2=exp(PR(kp));                                                             %        \  |  /
u2=uR(kp);                                                                  %     1   \ | /   2
a2=aR(kp);                                                                  %------------------------>
M2rel=(u2-w_guess)/a2;                                                      %     1   /   \   2       x
SM2rel=M2rel^2;                                                             %        /  0  \
                                                                            %      km       kp
p4=p2*(2*gamma*SM2rel-gm)/gp;                                               %    l-shock   r-shock
rho4rho2=gp*SM2rel/(gm*SM2rel+2);
u4=w_guess+(u2-w_guess)/rho4rho2;

u3=u4;
p3=p4;

p1=exp(PL(km));
u1=uL(km);
a1=aL(km);

rat=p3/p1;
SM1rel=(rat*gp+gm)/(2*gamma);
rho3rho1=gp*SM1rel/(gm*SM1rel+2);
wkm=(u3*rho3rho1-u1)/(rho3rho1-1);                                           % resulting w for right shock (km+2)
SM1rel_bar=((u1-wkm)/a1)^2;
error=SM1rel_bar-SM1rel;

function shock_contact_shock(km,tryb)
% SHOCK_CONTACT_SHOCK.M interaction results in shock/contact/shock
% configuration
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global kmax x_s w x gm gp uL uR PL PR SL SR ZL ZR aL aR dx 
global IS shock_type eps0 gamma P u S a Z

kp=km+1;
weak_contact_tol=.01;                                                       % tolerance used to eliminate weak contacts

nkm=IS(km);
nkp=IS(kp);
if nkm ~= nkp                                                               % if true, mesh point nkp is between km & kp. the mesh point needs to be updated
    u(nkp)=uR(kp);
    P(nkp)=PR(kp);
    S(nkp)=SR(kp);
    Z(nkp)=ZR(kp);
    a(nkp)=aR(kp);
end

w(km)=tryb;
p1=exp(PL(km));
u1=uL(km);
a1=aL(km);
Z1=ZL(km);
rho1=p1/Z1;
M1rel=(u1-w(km))/a1;
SM1rel=M1rel^2;
p2=exp(PR(kp));
u2=uR(kp);
Z2=ZR(kp);
rho2=p2/Z2;
p3=p1*(2*gamma*SM1rel-gm)/gp;
rho3rho1=gp*SM1rel/(gm*SM1rel+2);
rho3=rho3rho1*rho1;
Z3=p3/rho3;
a3=sqrt(gamma*Z3);
u3=w(km)+(u1-w(km))/rho3rho1;
u4=u3;
p4=p3;
p4p2=p4/p2;
SM2rel=(p4p2*gp+gm)/(2*gamma);
rho4rho2=gp*SM2rel/(gm*SM2rel+2);
rho4=rho4rho2*rho2;
Z4=p4/rho4;
a4=sqrt(gamma*Z4);

a_test=abs(a3-a4);

if a_test > weak_contact_tol                                                % if the jump in "a" is small ignore contact
kpp=km+2;
    if kmax > 2                                                             % if true, we have to shift shock indexes to make room for contact
        kmax=kmax+1;
        for k=kpp:kmax
            m=kmax+kpp-k;
            if m < kpp
                break
            end
            PR(m)=PR(m-1);
            PL(m)=PL(m-1);
            uR(m)=uR(m-1);
            uL(m)=uL(m-1);
            SR(m)=SR(m-1);
            SL(m)=SL(m-1);
            aR(m)=aR(m-1);
            aL(m)=aL(m-1);
            ZR(m)=ZR(m-1);
            ZL(m)=ZL(m-1);
            w(m)=w(m-1);
            x_s(m)=x_s(m-1);
            shock_type(m)=shock_type(m-1);
        end
    else
        kmax=3;
        m=3;
        PR(m)=PR(m-1);
        uR(m)=uR(m-1);
        SR(m)=SR(m-1);
        aR(m)=aR(m-1);
        ZR(m)=ZR(m-1);
    end

    shock_type(km)='r';
    shock_type(kp)='c';                                                     % insert a contact discontinuity
    shock_type(kpp)='l';
    w(kp)=u3;
    x_s(kp)=x_s(km)+eps0;
    PL(kp)=log(p3);
    uL(kp)=u3;
    SL(kp)=PL(kp)-gamma*log(rho3);
    ZL(kp)=Z3;
    aL(kp)=a3;
    PR(kp)=log(p4);
    uR(kp)=u4;
    SR(kp)=PR(kp)-gamma*log(rho4);
    ZR(kp)=Z4;
    aR(kp)=a4;
else
    disp(['speed of sound jump across contact was small (' num2str(a_test)...
        ') and contact was deleted'])
    shock_type(km)='r';
    shock_type(kp)='l';
    kpp=kp;
end

PR(km)=log(p3);
uR(km)=u3;
SR(km)=PR(km)-gamma*log(rho3);
ZR(km)=Z3;
aR(km)=a3;

w(kpp)=(u4*rho4rho2-u2)/(rho4rho2-1);
x_s(kpp)=x_s(kp)+eps0;
PL(kpp)=log(p4);
uL(kpp)=u4;
SL(kpp)=PL(kpp)-gamma*log(rho4);
ZL(kpp)=Z4;
aL(kpp)=a4;

shock_index

ns=IS(km);
            
if abs(x(ns)-x_s(km)) < dx/100                                              % this takes care of the unusual event when x(ns) coincides
    P(ns)=PL(km);                                                           % with x_s(km) and mesh values at ns are still those of region 0
    u(ns)=uL(km);
    S(ns)=SL(km);
    a(ns)=aL(km);
    Z(ns)=ZL(km);
end

function fan_contact_shock(km,tryb)
% FAN_CONTACT_SHOCK.M interaction results in an exp. fan/contact/shock
% configuration
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.
 
global uL uR PL PR SL SR ZL ZR aL aR shock_type eps0 gamma x_s w  
global IS u P S Z a gm gp

kp=km+1;

nkm=IS(km);
nkp=IS(kp);
if nkm ~= nkp                                                               % if true, mesh point nkp is between km & kp. the mesh point needs to be updated
    u(nkp)=uR(kp);
    P(nkp)=PR(kp);
    S(nkp)=SR(kp);
    Z(nkp)=ZR(kp);
    a(nkp)=aR(kp);
end

S1=SL(km);

p2=exp(PR(kp));
u2=uR(kp);
S2=SR(kp);
a2=aR(kp);
Z2=ZR(kp);
rho2=p2/Z2;
SM2rel=((u2-tryb)/a2)^2;

p4p2=(2*gamma*SM2rel-gm)/gp;
p4=p4p2*p2;
rho4rho2=gp*SM2rel/(gm*SM2rel+2);
rho4=rho4rho2*rho2;
u4=tryb+(u2-tryb)/rho4rho2;
S4=log(p4)-gamma*log(rho4);

p3=p4;
u3=u4;
S3=S1;

PL(km)=log(p3);                                                             % contact discontinuity
SL(km)=S3;
uL(km)=u3;
rho3=exp((PL(km)-S3)/gamma);
ZL(km)=p3/rho3;
aL(km)=sqrt(gamma*p3/rho3);
PR(km)=PL(km);
SR(km)=S4;
uR(km)=u4;
ZR(km)=exp((SR(km)+gm*PR(km))/gamma);
aR(km)=sqrt(gamma*ZR(km));

PL(kp)=PR(km);                                                              % shock
SL(kp)=SR(km);
uL(kp)=uR(km);
ZL(kp)=exp((SL(kp)+gm*PL(kp))/gamma);
aL(kp)=sqrt(gamma*ZL(kp));
PR(kp)=log(p2);
SR(kp)=S2;
uR(kp)=u2;
ZR(kp)=exp((SR(kp)+gm*PR(kp))/gamma);
aR(kp)=sqrt(gamma*ZR(kp));

w(km)=u4;
w(kp)=tryb;
            
shock_type(km)='c';                                                         % insert a contact discontinuity
shock_type(kp)='l';
x_s(kp)=x_s(km)+eps0;

shock_index

function shock_contact_fan(km,tryb)
% FAN_CONTACT_SHOCK.M interaction results in a shock/contact/exp. fan
% configuration
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.
  
global uL uR PL PR SL SR ZL ZR aL aR gm gp shock_type eps0 gamma x_s w  
global IS u P S Z a

kp=km+1; 

nkm=IS(km);
nkp=IS(kp);
if nkm ~= nkp                                                               % if true, mesh point nkp is between km & kp. the mesh point needs to be updated
    u(nkp)=uR(kp);
    P(nkp)=PR(kp);
    S(nkp)=SR(kp);
    Z(nkp)=ZR(kp);
    a(nkp)=aR(kp);
end

w(km)=tryb;

S2=SR(kp);

p1=exp(PL(km));
u1=uL(km);
a1=aL(km);
Z1=ZL(km);
rho1=p1/Z1;
SM1rel=((u1-w(km))/a1)^2;

p4p1=(2*gamma*SM1rel-gm)/gp;
p4=p4p1*p1;
rho4rho1=gp*SM1rel/(gm*SM1rel+2);
rho4=rho4rho1*rho1;
u4=w(km)+(u1-w(km))/rho4rho1;
S4=log(p4)-gamma*log(rho4);
w(kp)=u4;

p3=p4;
u3=u4;
S3=S2;

PR(kp)=log(p3);
SR(kp)=S3;
uR(kp)=u3;
ZR(kp)=exp((SR(kp)+gm*PR(kp))/gamma);
aR(kp)=sqrt(gamma*ZR(kp));
PL(kp)=log(p4);
SL(kp)=S4;
uL(kp)=u4;
ZL(kp)=exp((SL(kp)+gm*PL(kp))/gamma);
aL(kp)=sqrt(gamma*ZL(kp));
PR(km)=log(p4);
SR(km)=S4;
uR(km)=u4;
ZR(km)=exp((SR(km)+gm*PR(km))/gamma);
aR(km)=sqrt(gamma*ZR(km));

shock_type(km)='r';
shock_type(kp)='c';
x_s(kp)=x_s(km)+eps0;

shock_index

function tryb = error1(km,tol)
% error1.M interaction results in a shock/contact/exp. fan
% configuration
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.
  
global uL aL LAST

lambda=uL(km)-aL(km);                                                       % the reflected shock slope should be less than lambda                                                                                   % this provides a guess
try0(1)=lambda;                                                             % start by guessing w for left shock; use factor*w(km)as first guess
err(1)=shock_shock_1(try0(1));
try0(2)=1.2*try0(1);
err(2)=shock_shock_1(try0(2));
tryb=try0(2);
for kip=1:20
    if abs(err(2)) > tol
        tryb=try0(1)-err(1)*(try0(2)-try0(1))/(err(2)-err(1));
        try0(1)=try0(2);
        try0(2)=tryb;
        err(1)=err(2);
        err(2)=shock_shock_1(try0(2));
        %{
        disp(['kip= ' num2str(kip) ', err2= ' num2str(err(2))...
            ', try2= ' num2str(try0(2))])
        %}
    else
        break
    end
end
if abs(err(2)) > tol
    disp('shock-shock iteration failed, job aborted. ')
    disp('check shock_cross routine at "function error1" ')
    LAST=1;
end

function [tryb,rat] = error2(km,kp,tol)
% error2.M interaction results in a shock/contact/exp. fan
% configuration
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.
  
global LAST ZL ZR w

Z0=ZL(kp);
Z2=ZR(kp);
factor=Z2/Z0;
try0(1)=factor*w(km);                                                       % start by guessing w for out going shock
[err(1),rat]=shock_shock_2(try0(1));
try0(2)=0.9*try0(1);
[err(2),rat]=shock_shock_2(try0(2));
for kip=1:20
    if abs(err(2)) > tol
        tryb=try0(1)-err(1)*(try0(2)-try0(1))/(err(2)-err(1));
        try0(1)=try0(2);
        try0(2)=tryb;
        err(1)=err(2);
        [err(2),rat]=shock_shock_2(try0(2));
        %{
        disp(['kip= ' num2str(kip) ', err2= ' num2str(err(2))...
            ', try2= ' num2str(try0(2))])
        %}
    else
        break
    end
end
if abs(err(2)) > tol
    disp('shock-shock iteration failed, job aborted. ')
    disp('check shock_cross routine at "function error2". ')
    LAST=1;
end

function [tryb,rat] = error3(km,kp,tol)
% error3.M interaction results in a shock/contact/exp. fan
% configuration
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.
  
global LAST ZL ZR w

Z0=ZL(km);
Z2=ZR(km);
factor=Z0/Z2;
try0(1)=factor*w(kp);                                                       % start by guessing w for out going shock
[err(1),rat]=shock_shock_3(try0(1));
try0(2)=0.95*try0(1);
[err(2),rat]=shock_shock_3(try0(2));
tryb=try0(2);
for kip=1:20
    if abs(err(2)) > tol
        tryb=try0(1)-err(1)*(try0(2)-try0(1))/(err(2)-err(1));
        try0(1)=try0(2);
        try0(2)=tryb;
        err(1)=err(2);
        [err(2),rat]=shock_shock_3(try0(2));
        %{
        disp(['kip= ' num2str(kip) ', err2= ' num2str(err(2))...
            ', try2= ' num2str(try0(2))])
        %}
    else
        break
    end
end
if abs(err(2)) > tol
    disp('shock-shock iteration failed, job aborted. ')
    disp('check shock_cross routine at "function error3" ')
    LAST=1;
end

function wkm = error4(kp,tol)
% error4.M interaction results in a shock/contact/exp. fan
% configuration
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.
  
global uR aR LAST

lambda=uR(kp)+aR(kp);                                                       % reflected shock slope > than lambda                                                  % this provides a guess
try0(1)=lambda;
[err(1), wkm]=shock_shock_4(try0(1));
try0(2)=1.2*try0(1);
[err(2), wkm]=shock_shock_4(try0(2));
for kip=1:20
    if abs(err(2)) > tol
        tryb=try0(1)-err(1)*(try0(2)-try0(1))/(err(2)-err(1));
        try0(1)=try0(2);
        try0(2)=tryb;
        err(1)=err(2);
        [err(2), wkm]=shock_shock_4(try0(2));
        %{
                    disp(['kip= ' num2str(kip) ', err2= ' num2str(err(2))...
                        ', try2= ' num2str(try0(2))])
        %}
    else
        break
    end
end
if abs(err(2)) > tol
    disp('shock-shock iteration failed, job aborted. ')
    disp('check shock_cross routine at "function error4" ')
    LAST=1;
end
            

