function shock_elimination
% SHOCK_ELIMINATION.M - checks for weak shocks
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global shock_type kout kmax PL PR aR aL

weak_shock_tol=1.01;
weak_contact_tol=.01;

m=0;
for k=1:kmax
    if shock_type(k) == 'l'
        p_rat=exp(PL(k))/exp(PR(k));
        if p_rat < weak_shock_tol
            m=m+1;
            ksave(m)=k;
        end
    elseif shock_type(k) == 'r'
        p_rat=exp(PR(k))/exp(PL(k));
        if p_rat < weak_shock_tol
            m=m+1;
            ksave(m)=k;
        end
    elseif shock_type(k) == 'c'
        a_test=abs(aR(k)-aL(k));
        if a_test < weak_contact_tol;
            m=m+1;
            ksave(m)=k;
        end
    end
end

if m > 0    
    print_event
    for n=1:m
        k=ksave(n);
        if shock_type(k) ~= 'c'
            disp(['weak shock (k= ' num2str(k) ') detected at kout= ' num2str(kout) ])
        else
            disp(['weak contact (k= ' num2str(k) ') detected at kout= ' num2str(kout) ])
        end
        re_order(k)
    end
    print_shock
end

function re_order(ks)
% RE_ORDER.M - re-orders shocks
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global uL uR PL PR SL SR ZL ZR aL aR kmax x_s w shock_type RR RL QR QL

kmax=kmax-1;
if ks ~= kmax+1
    for k=ks:kmax
        kp=k+1;
        uL(k)=uL(kp);
        uR(k)=uR(kp);
        PL(k)=PL(kp);
        PR(k)=PR(kp);
        SL(k)=SL(kp);
        SR(k)=SR(kp);
        ZL(k)=ZL(kp);
        ZR(k)=ZR(kp);
        aL(k)=aL(kp);
        aR(k)=aR(kp);
        RR(k)=RR(kp);
        RL(k)=RL(kp);
        QR(k)=QR(kp);
        QL(k)=QL(kp);
        w(k)=w(kp);
        x_s(k)=x_s(kp);
        shock_type(k)=shock_type(kp);
    end
end
shock_index

        