function shock_index
% SHOCK_INDEX.M - builds shock indexes I & IS
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.
%
global eps0 I IS N NT kmax x x_s t 

I(:,1)=zeros(NT,1);                                                         % indexes to track shock presence in interval between mesh points [n,n+1]
I(:,2)=zeros(NT,1);                                                         % I(n,1) is the lowest shock k index in interval, I(n,2) is the highest k index

if kmax == 0
    return
end

b=pistonlaw(t);
c=exitboundary(t); 

for k=1:kmax

    ns=1+fix(N*(x_s(k)-b)/(c-b)); 
    ns=min([ns, N]);
    
    if x(ns) == x_s(k)
        x_s(k)=x_s(k)+eps0;
    end
    IS(k)=ns;                                                               % store mesh point to left of k-th shock
    if I(ns,1) == 0                                                         % true if this is the first shock in this cell
        I(ns,1)=k;                                                          % then, define first shock in cell [n,n+1]
        I(ns,2)=k;                                                          % and, in case this is the only shock, set last shock in cell to k
    else                                                                    % if not first shock in cell
        I(ns,2)=k;                                                          % define last shock in cell [n,n+1]
    end

    if x(N) < x_s(k)
        IS(k)=N;
        if I(N,1) == 0
            I(N,1)=k;
            I(N,2)=k;
        else
            I(N,2)=k;
        end
    end
end
