function shock_wall
% SHOCK_WALL.M - performs computation for shock-wall interactions
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
%
% Copyright 2005-2008 M.D.Salas.

global kmax w t LAST WALL NT shock_type gamma P u S a Z ct bt gm gp
global uL uR PL PR SL SR ZL ZR aL aR kout RR RL QR QL R Q bc_type

b=pistonlaw(t);
bt=pistonlawt(t);
c=exitboundary(t);
ct=exitboundaryt(t);
tol=10^-7;

if WALL(1) == 1                                                             % reflect shock from wall or contant pressure left end
    print_event

    if bc_type(1) == 0
        disp([ 'shock reflection from left-wall detected at kout= '...
            num2str(kout) ])
        disp([shock_type(1) '-shock (k= 1) reflected of wall at t= '...
            num2str(t) ' and x= ' num2str(b)])

        u1=uL(1);
        a1=aL(1);
        M1rel=(u1-w(1))/a1;
        SM1rel=M1rel^2;
        factor=1-2*gm*(1-1/SM1rel)/gp;                                      % this provides a good guess
        try0(1)=-factor*w(1)+bt;
        err(1)=shock_wall_1(try0(1));
        try0(2)=0.95*try0(1);
        err(2)=shock_wall_1(try0(2));
        for kip=1:20
            if abs(err(2)) > tol
                tryb=try0(1)-err(1)*(try0(2)-try0(1))/(err(2)-err(1));
                try0(1)=try0(2);
                try0(2)=tryb;
                err(1)=err(2);
                err(2)=shock_wall_1(try0(2));
                %{
            disp(['kip= ' num2str(kip) ', err2= ' num2str(err(2))...
                ', try2= ' num2str(try0(2))])
                %}
            else
                break
            end
        end
        if abs(err(2)) > tol
            print_event
            disp('shock-wall iteration failed, job aborted. ')
            disp('check shock_wall routine at (a). ')
            LAST=1;
        end
        w(1)=try0(2);

        p2=exp(PR(1));
        u2=uR(1);
        a2=aR(1);
        Z2=ZR(1);
        rho2=p2/Z2;
        SM2rel=((u2-w(1))/a2)^2;

        p3p2=(2*gamma*SM2rel-gm)/gp;
        p3=p3p2*p2;
        rho3rho2=gp*SM2rel/(gm*SM2rel+2);
        rho3=rho3rho2*rho2;
        u3=w(1)+(u2-w(1))/rho3rho2;
        S3=log(p3)-gamma*log(rho3);

        PL(1)=log(p3);
        SL(1)=S3;
        uL(1)=u3;
        ZL(1)=exp((SL(1)+gm*PL(1))/gamma);
        aL(1)=sqrt(gamma*ZL(1));
        u(1)=uL(1);
        P(1)=PL(1);
        S(1)=SL(1);
        Z(1)=ZL(1);
        a(1)=aL(1);
        shock_type(1)='l';
        RR=2*aR/gm+uR;
        RL=2*aL/gm+uL;
        QR=2*aR/gm-uR;
        QL=2*aL/gm-uL;
        shock_index
    else                                                                    % reflection from constant pressure exit, requires bc_type(1)=1
        disp([ 'shock reflection from left- constant pressure -exit detected at kout= '...
            num2str(kout) ])
        disp([shock_type(1) '-shock (k= 1) eliminated at t= '...
            num2str(t) ' and x= ' num2str(b)])
        S(1)=SR(1);
        rho1=exp((P(1)-S(1))/gamma);
        p1=exp(P(1));
        Z(1)=p1/rho1;
        a(1)=sqrt(gamma*Z(1));
        u(1)=2*a(1)/gm-QR(1);
        R(1)=2*a(1)/gm+u(1);
        Q(1)=2*a(1)/gm-u(1);
        ks=1;
        re_order(ks)      
    end
    WALL(1)=0;
end

if (WALL(2) == 1) && (shock_type(kmax) == 'l')                              % reflect shock from wall or constant pressure at right end
    print_event
    if bc_type(2) == 0
        disp([ 'shock reflection from right-wall detected at kout= '...
            num2str(kout) ])
        disp([shock_type(kmax) '-shock (k= ' num2str(kmax)...
            '), reflected of wall at t= ' num2str(t) ' and x= ' num2str(c)])
        u1=uR(kmax);
        a1=aR(kmax);
        M1rel=(u1-w(kmax))/a1;
        SM1rel=M1rel^2;
        factor=1-2*gm*(1-1/SM1rel)/gp;                                      % this provides a good guess
        try0(1)=-factor*w(kmax)+ct;
        err(1)=shock_wall_2(try0(1));
        try0(2)=0.95*try0(1);
        err(2)=shock_wall_2(try0(2));
        for kip=1:20
            if abs(err(2)) > tol
                tryb=try0(1)-err(1)*(try0(2)-try0(1))/(err(2)-err(1));
                try0(1)=try0(2);
                try0(2)=tryb;
                err(1)=err(2);
                err(2)=shock_wall_2(try0(2));
                %{
            disp(['kip= ' num2str(kip) ', err2= ' num2str(err(2))...
                ', try2= ' num2str(try0(2))])
                %}
            else
                break
            end
        end
        if abs(err(2)) > tol
            print_event
            disp('shock-wall iteration failed, job aborted. ')
            disp('check shock_wall routine at (b). ')
            LAST=1;
        end
        w(kmax)=try0(2);

        p2=exp(PL(kmax));
        u2=uL(kmax);
        a2=aL(kmax);
        Z2=ZL(kmax);
        rho2=p2/Z2;
        SM2rel=((u2-w(kmax))/a2)^2;

        p3p2=(2*gamma*SM2rel-gm)/gp;
        p3=p3p2*p2;
        rho3rho2=gp*SM2rel/(gm*SM2rel+2);
        rho3=rho3rho2*rho2;
        u3=w(kmax)+(u2-w(kmax))/rho3rho2;
        S3=log(p3)-gamma*log(rho3);

        PR(kmax)=log(p3);
        SR(kmax)=S3;
        uR(kmax)=u3;
        ZR(kmax)=exp((SR(kmax)+gm*PR(kmax))/gamma);
        aR(kmax)=sqrt(gamma*ZR(kmax));
        u(NT)=uR(kmax);
        P(NT)=PR(kmax);
        S(NT)=SR(kmax);
        Z(NT)=ZR(kmax);
        a(NT)=aR(kmax);
        shock_type(kmax)='r';
        RR=2*aR/gm+uR;
        RL=2*aL/gm+uL;
        QR=2*aR/gm-uR;
        QL=2*aL/gm-uL;
        shock_index
    else                                                                    % reflection from contant pressure boundary, requires bc_type(2)=1
        disp([ 'shock reflection from right- constant pressure -exit detected at kout= '...
            num2str(kout) ])
        disp([shock_type(kmax) '-shock (k= ' num2str(kmax)...
            '), eliminated at t= ' num2str(t) ' and x= ' num2str(c)])
        
        S(NT)=SL(kmax);
        rhoNT=exp((P(NT)-S(NT))/gamma );
        pNT=exp(P(NT));
        Z(NT)=pNT/rhoNT;
        a(NT)=sqrt(gamma*Z(NT));
        u(NT)=-2*a(NT)/gm+RL(kmax);
        R(NT)=2*a(NT)/gm+u(NT);
        Q(NT)=2*a(NT)/gm-u(NT);
        kmax=kmax-1;
        shock_index
    end
    
    if (WALL(2) == 1) && (shock_type(kmax) == 'c')                          % let contact exit right end
        print_event
        disp([ 'contact exits right exit detected at kout= '...
            num2str(kout) ])
        disp([shock_type(kmax) '-shock (k= kmax) eliminated at t= '...
            num2str(t) ' and x= ' num2str(b)])
        kmax=kmax-1;
        shock_index
    end
    WALL(2)=0;
end

function error = shock_wall_1(w_guess)
% SHOCK_WALL_1.M - solves r-shock/wall interaction
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global uR aR bt gm gp                                                       %wall,bt   l-shock
                                                                            %    |    / k=1
u2=uR(1);                                                                   %    | 3 /  2
a2=aR(1);                                                                   %    |  /
M2rel=(u2-w_guess)/a2;                                                      %    | / 
SM2rel=M2rel^2;                                                             %-----------------
                                                                            %    | \                                                                                                                         %    |  \r-shock
rho3rho2=gp*SM2rel/(gm*SM2rel+2);                                           %    |  \  2
u3=w_guess+(u2-w_guess)/rho3rho2;                                           %    |   \ r-shock, k=1

error=bt-u3;

function error = shock_wall_2(w_guess)
% SHOCK_WALL_2.M - solves l-shock/wall interaction
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global uL aL kmax ct gm gp
                                                                            % r-shock   wall, ct   
                                                                            % k=kmax\    | 
u2=uL(kmax);                                                                %        \ 3 | 
a2=aL(kmax);                                                                %    2    \  | 
M2rel=(u2-w_guess)/a2;                                                      %          \ | 
SM2rel=M2rel^2;                                                             %------------------
                                                                            %          / |                                                                                                                          %    |  \r-shock
rho3rho2=gp*SM2rel/(gm*SM2rel+2);                                           %    2    /  |  
u3=w_guess+(u2-w_guess)/rho3rho2;                                           %  k=kmax/   |   
                                                                            %  l-shock
error=ct-u3;                                                                

function re_order(ks)
% RE_ORDER.M - re-orders shocks
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global uL uR PL PR SL SR ZL ZR aL aR kmax x_s w shock_type RR RL QR QL

kmax=kmax-1;
if ks ~= kmax+1
    for k=ks:kmax
        kp=k+1;
        uL(k)=uL(kp);
        uR(k)=uR(kp);
        PL(k)=PL(kp);
        PR(k)=PR(kp);
        SL(k)=SL(kp);
        SR(k)=SR(kp);
        ZL(k)=ZL(kp);
        ZR(k)=ZR(kp);
        aL(k)=aL(kp);
        aR(k)=aR(kp);
        RR(k)=RR(kp);
        RL(k)=RL(kp);
        QR(k)=QR(kp);
        QL(k)=QL(kp);
        w(k)=w(kp);
        x_s(k)=x_s(kp);
        shock_type(k)=shock_type(kp);
    end
end
shock_index








