% program blunt
% two-dimensional symmetric and axisymmetric blunt body code
%{
    A SHOCK-FITTING PRIMER
    Boundary Shock-Fitting Method
    Copyright (c) 2005 - 2009 by M.D.Salas
%}
clear all

global na ma ka ja lb ld  gamma stab p ell nc mc blunt_movie
global themax mach x0 angle le method supcir k

%{
USER INPUTS:
runnum= run number
na=     intervals between shock and body
ma=     intervals along body
ka=     total number of steps
method=1 standard shock-fitting method
method=2 nuovo shock-fitting method
ja=0    no movie, otherwise do movie at ja steps; other outputs supressed
blunt_movie=    1 for shock & sonic line shape
blunt_movie=    2 for shock speed
jb=0    don't read previous solution
jb=1    read previous solution and interpolate to finer mesh
jb=2    read previous solution on grid same size, continuation run
jc=0    don't save solution for continuation
jc=1    save solution for continuation
lb=n    to print partial outputs every n steps
ld=1    for two-dimensional flow; ellipse
ld=2    for axisymmetric flow; ellipsoid or paraboloid
le=1    geometry selection: ellipse/ellipsoid
le=2    geometry selection: paraboloid (only the axisymmetric case is build in), set ld=2
le=3    geometry selection: super-circle or super-sphere, x0 not used, set ell=1
supcir= power for super-circle/sphere, should be even >= 2,(2 is a circle or sphere), can be a fraction
ell=    ellipticity ratio=b/a where (x/a)^2+(y/b)^2=1 and b=1; needed for le=1
mach=   free stream Mach number
gamma=  ratio of specific heats
stab=   Courant Number (0.75)
themax= maximum theta in degrees, outflow-line
x0=     displacement for (r,theta) coordinate origin
angle=  angle in degrees for wedge following elliptical/ellisoidal nose (optional), not used with paraboloid
loadrunint= name of file previously saved, interpolate to solution finer grid
loadruncon= name of file previously saved, continuation run on grid same size
saverunana= name of file to save solution for further analysis
saveruncon= name of file to save solution for contiuation

%}

% grid & step parameters:
runnum=81;
loadrunint='run81';
loadruncon='run81';
saverunana='prun81';
saveruncon='run81';
na=20;
ma=40;
ka=3000;
method=1;
stab=.75;

% movie option:
ja=0;  % every ja frames displayed 
blunt_movie=1;

% read & write selection:
jb=0;
jc=0;
lb=200;

% geometry parameters:
le=3;     % 1=elliptical x-section, 2=parabolic x-section, 3=super-circle x-section
ld=2;     % 1=plane flow, 2= axisymmetric
ell=1;    % ellipticity ratio, set ell=1 for le=3
supcir=4; % super-cicle x-section exponent, 2->circle, can be a fraction
themax=70;
x0=0;
angle=-10;

% flow parameters:
mach=2;
gamma=1.4;

% persistant variables
save blunt_dum runnum ka ja blunt_movie jb jc lb saverunana

k=0;
blunt_initial

%--------------INTERPOLATE PREVIOUS SOLUTION TO FINER GRID----------------
%{
load statement should read: load runxx c s u p v w
where xx is a previous run number
%}
if jb == 1
    load (loadrunint, 'c', 's', 'u', 'p', 'v', 'w')
    blunt_interpolate
    
    %--------------------------CONTINUATION RUN-------------------------------
elseif jb == 2
    %{
read previous solution on grid same size, load statement should read:
load runxx where xx is a previous run number
    %}
    load (loadruncon)
    load blunt_dum runnum ka ja blunt_movie jb jc lb saverunana
end
%---------------------------START RUN-------------------------------------

blunt_run

%---------------------------SAVE SOLUTION FOR ANALYSIS--------------------
%{
save statement should read: save prunxx press c b cxst cyst
where xx is current run number
%}
for n=2:nc
    for m=2:mc
        press(n,m)=exp(p(n,m));
    end
end
%save (saverunana, 'press', 'c', 'b', 'cxst', 'cyst', 'na', 'ma', 'pst')

%---------------------------SAVE SOLUTION FOR CONTINUATION----------------
%{
save statement should read: save runxx
where xx is current run number
%}
if jc == 1
    save (saveruncon)
end
disp(' ')
disp('______________________________________________________________')
disp('Memory Usage Report')
feature('memstats'); % memory report