function blunt_curvature
%
% evaluate shock curvature
%{
    Boundary Shock-Fitting Method
    Copyright (c) 2005 - 2009 by M.D.Salas
%}
global c mc mcm nfig y sigma angst k 

% find curvature at y=0 with local quartic fit
cl=[c(4) c(3), c(2), c(3) c(4)];
yl=[-y(4), -y(3), y(2), y(3), y(4)];
pl=polyfit(yl,cl,4);
curv=(pl(5)-2*pl(3))/pl(5)^2;
disp('  ')
disp('Shock radius of curvature on symmetry line (from local fit):')
fprintf('%8.5f \t \n',1/curv);

ys=angst*pi/180; % y at shock sonic point
for m=4:mc % find mesh point "ms" near sonic point
    ms=m;
    if (y(m)-ys)*(y(m-1)-ys) <= 0
        break
    end
end

% find shock slope and shock radius at y=y(ms)
cl=[c(ms-2), c(ms-1), c(ms), c(ms+1), c(ms+2)];
yl=[y(ms-2), y(ms-1), y(ms), y(ms+1), y(ms+2)];
pl=polyfit(yl,cl,4);
rys=((4*pl(1)*yl(3)+3*pl(2))*yl(3)+2*pl(3))*yl(3)+pl(4);
rs=c(ms);

% coefficients for a quartic fit that matches standoff distance, curvature,
% and slope at y=0, and radius and slope at y=y(ms)

p5=c(2);
p4=0;
p3=c(2)*(1-curv*c(2))/2;
p2=-((2*p3*y(ms)+rys)*y(ms)+4*(c(2)-rs))/y(ms)^3;
p1=((p3*y(ms)+rys)*y(ms)+3*(c(2)-rs))/y(ms)^4;
sum=0;
maxdev=0;
for m=2:mc % create fit
    temp(m)=y(m)*(y(m)*(y(m)*(y(m)*p1+p2)+p3))+p5;
    dev=abs(temp(m)/c(m)-1);
    if dev > maxdev
        maxm=m;
        maxdev=dev;
    end
    sum=sum+dev;
end
temp(1)=temp(3);

% find mean deviation
meandev=sum/mcm;
disp('  ')
disp('global shock curve fit % mean deviation:')
fprintf('%8.5f \t \n',100*meandev);
disp('global shock curve fit max % mean deviation:')
fprintf('per cent max dev= ')
fprintf('%8.5f \t ',100*maxdev);
fprintf('per cent at Y= ')
fprintf('%8.5f \t \n',y(maxm));
disp(['shock curve fit coefficients: r=p1*Y^4+p2*Y^3+p3*Y^2+p4*Y+p5, Y[',num2str(y(2)),',',num2str(y(mc)),']'])
fprintf('p1= ')
fprintf('%8.5f \t \n',p1);
fprintf('p2= ')
fprintf('%8.5f \t \n',p2);
fprintf('p3= ')
fprintf('%8.5f \t \n',p3);
fprintf('p4= ')
fprintf('%8.5f \t \n',p4);
fprintf('p5= ')
fprintf('%8.5f \t \n',p5);

for m=2:mc % find curvature
    %thep(m)=(pi-y(m))*180/pi;
    tempt2=(-y(m)*(y(m)*(4*p1*y(m)+3*p2)+2*p3))^2;
    temptt=y(m)*(12*p1*y(m)+6*p2)+2*p3;
    curvature(m)=(temp(m)*(temp(m)-temptt)+2*tempt2)/(temp(m)^2+tempt2)^1.5;
end
curvature(1)=curvature(3);
%thep(1)=(pi-y(1));

nfig=nfig+1;
figure(nfig)
cv=[c(2),c(mcm)];
yv=[angst, angst];
plot(y*180/pi,temp,'k-',y*180/pi,c,'ko',...
    y(maxm)*180/pi,c(maxm),'r+',yv,cv,'r-')
text(yv(1),cv(1),'sonic point')
title(['SHOCK RADIUS at k=',num2str(k)])
legend('curve fit','original c','max dev')
xlabel('Y (degrees)')
ylabel('shock radius')
grid on

curvv=[curvature(2), curvature(mcm)];
nfig=nfig+1;
figure(nfig)
plot(y*180/pi,curvature,'k-o',yv,curvv,'r-')
text(yv(1),curvv(1),'sonic point')
xlabel('Y (degrees)')
ylabel('shock curvature')
title(['SHOCK CURVATURE at k=',num2str(k)])
grid on

nfig=nfig+1;
figure(nfig)
sigmav=[sigma(2), sigma(mcm)];
plot(y*180/pi,sigma,'ko-',yv,sigmav,'r-')
text(yv(1),sigmav(1),'sonic point')
xlabel('Y (degrees)')
ylabel('sigma')
title(['SHOCK SLOPE at k=',num2str(k)])
grid on

% release local variables
clear temp curvature thep


 





    