% subroutine initial_blunt.m
% two-dimensional symmetric and axisymmetric blunt body
%{
Boundary Shock-Fitting Method
Copyright (c) 2005 - 2009 by M.D.Salas
%}
global z y costhe sinthe b c d w wn bpr sigma u v s p ga gb gc gd ge gf
global ben1 ben2 t cosin RB ell mach_target dmach u0 gamma stab ma ka ld
global dz dy nc ncm mc mcm smin pst rst k j time na ddy ddz angle hst
global themax shape p1 r1 mach x0 le supcir 

%--------------------------------------------------------------------------
% save mach number, needed for low mach cases
mach_target=mach;
%
% for Mach numbers below 1.2, start at 1.2 and slowly bring Mach down
%
if mach < 1.2
    dmach=1.2-mach;
    mach=1.2;
end
%--------------------------------------------------------------------------
mc=ma+2;
nc=na+2;
ncm=nc-1;
mcm=mc-1;
%
time=0;
k=0;
j=0;
themax=themax*pi/180;
gb=1./(gamma-1.);
ga=gamma*gb;
gd=.5/gb;
ge=gd+1.;
gc=ge/gd;
gf=sqrt(gamma);
gg=ge^(-ga);
dz=1./na;
dy=themax/ma;
u0=mach*gf;
smin=mach^2;
hst=ga+gamma*.5*smin;
ddy=1./dy;
ddz=1./dz;
%
for m=1:mc
    wtsave(m)=0;
    y(m)=dy*(m-2);
    the=pi-y(m);
    costhe(m)=cos(the);
    sinthe(m)=sin(the);
    if m ~= 2
        cosin(m)=costhe(m)/sinthe(m);
    end
end
costhe(2)=-1.;
%
for n=2:nc
    z(n)=dz*(n-2);
end
y(1)=-y(3);
%
angle=angle*pi/180;
%
%    define body geometry
%
jj=0;
theta=0;

blunt_geom(jj,theta)

%{
fitting an initial shock, assumed at rest
standoff distance correlated from previous runs
%}
p1=(2.*ga*smin-1.)/gc;
r1=(gc*p1+1.)/(gc+p1);
%
if le == 1
    standoff_dist % get standoff distance d(2)
elseif le == 2
    standoff_dist_parb
elseif le == 3
    standoff_dist
    d(2)=1.2*d(2);
end

%
c(2)=b(2)+d(2);
sigma(2)=0.;
w(2)=0.;
u(nc,2)=-u0*(1.-(smin-1.)/smin/ge);
v(nc,2)=0.;
p(nc,2)=log(p1);
s(nc,2)=p(nc,2)-gamma*log(r1);
t(nc,2)=p1/r1;
%
if ld == 1 % ellipse
    shape=1.;
    if mach > 5
        c1=1-.05*mach;
        shape=max(.5,c1);
        if mach > 10
            shape=.5 -0.0125*mach;
        end
        if mach > 20
            shape=-.25*(mach-50)/30+.1*(mach-20)/30;
            shape=max(.1,shape);
        end
    end
elseif ld == 2 % ellipsoid
    shape=0;
    if mach <= 2
        shape=(-2.8652*mach+11.065)*mach-8.2687;
    elseif mach >2 && mach <= 15
        cc=[-6.5168e-006,0.00044443,-0.011743,0.15227,-1.0397,4.0279];
        for m=1:5
            shape=(shape+cc(m))*mach;
        end
        shape=shape+cc(6);
    else
        cb=[2.6418e-10,-5.7029e-8,5.0613e-6,-.00023667,.0061751,-.086391,...
            .52979,1.851e-5];
        for m=1:7
            shape=(shape+cb(m))*mach;
        end
        shape=shape+cb(8);
    end
    shape=max(shape,.1);
end
%
bs=.047*mach*shape;
param=1; % ellipse/ellipsoid or super-circle/sphere
if le == 2
    param=0.7; % paraboloid
end
for m=3:mc
    bss=param*bs*sinthe(m)^2;
    % parabolic curve fit for shock, limit (theta->pi), c(2)=c(2)
    c(m)=(costhe(m)+sqrt(costhe(m)^2+4.*c(2)*bss))/(2.*bss);
    cot=2.*bs*c(m)*sinthe(m);
    sigma(m)=c(m)*(sinthe(m)+costhe(m)*cot)/(costhe(m)-sinthe(m)*cot);
    dum=smin/(1.+cot^2);
    ps=(2.*ga*dum-1.)/gc;
    rs=(gc*ps+1.)/(gc+ps);
    p(nc,m)=log(ps);
    s(nc,m)=p(nc,m)-gamma*log(rs);
    t(nc,m)=ps/rs;
    ucart=u0*(1.-(dum-1.)/smin/ge);
    vcart=(u0-ucart)*cot;
    u(nc,m)=ucart*costhe(m)+vcart*sinthe(m);
    v(nc,m)=-ucart*sinthe(m)+vcart*costhe(m);
    wn(m)=0.;
    w(m)=0.;
    d(m)=c(m)-b(m);
end

%
w(1)=0.;
c(1)=c(3);
sigma(1)=-sigma(3);
%
%    locate sonic point on body
%
if le == 1
    tstar=sang_ell; % sonic angle
elseif le == 2
    tstar=sang_parb;
elseif le == 3
    tstar=pi*50/180;
end
%
sstar=sin(tstar); % good for x0=0, needs shift if x0 not 0
jj=1;
blunt_geom(jj,tstar) % obtain RB, radius to sonic point from x=0
yd2=(RB*sstar)^2;
xd2=(RB*cos(tstar)+x0)^2;
rbar=sqrt(yd2+xd2); % radius from center at x0
sstar=RB*sstar/rbar; % angle from center at x0
tstar=asin(sstar); % sonic angle
%
%    distribute values of physical parameters between shock and body
%
% first define stagnation values
%
sm=u(nc,2)^2*r1/gamma/p1;
dem=1.+sm*gd;
rst=r1*dem^gb;
pst=p1*dem^ga;
u(2,2)=0.;
v(2,2)=0.;
p(2,2)=log(pst);
t(2,2)=pst/rst;
s(2,2)=p(2,2)-gamma*log(rst);
%
pw=2.22*ell;
if le == 2
    pw=2.22;
end
for m=3:mc
    sms=(y(m)/tstar)^pw;
    dum=1.+gd*sms;
    pb=pst/dum^ga;
    rb=rst/dum^gb;
    p(2,m)=log(pb);
    s(2,m)=p(2,m)-gamma*log(rb);
    t(2,m)=pb/rb;
    asq=gamma*t(2,m);
    q=sqrt(asq*sms);
    bprb=bpr(m)/b(m);
    ben1(m)=-1./sqrt(1.+bprb^2);
    ben2(m)=-ben1(m)*bprb;
    v(2,m)=q*ben1(m);
    u(2,m)=v(2,m)*bprb;
end
ben1(2)=-1.;
ben2(2)=0.;
ben1(1)=ben1(3);
ben2(1)=-ben2(3);
for m=2:mc
    xb(m-1)=b(m)*costhe(m)+x0;
    yb(m-1)=b(m)*sinthe(m);
    dum=u(nc,m)-u(2,m);
    dam=v(nc,m)-v(2,m);
    dem=s(nc,m)-s(2,m);
    dom=p(nc,m)-p(2,m);
    %dom=t(nc,m)-t(2,m);
    for n=3:ncm
        p(n,m)=p(2,m)+dom*z(n);
        s(n,m)=s(2,m)+dem*z(n);
        u(n,m)=u(2,m)+dum*z(n)^2;
        v(n,m)=v(2,m)+dam*z(n)^2;
        t(n,m)=exp(p(n,m)/ga+s(n,m)/gamma);
    end
end
for n=2:nc
    u(n,1)=u(n,3);
    p(n,1)=p(n,3);
    s(n,1)=s(n,3);
    v(n,1)=-v(n,3);
end
%-----------------------------------INITIAL OUTPUT-------------------------
disp(' ')
disp('PROGRAM BLUNT')
if ld==2
    disp('Axisymmetric blunt body, perfect gas ')
else
    disp('Two-dimensional blunt body, perfect gas ')
end
disp(' ')
disp(['Run No.=' num2str(runnum)])
disp(['Mach=' num2str(mach) ', gamma=' num2str(gamma) ', stab=' num2str(stab)...
    ', themax=' num2str(themax*180/pi) ' deg.'])
disp(['na=' num2str(na) ', ma=' num2str(ma) ', ka=' num2str(ka)])
%
%    output geometry parameters
%
if le == 1
    if ld == 1
        disp(['The body is an ellipse, ell= ' num2str(ell) ', origin shifted to x0=' num2str(x0)])
    else
        disp(['The body is an ellipsoid, ell= ' num2str(ell) ', origin shifted to x0=' num2str(x0)])
    end
    disp(['Followed by a ' num2str(angle*180/pi) ' degree wedge'])
elseif le == 2
    disp(['The body is a paraboloid origin shifted to x0=' num2str(x0)])
elseif le == 3
    if ld == 1
        disp(['The body is a super-circle, supcir=' num2str(supcir)])
    else
       disp(['The body is a super-sphere, supcir=' num2str(supcir)]) 
    end
end
%
% cell aspect ratio 
%
drasp=dz*(c(2)-b(2));
rav=.5*(b(2)+b(3));
dyasp=rav*dy;
aspectratio=drasp/dyasp;
disp(' ')
disp(['Cell aspect ratio near stagnation point=' num2str(aspectratio)])
drasp=dz*(c(mc)-b(mc));
rav=.5*(c(mc)+c(mcm));
dyasp=rav*dy;
aspectratio1=drasp/dyasp;
disp(['Cell aspect ratio near shock/outflow=' num2str(aspectratio1)])
disp(['Ratio of cell aspect ratio=' num2str(aspectratio1/aspectratio)])
