% subroutine interpolate_blunt
% two-dimensional symmetric and axisymmetric blunt body
% interpolate solution to finer grid
%
% Boundary Shock-Fitting Method
%
% Copyright (c) 2005 - 2009 by M.D.Salas
%
global  c w wn sigma u v s p un vn sn pn t ma gamma ga nc mc mcm ddy   

%
mm=2;
for m=2:2:mc
    wn(m)=w(mm);
    cn(m)=c(mm);
    mm=mm+1;
end
mm=2;
for m=3:2:mc
    wn(m)=.5*(w(mm)+w(mm+1));
    cn(m)=.5*(c(mm)+c(mm+1));
    mm=mm+1;
end
cn(1)=cn(3);
wn(1)=wn(3);

nn=2;
for n=2:2:nc % injection
    mm=2;
    for m=2:2:mc
        un(n,m)=u(nn,mm);
        pn(n,m)=p(nn,mm);
        vn(n,m)=v(nn,mm);
        sn(n,m)=s(nn,mm);
        mm=mm+1;
    end
    nn=nn+1;
end
nn=2;
for n=2:2:nc % interpolate in m
    mm=2;
    for m=3:2:mc
        un(n,m)=.5*(u(nn,mm)+u(nn,mm+1));
        pn(n,m)=.5*(p(nn,mm)+p(nn,mm+1));
        vn(n,m)=.5*(v(nn,mm)+v(nn,mm+1));
        sn(n,m)=.5*(s(nn,mm)+s(nn,mm+1));
        mm=mm+1;
    end
    nn=nn+1;
end
mm=2;
for m=2:2:mc % interpolate in n
    nn=2;
    for n=3:2:nc
        un(n,m)=.5*(u(nn,mm)+u(nn+1,mm));
        pn(n,m)=.5*(p(nn,mm)+p(nn+1,mm));
        vn(n,m)=.5*(v(nn,mm)+v(nn+1,mm));
        sn(n,m)=.5*(s(nn,mm)+s(nn+1,mm));
        nn=nn+1;
    end
    mm=mm+1;
end
mm=2;
for m=3:2:mc % interpolate in n & m
    nn=2;
    for n=3:2:nc
        un(n,m)=.25*(u(nn,mm)+u(nn+1,mm)+u(nn,mm+1)+u(nn+1,mm+1));
        pn(n,m)=.25*(p(nn,mm)+p(nn+1,mm)+p(nn,mm+1)+p(nn+1,mm+1));
        vn(n,m)=.25*(v(nn,mm)+v(nn+1,mm)+v(nn,mm+1)+v(nn+1,mm+1));
        sn(n,m)=.25*(s(nn,mm)+s(nn+1,mm)+s(nn,mm+1)+s(nn+1,mm+1));
        nn=nn+1;
    end
    mm=mm+1;
end

for l=1:10;
    for m=2:mcm
        cn(m)=.25*(cn(m-1)+2*cn(m)+cn(m+1));
        wn(m)=.25*(wn(m-1)+2*wn(m)+wn(m+1));
    end
end

for m=2:mc
    c(m)=cn(m);
    w(m)=wn(m);
    for n=2:nc
        p(n,m)=pn(n,m);
        u(n,m)=un(n,m);
        v(n,m)=vn(n,m);
        s(n,m)=sn(n,m);
        t(n,m)=exp(p(n,m)/ga+s(n,m)/gamma);
    end
end
for n=2:nc
    u(n,1)=u(n,3);
    p(n,1)=p(n,3);
    s(n,1)=s(n,3);
    v(n,1)=-v(n,3);
end
c(1)=c(3);
w(1)=w(3);

for m=3:mcm
    sigma(m)=(-c(m-2)+6.*c(m-1)-3.*c(m)-2.*c(m+1))/6.*ddy;
end
sigma(2)=0.;
sigma(1)=-sigma(3);
sigma(mc)=2*sigma(mcm)-sigma(ma);

for m=2:mcm
    sigma(m)=.25*(sigma(m-1)+2*sigma(m)+sigma(m+1));
end
sigma(2)=0.;
sigma(1)=-sigma(3);
sigma(mc)=2*sigma(mcm)-sigma(ma);
