function blunt_out
%
% output routine
%{
    Boundary Shock-Fitting Method
    Copyright (c) 2005 - 2009 by M.D.Salas
%}
global z y costhe sinthe b c d w u v s p t nfig cxst cyst ma 
global u0 nc mc mcm pst k time themax mach gf x0 runlabel le angst

%
disp(' ')
disp('Output from BLUNT_OUT: ')
disp(' ')
        
nfig=nfig+1;
figure (nfig)

for m=2:mc
    xb(m-1)=b(m)*costhe(m)+x0;
    yb(m-1)=b(m)*sinthe(m);
    xs(m-1)=c(m)*costhe(m)+x0;
    ys(m-1)=c(m)*sinthe(m);
end
if le == 2
    stndoff=0.1359+0.8998*(1/(mach^2-1)); % paraboloid theoretical standoff distance
    shkcurvth=1.415+3.156*(1/(mach^2-1))+1.22*(1/(mach^2-1))^2; % paraboloid theoretical shock curvature
    for m=1:4:mcm
        xtheory(m)=ys(m)^2/(2*shkcurvth)-(stndoff+2);
        ytheory(m)=ys(m);
    end
    plot(xtheory,ytheory,'ko')
    legend('empirical shock shape')
    hold on
end

plot(xb,yb,'k-',xs,ys,'k-')
axis equal
hold on

%
% find sonic line
%
nn=0;
for n=2:nc
    for m=2:mc
        q2=u(n,m)^2+v(n,m)^2;
        ach(m)=sqrt(q2/t(n,m))/gf;
    end
    for m=3:mc
        if (ach(m-1)-1)*(ach(m)-1) <= 0
            zst=z(n);
            nn=nn+1;
            yso(nn)=(y(m-1)*(ach(m)-1)-y(m)*(ach(m-1)-1))/(ach(m)-ach(m-1));
            cst=(c(m-1)*(ach(m)-1)-c(m)*(ach(m-1)-1))/(ach(m)-ach(m-1));
            bst=(b(m-1)*(ach(m)-1)-b(m)*(ach(m-1)-1))/(ach(m)-ach(m-1));
            rso(nn)=zst*(cst-bst)+bst;
            cxst(n-1)=rso(nn)*cos(pi-yso(nn))+x0;
            cyst(n-1)=rso(nn)*sin(pi-yso(nn));
        end
    end
    if n == 2
        Rb=sqrt(cyst(1)^2+cxst(1)^2);
        angst=asin(cyst(1)/Rb)*180/pi; % sonic angle from x=0 center
    end
end

plot(cxst,cyst,'k--')
axis tight
title(['Shock and sonic line shape at k=',num2str(k)])
xlabel('x')
ylabel('y')
hold off

pl=polyfit(rso,yso,4);
nlast=length(rso);
for nn=1:nlast
    temp(nn)=rso(nn)*(rso(nn)*(rso(nn)*(rso(nn)*pl(1)+pl(2))+pl(3))+pl(4))+pl(5);
end

nfig=nfig+1;
figure(nfig)
plot(temp*180/pi,rso,'ko',yso*180/pi,rso,'k-')
ylabel('radius')
xlabel('sonic line Y (degrees)')
title(['sonic line curve fit at k=',num2str(k)])
legend('curve fit','original curve')
grid on

%
if k > 0
    if runlabel == 1
        disp('Shock computation based on bicharacteristic and R-H jumps')
    else  
        disp('Shock computation based on Riemann variable and R-H jumps')
    end
    %{
    disp('Shock wave coordinates: xsh, ysh')
    for m=1:mcm
        fprintf('%8.5f %8.5f \n',xs(m),ys(m));
    end
    disp(' ')
    disp('Sonic line coordinates: xso, yso')
    for n=1:nc-1
        fprintf('%8.5f %8.5f \n',cxst(n),cyst(n));
    end
    %}
    disp(' ')
end
disp(['sonic line curve fit coefficients: Y=p1*r^4+p2*r^3+p3*r^2+p4*r+p5, r[',num2str(rso(1)),',',num2str(rso(nlast)),']'])
fprintf('p1= ')
fprintf('%8.5f \t \n',pl(1));
fprintf('p2= ')
fprintf('%8.5f \t \n',pl(2));
fprintf('p3= ')
fprintf('%8.5f \t \n',pl(3));
fprintf('p4= ')
fprintf('%8.5f \t \n',pl(4));
fprintf('p5= ')
fprintf('%8.5f \t \n',pl(5));
disp(' ')

%
nfig=nfig+1;
%
% plot symmetry line pressure
%
for n=2:nc
    pp(n-1)=exp(p(n,2))/pst;
    zz(n-1)=z(n);
end

figure(nfig)
plot(zz,pp,'k-')
grid on
title(['Symmetry Line Pressure at k=',num2str(k)])
xlabel('Z')
ylabel('p')
%
dam=w(2)^2;
for m=3:mcm
    dam=dam+w(m)^2;
end
range=sqrt(dam/ma)/u0;
error=pp(1)-1.;
if le == 1
    disp(['Body sonic angle from (0,0)= ' num2str(angst) ' degrees'])
else
    disp(['Body sonic angle from (2,0)= ' num2str(angst) ' degrees'])
end
disp([ 'time=' num2str(time) ', d(2)=' num2str(d(2))...
    ', range=' num2str(range) ', w(2)=' num2str(w(2))...
    ', error in stag. pressure= ' num2str(error)])
disp(' ')
if k > 1
    disp('  theta   body pressure');
    for m=2:mc
        the=(pi-y(m))*180/pi;
        pbody=exp(p(2,m));
        fprintf('%8.5f %8.5f \n',the,pbody);
    end
    disp(' ')
    disp(' radius  symm line pressure');
    for n=2:nc
        rad=b(2)+z(n)*(c(2)-b(2));
        psym=exp(p(n,2));
        fprintf('%8.5f %8.5f \n',rad,psym);
    end
end
    %
    nfig=nfig+1;
    figure(nfig)
    %
    % plot Mach number along body
    %
    for m=2:mc
        q2=u(2,m)^2+v(2,m)^2;
        achp(m-1)=sqrt(q2/t(2,m))/gf;
        thep(m-1)=y(m)*180./pi;
    end
    thep1=[0, themax*180/pi];
    achp1=[1, 1];
    plot(thep,achp,'k-', thep1, achp1, 'k--')
    grid on
    title(['Surface Mach No. at k=',num2str(k)])
    xlabel('Y, deg.')
    ylabel('M')
    
    nfig=nfig+1;
    figure(nfig)
    %
    % plot entropy along body
    %
    for m=2:mc
        splot(m-1)=s(2,m);
    end
    plot(thep,splot,'k-')
    grid on
    title(['Surface entropy at k=',num2str(k)])
    xlabel('Y, deg.')
    ylabel('S')
    
    %__________________________________________________________________________
    % PRESSURE CONTOUR PLOT
    %__________________________________________________________________________
    nfig=nfig+1;
    figure(nfig);
    plot(xb,yb,'k-',x0,0.,'r+');
    hold on;
    plot(xs,ys,'k-',cxst,cyst,'k--');
    axis equal
    axis tight
    xlabel('x')
    ylabel('y')
    title(['Isobars at k=',num2str(k)])
    for n=2:nc
        for m=2:mc
            rr=z(n)*(c(m)-b(m))+b(m);
            xx(n-1,m-1)=rr*costhe(m)+x0;
            yy(n-1,m-1)=rr*sinthe(m);
            press(n-1,m-1)=exp(p(n,m));
            machlevels(n-1,m-1)=sqrt((u(n,m)^2+v(n,m)^2)/t(n,m))/gf;
            entlevels(n-1,m-1)=s(n,m);
        end
    end
    [Co,ho]=contour(xx,yy,press);
    
    clabel(Co,ho);
    colorbar('vert')
    hold off;
    
    %__________________________________________________________________________
    % MACH CONTOUR PLOT
    %__________________________________________________________________________
    nfig=nfig+1;
    figure(nfig);
    plot(xb,yb,'k-',x0,0.,'r+');
    hold on;
    plot(xs,ys,'k-',cxst,cyst,'k--');
    axis equal
    axis tight
    xlabel('x')
    ylabel('y')
    title(['Mach Contours at k=',num2str(k)])
    [Co,ho]=contour(xx,yy,machlevels);
    clabel(Co,ho);
    colorbar('vert')
    hold off;
    
    %__________________________________________________________________________
    % ENTROPY CONTOUR PLOT
    %__________________________________________________________________________
    nfig=nfig+1;
    figure(nfig);
    plot(xb,yb,'k-',x0,0.,'r+');
    hold on;
    plot(xs,ys,'k-',cxst,cyst,'k--');
    axis equal
    axis tight
    xlabel('x')
    ylabel('y')
    title(['Entropy Contours at k=',num2str(k)])
    [Co,ho]=contour(xx,yy,entlevels);
    clabel(Co,ho);
    colorbar('vert')
    hold off;
    
    % release local variables
    clear xx yy press machlevels entlevels ach xb yb xs ys pp zz temp
    clear achp thep xtheory ytheory rso yso splot
    
    blunt_curvature
