% function blunt_run.m
% two-dimensional symmetric and axisymmetric blunt body
%{
Boundary Shock-Fitting Method
Copyright (c) 2005 - 2009 by M.D.Salas
%}
global z y costhe sinthe b c d w sigma u v p t nfig ell cxst cyst
global ma ka ja lb ld u0 gamma stab ga gc pst rst k j time
global dz dy dt dt1 nc mc mcm smin n1 m1 themax p1 r1 mach gf x0 hst
global mach_target dmach blunt_movie pfirst runlabel method 

pfirst=p;

blunt_plot1

if ja == 0 % call output
    blunt_out
end
%
%    time-dependent computation begins here
%    k=time step counter
%
disp(' ')
disp('Run Started:')
today=clock;
disp([num2str(today(2)) '/' num2str(today(3)) '/' num2str(today(1))...
    '  ' num2str(today(4)) ':' num2str(today(5)) ':' num2str(today(6))])
%
ysmax=1.2*c(mc)*sinthe(mc);     % used to define window in movie
xsmin=1.1*(c(2)*costhe(2)+x0);  % used to define window in movie
n1=0;
m1=0;
dt=0.;
mk=0;
midtheta=.5*themax*180/pi;

for k=1:ka % main iteration loop
    %
    % low Mach strategy, for Mach < 1.2, the free stream Mach will be
    % approached from above
    %
    if mach_target < 1.2 && k > 500
        if mach > mach_target
            mach=mach-dmach/500;
            mach=max(mach_target,mach);
            disp(['k= ' num2str(k) ' Mach=' num2str(mach)])
            u0=mach*gf;
            smin=mach^2;
            hst=ga+gamma*.5*smin;
            p1=(2.*ga*smin-1.)/gc;
            r1=(gc*p1+1.)/(gc+p1);
        end
    end
    %
    %--------------------------MOVIE---------------------------------------
    %
    if ja ~= 0
        test=rem(k,ja);
        if  test == 0 || k == 1
            mk=mk+1;
            timethe=2*k*pi/ka;
            xclock=.2*sin(timethe);
            yclock=.2*cos(timethe);
            xc0=(-1+x0)/2;
            yc0=.4;
            xcl=[xc0,xclock+xc0];
            ycl=[yc0,yclock+yc0];
            xtick=[xc0,xc0];
            ytick=[.62,.64];
            
            if blunt_movie == 1
                %
                % shock coordinates
                %
                for m=2:mc
                    xs(m-1)=c(m)*costhe(m)+x0;
                    ys(m-1)=c(m)*sinthe(m);
                end
                %
                % find sonic line coordinates
                %
                for n=2:nc
                    for m=2:mc
                        q2=u(n,m)^2+v(n,m)^2;
                        ach(m)=sqrt(q2/t(n,m))/gf;
                    end
                    for m=3:mc
                        aa=ach(m-1)-1;
                        ab=ach(m)-1;
                        if aa*ab <= 0
                            ac=ach(m)-ach(m-1);
                            zst=z(n);
                            yst=(y(m-1)*ab-y(m)*aa)/ac;
                            cst=(c(m-1)*ab-c(m)*aa)/ac;
                            bst=(b(m-1)*ab-b(m)*aa)/ac;
                            rst=zst*(cst-bst)+bst;
                            cxst(n-1)=-rst*cos(yst)+x0;
                            cyst(n-1)=rst*sin(yst);
                        end
                    end
                end
                
                if k == 1
                    xsold=xs;
                    ysold=ys;
                    xsonold=cxst;
                    ysonold=cyst;
                end
                %---------movie option one, shock shape, sonic line
                plot(xb,yb,'k-',xs,ys,'k-',cxst,cyst,'r-',xc0,yc0,'ro',...
                    xcl,ycl,'r-',xclock+xc0,yclock+yc0,'kd',xsold,...
                    ysold,'b--',xsonold,ysonold,'b--',xtick,ytick,'k-')
                xlabel('x')
                ylabel('y')
                axis equal
                axis([xsmin,x0,0.,ysmax]);
                text(xc0,.1, num2str(k));
            elseif blunt_movie == 2
                %-------------alternative movie, shock speed
                for m=2:mc
                    thep(m-1)=y(m)*180./pi;
                    sigma1(m-1)=sigma(m);
                    w1(m-1)=w(m);
                end
                plot(thep,w1,'k-')
                text(midtheta,w(2), num2str(k));
                xlabel('theta, deg.')
                ylabel('w')
                grid on
            end
            %--------------------------------------------------------------
            if le == 1
                if ld == 2;
                    if ell == 1;
                        title(['SPHERE',' Mach=', num2str(mach)]);
                    else
                        title(['ELLIPSOID',' Mach=', num2str(mach)]);
                    end;
                else
                    if ell == 1;
                        title(['CIRCLE',' Mach=', num2str(mach)]);
                    else
                        title(['ELLIPSE',' Mach=', num2str(mach)]);
                    end;
                end;
            elseif le == 2
                title(['PARABOLOID',' Mach=', num2str(mach)])
            elseif le == 3
                if ld == 1
                    title(['SUPER-CIRCLE',' Mach=', num2str(mach)])
                else
                    title(['SUPER-SPHERE',' Mach=', num2str(mach)])
                end
            end
            M(mk)=getframe;
        end;
    end;
    %
    %------------------------MOVIE-----------------------------------------
    %
    dsav(k)=d(2);
    tsav(k)=time;
    pstsav(k)=exp(p(2,2));
    j=j+1;
    %
    %    determining time step size
    %
    dt=1.;
    for m=2:mc
        dam=d(m)*dz;
        for n=2:nc
            dum=(b(m)+d(m)*z(n))*dy;
            ds=min(dum,dam);
            dt1=stab*ds/(sqrt(u(n,m)^2+v(n,m)^2)+gf*sqrt(t(n,m)));
            if dt1 < dt
                dt=dt1;
                n1=n;
                m1=m;
            end
        end
    end
    %
    if k == 1;
        disp(' ')
        disp('    k   time     d(2)       range      w(2)       |wmax|     e-pstg   e-psonic' );
    end;
    test=rem(k,lb);
    if  test == 0 % partial output
        dam=w(2)^2;
        wmax=max(abs(w));
        for m=3:mcm
            dam=dam+w(m)^2;
        end
        for m=2:mc
            q2=u(2,m)^2+v(2,m)^2;
            achbod(m-1)=q2/(gamma*t(2,m));
            pbod(m-1)=p(2,m);
        end
        pbs=interp1(achbod,pbod,1,'linear'); % interpolate for sonic pressure on body
        pbsonic=exp(pbs);
        p22=exp(p(2,2));
        prat=pbsonic/p22;
        pserr=prat/gg-1.; % relative error in sonic pressure
        range=sqrt(dam/ma)/u0;
        err=p22/pst-1.; % relative error in stagnation pressure
        fprintf('%6g %8.5f %10.7f %10.7f %10.7f %10.7f %8.5f %8.5f\n'...
            ,k,time,d(2),range,w(2),wmax,err,pserr)
    end
    %
    time=time+dt;
    %
    %----------------------COMPUTE FLOW FIELD------------------------------
    %
    if method == 1
        % blunt_mac_1's shock computation based on bicharacteristic & R-H jumps
        % standard shock-fitting method
        blunt_mac_1
        runlabel=1;
        %
    else
        % blunt_mac_2's shock computation based on dQ/dt & R-H jumps
        % nuovo shock fitting method
        blunt_mac_2
        runlabel=2;
    end
    %
end % end of main integration loop
%
% run completed
%
disp(' ')
disp('Run Ended:')
today=clock;
disp([num2str(today(2)) '/' num2str(today(3)) '/' num2str(today(1))...
    '  ' num2str(today(4)) ':' num2str(today(5)) ':' num2str(today(6)) ])
%-------------------------------FINAL OUTPUT & PLOTS-----------------------

if ja == 0;
    
    blunt_out;
    
    nfig=nfig+1;
    figure(nfig);
    plot(tsav,dsav,'r-');
    xlabel('time');
    ylabel('shock standoff distance');
    title('shock standoff distance vs time')
    grid on;
    
    nfig=nfig+1;
    figure(nfig);
    plot(tsav,pstsav,'r-');
    xlabel('time');
    ylabel('stagnation pressure');
    title('stagnation pressure vs time')
    grid on;
end;
%
% find sonic line coordinates
%
for n=2:nc
    for m=2:mc
        q2=u(n,m)^2+v(n,m)^2;
        ach(m)=sqrt(q2/t(n,m))/gf;
    end
    for m=3:mc
        aa=ach(m-1)-1;
        ab=ach(m)-1;
        if aa*ab <= 0
            ac=ach(m)-ach(m-1);
            zst=z(n);
            yst=(y(m-1)*ab-y(m)*aa)/ac;
            cst=(c(m-1)*ab-c(m)*aa)/ac;
            bst=(b(m-1)*ab-b(m)*aa)/ac;
            rst=zst*(cst-bst)+bst;
            cxst(n-1)=-rst*cos(yst)+x0;
            cyst(n-1)=rst*sin(yst);
        end
    end
end