% cone.m
% Ferri's potential solution for a cone at angle of attack
% after Bakker & Bannink
% Delft University of Technology, Report VTH 184, November 1974
%
% A SHOCK-FITTING PRIMER
% CHAPTER 5
% Copyright (c) 2005 - 2009 by M.D.Salas;
clear all
%
mach=2;   % Mach number
del=2;    % cone half angle, in degrees
alpha=4;  % angle of attack, in degrees
phi=90;   % meridional plane measured from leeward plane, in degrees
%
rat=pi/180;
del=del*rat;
del2=del^2;
alpha=alpha*rat;
phi=phi*rat;
bet2=mach^2-1;
bet=sqrt(bet2);
cosa=cos(alpha);
sina=sin(alpha);
cosp=cos(phi);
sinp=sin(phi);
tand=tan(del);
tand2=tand^2;
kd=bet*tand; 
skd=sqrt(1-kd^2);
error=abs(kd^2*log(kd));
%
disp('Slender cone theory compared to potential solution')
disp('After DUT Report VTH 184')
disp(' ')
disp(['Mach= ' num2str(mach), ', Alpha= ' num2str(alpha/rat), ', Cone half angle= ' num2str(del/rat), ', Phi=' num2str(phi/rat)])
disp(['the order of the error is ' num2str(error)])
%
N=100;
nc=N+1;
dth=2*del/N; 

for n=1:nc
    the(n)=del+(n-1)*dth;
    tant=tan(the(n));
    k=bet*tant;
    k2=k^2;
    sk=sqrt(1-k2);
    A=-cosa*tand2/(tand2*asech(kd)+skd);
    B=-2*sina*tand2/((1+2*tand2)*skd+asech(kd)*kd^2);
    % equation 5.5 Report VTH 184
    asechk=asech(k);
    up(n)=cosa+A*asechk-.5*B*bet*sk*cosp/k;
    vp(n)=sina*cosp-bet*A*sk/k+.5*B*bet2*(sk/k2+asechk)*cosp;
    wp(n)=-sina*sinp+.5*B*bet2*(sk/k2-asechk)*sinp;
    u(n)=sqrt(up(n)^2+vp(n)^2);
    v(n)=vp(n)*cos(the(n))-up(n)*sin(the(n));
    w(n)=wp(n);
    %
    % slender body approximation
    %
    As=-del2;
    Bs=-2*alpha*del2;
    % equation 5.6 & 5.7 Report VTH 184
    ups(n)=1+As*log(2/k)-Bs*bet*cosp/k;
    vps(n)=-As*Bs/k+(alpha+.5*Bs*bet2/k2)*cosp;
    wps(n)=(-alpha+.5*Bs*bet2/k2)*sinp;
    vs(n)=vps(n)*cos(the(n))-ups(n)*sin(the(n));
    us(n)=sqrt(ups(n)^2+vps(n)^2);
    %
    theb=the(n)/del;
    theb2=theb^2;
    alpb=alpha/del;
    % equation 5.8 & 5.9 Report VTH 184
    vs1(n)=-del*(theb2-1)*(theb-alpb*cosp)/theb2;
    ws1(n)=-alpb*del*(1+theb2)*sinp/theb2;
end

figure(1)
plot(the/rat,wp,'r-',the/rat,ws1,'b-')
xlabel('theta (degrees)')
ylabel('w')
legend('potential','slender body')
title(['comparison of potential & slender body theory, phi= ', num2str(phi/rat),' degrees'])
grid on

figure(2)
plot(the/rat,v,'r-',the/rat,vs1,'b-')
xlabel('theta (degrees)')
ylabel('v')
title(['comparison of potential & slender body theory, phi= ', num2str(phi/rat),' degrees'])
legend('potential','slender body')
grid on
