%corner_mu.m
%
%        symmetrical external conical corner program
%              WING TIP WITH SIDE SLIPE mu
%
% A SHOCK-FITTING PRIMER
% CHAPTER 5
% Copyright (c) 2005 - 2009 by M.D.Salas;
%
clear all
global tryme th ph thw thsp x y err u v w p t s ws time dt dx dy ddx
global ddy na ma nc mc ach uinf wso phcs gamma gb ga gd ge gc gf timen
global phisym thwx thsx wsp wsn qmax2 pn k sn vn wn po so
global ths thwp thsn vo wo thso phcsy phcsth cax0 cay0 car0 phica tdel2
global talam2 tdelst icut q qn sinmu cosmu
%
t1 = clock;
t0 = t1;
bet_data=[.961 .860 .686 0.610 .541 .420 .316 0.257 .226]; % previous runs data
therat_data=[.849 .830 .799 0.787 .777 .762 .758 0.773 .774];
prat_data=[.969 .943 .906 .893 .884 .879 .908 0.967 .973];
cax0_data=[0 -.1 -.12 -.13 -2];
therat_data1=[0.706 0.657 0.654 0.648 0.648];

%
%     th=theta
%     ph=phi
%     thsp=shock slope d(the)/d(phi)
%     u,v,w=spherical velocity components in the r,th,ph directions
%     p=logarithm of pressure
%     t=temperature
%     s=entropy
%
id=301;
kout=100;
klast=1;
na=60;
ma=40;
iload=1;
icut=4;
alam2=-30;
del2=10;
omega=90;
mu=0;
ach=3;
gamma=1.4;
car0=0;
cax0=0;
istream=0;
%
%     id=to identify run
%     kout=steps taken between outputs
%     klast=last step computed
%     na=number of mesh points along wedge minus one
%     ma=number of mesh points along symm. plane minus one
%     iload=1 load solution from file
%     icut=0 triangular delta wing
%     icut=1 truncated delta wing configuration
%     icut=2 circular arc rounded corner
%     icut=3 otherwise
%     icut=4 omega=-90, circular arc corner, "wing tip"
%
%     alam2=leading edge sweep in degrees - swept forward is positive
%     del2=wedge compression angle in degrees
%     omega=interior angle in degrees measured clock wise
%       from the vertical; 0 for corner; 90 for delta wing
%     mu=side slip angle in degrees, valid only for omega=-90, icut=4
%     ach=free stream mach no.
%     gamma=ratio of specific heats
%     car0=circular arc radius for icut=2, not used otherwise
%     cax0=x location of circular arc center for icut=4
%       not used otherwise; for a shift to right cax0 is negative
%
%     istream=0 no streamline plot,
%            =1, streamline plot, this is a very time consuming calculation
%                by Runge-Kutta scheme
%                it is intended to be a postprocessor of a saved run data 
%
%------------------NO CHANGES BELOW THIS LINE------------------------------

if icut == 4;
    omega=-90.;
else
    mu=0;
end

stab=0.7;
pi180=pi/180.;
k=0;
ko=0;
wsav=0;
time=0.;
gb=1./(gamma-1.);
ga=gamma*gb;
gd=.5/gb;
ge=1.+gd;
gc=ge/gd;
gf=sqrt(gamma);
omega=omega*pi180;
mu=mu*pi180;
del1=del2;
alam1=alam2;
alam1=alam1*pi180;
alam2=alam2*pi180;
del1=del1*pi180;
del2=del2*pi180;
talam1=tan(alam1);
talam2=tan(alam2);
talam22=talam2^2;
tdel1=tan(del1);
tdel2=tan(del2);
somg=sin(omega);
comg=cos(omega);
cala2=cos(alam2);
sala2=sin(alam2);
sinmu=sin(mu);
cosmu=cos(mu);
%
%
nfig=1;
%
%     from eq.(9)
%
phisym=.5*(1.5*pi-omega);
PHIwall=atan(talam2*tdel2);
PHIsym=phisym-PHIwall; % angle phi, from wall to symm. plane
bet=pi/PHIsym-1;
%
%     surface sigma2 unit normal eq.(11)
%
an2j=1./sqrt(1.+tdel2^2*(talam22+1.));
an2i=talam2*tdel2*an2j;
an2k=-tdel2*an2j;
%
%     surface sigma1 unit normal eq.(10)
%
sqr=1./sqrt((comg-talam1*tdel1*somg)^2+(somg...
    +talam1*tdel1*comg)^2+tdel1^2);
an1i=(comg-talam1*tdel1*somg)*sqr;
an1j=(somg+talam1*tdel1*comg)*sqr;
an1k=-tdel1*sqr;
%
%     Mach no. normal to leading edge, eq.(40)
%
ach2n=ach*(cosmu*cala2-sinmu*sala2);
%
%     beta2 is the effective wedge angle eq.(47)
%
beta2=atan(tdel2/cos(mu+alam2));

[p2,d2,s2,q2,delbs2,fail] = corsym_shock(ach2n,beta2,gamma);
if fail == 1
    break
end

tdels2=tan(delbs2)*cos(mu+alam2);
%
%     dels2 is the 2d-shock angle
%
dels2=atan(tdels2);
%
%     magnitude of vel. comp. along leading edge eq.(64)
%
v2t=ach*gf*(cosmu*sala2+sinmu*cala2);
%
%     velocity components in free stream cartesian frame
%     high pressure side of 2d-shock   eq.(63)
%
uu2=v2t*cala2-q2*sala2*an2j;
vv2=q2*(sala2*an2i-cala2*an2k);
ww2=v2t*sala2+q2*cala2*an2j;
%
%     external angle phi star
%
phis=pi+asin(sqr);
check=an1i*an2i+an1j*an2j+an1k*an2k;
if check < 0.;
    phis=2.*pi-asin(sqr);
end
phisd=phis/pi180;
%
%     shock normal in free stream cartesian frame eq.(57)
%
sqr=sqrt(1.+tdels2^2*(talam22+1.));
ansi=talam2*tdels2/sqr;
ansj=1./sqr;
ansk=-tdels2/sqr;
%
%     locate body sonic point
%
a=sqrt(gamma*p2/d2);
mach2d=sqrt(q2^2+v2t^2)/a;
amu=asin(1/mach2d);
if v2t ~= 0;
    sigma=atan(q2/abs(v2t));
else
    sigma=pi/2;
end
b2=tdel2/cos(alam2);
dum=sqrt(1.+b2^2);
xl=-dum/(tan(sigma-amu)-talam2*dum);
xson=xl*(1+talam22)-talam2;
yson=tdel2*(1-talam2*xson);
phi=-atan(yson/xson);
sonic=1;
me=1;
tryme(1)=phi;
for kip=1:20;
    phi=tryme(me);
    cosphi=cos(phi);
    sinphi=sin(phi);
    tthw=tdel2/(sinphi-talam2*tdel2*cosphi);
    thew=atan(tthw);
    costhe=cos(thew);
    sinthe=sin(thew);
    sv=-uu2*costhe*cosphi-ww2*sinthe+vv2*costhe*sinphi;
    sw=uu2*sinphi+vv2*cosphi;
    sach=(sv^2+sw^2)/a^2;
    err(me)=sonic-sach;
    if abs(err(me)) < 1.e-6;
        break;
    end;
    if me == 2;
        tryb=tryme(1)-err(1)*(tryme(2)-tryme(1))/(err(2)-err(1));
        tryme(1)=tryme(2);
        tryme(2)=tryb;
        err(1)=err(2);       
    else
        me=2;
        tryme(2)=tryme(1)*1.1;
    end
end
if abs(err(me)) > 1.e-6;
    disp('failed to converge while body sonic point')
end
phcs(1)=phi;
su=ww2*costhe+vv2*sinthe*sinphi-uu2*sinthe*cosphi;
%
%     from eq.(94)
%
phcsth(1)=sv*su/(sv*costhe+uu2*cosphi-vv2*sinphi)/sw;
%
%
nc=na+1;
ddx=na;
dx=1./ddx;
for n=1:nc;
    x(n)=(n-1)*dx;
end
mc=ma+1;
ddy=ma;
dy=1./ddy;
for m=1:mc;
    y(m)=(m-1)*dy;
end
%
%     locate shock sonic point
%
sonic=1.;
me=1;
tryme(1)=phcs(1)*1.2;
for kip=1:20;
    phi=tryme(me);
    cosphi=cos(phi);
    sinphi=sin(phi);
    tths=tdels2/(sinphi-talam2*tdels2*cosphi);
    the=atan(tths);
    costhe=cos(the);
    sinthe=sin(the);
    sv=-uu2*costhe*cosphi-ww2*sinthe+vv2*costhe*sinphi;
    sw=uu2*sinphi+vv2*cosphi;
    sach=(sv^2+sw^2)/a^2;
    err(me)=sonic-sach;
    if abs(err(me)) < 1.e-6;
        break
    end
    if me == 2;
        tryb=tryme(1)-err(1)*(tryme(2)-tryme(1))/(err(2)-err(1));
        tryme(1)=tryme(2);
        tryme(2)=tryb;
        err(1)=err(2);
    else
        me=2;
        tryme(2)=tryme(1)*1.1;
    end
end
if abs(err(me)) > 1.e-6;
    disp('failed to converge while locating sonic point on shock');
end;
phcs(mc)=phi;
su=ww2*costhe+vv2*sinthe*sinphi-uu2*sinthe*cosphi;
%
%     from eq.(94)
%
phcsth(mc)=sv*su/(sv*costhe+uu2*cosphi-vv2*sinphi)/sw;
thes=the;
%
%     find sonic line phcs
%
for m=2:ma;
    me=1;
    tryme(1)=phcs(m-1)+(phcs(mc)-phcs(1))/ma;
    for kip=1:20;
        the=thew+y(m)*(thes-thew);
        costhe=cos(the);
        sinthe=sin(the);
        phi=tryme(me);
        cosphi=cos(phi);
        sinphi=sin(phi);
        sv=-uu2*costhe*cosphi-ww2*sinthe+vv2*costhe*sinphi;
        sw=uu2*sinphi+vv2*cosphi;
        sach=(sv^2+sw^2)/a^2;
        err(me)=sonic-sach;
        if abs(err(me)) < 1.e-6;
            break
        end
        if me == 2;
            tryb=tryme(1)-err(1)*(tryme(2)-tryme(1))/(err(2)-err(1));
            tryme(1)=tryme(2);
            tryme(2)=tryb;
            err(1)=err(2);
        else
            me=2;
            tryme(2)=tryme(1)*1.1;
        end
    end
    if abs(err(me)) > 1.e-6;
        disp('failed to converge while locating sonic line');
    end
    phcs(m)=phi;
    su=ww2*costhe+vv2*sinthe*sinphi-uu2*sinthe*cosphi;
    %
    %     from eq.(94)
    %
    phcsth(m)=sv*su/(sv*costhe+uu2*cosphi-vv2*sinphi)/sw;
end
%
%     define wall
%
for n=1:nc;
    for m=1:mc;
        ph(n,m)=phisym-x(n)*(phisym-phcs(m));
        phi=ph(n,m);
        if m == 1;
            ic=0;
            [thew,thewp] = corsym_geom1(ic,phi);
        end
        thw(n)=thew;
        thwp(n)=thewp;
    end
end
%
%     truncated wing configuration icut=1 eq.(112-116)
%
if icut == 1;
    the=thw(nc);
    phi=ph(nc,1);
    tdelst=tdel2*(1.+talam2*tan(the)*cos(phi));
    for n=1:na;
        phi=ph(n,1);
        [thew,thewp] = corsym_geom1(icut,phi);
        thw(n)=thew;
        thwp(n)=thewp;
    end
end
%
%     circular arc icut=2 eq.(95-109a)
%
if icut == 2;
    phi=ph(1,1);
    tphis=tan(phi);
    caa=(1.+talam2*tdel2*tphis)^2-(1.+tphis^2)*(1.+talam22*tdel2^2);
    cab=2.*tdel2*(talam2*tdel2-tphis);
    cac=car0^2*(1.+tdel2^2*talam22)-tdel2^2;
    cax0=(-cab+sqrt(cab^2-4.*caa*cac))/(2.*caa);
    cay0=-tphis*cax0;
    xi=(cax0*(1.+talam2*tdel2*tphis)+talam2*tdel2^2)/(1....
        +talam22*tdel2^2);
    yi=tdel2*(1.-talam2*xi);
    phica=pi+atan(-yi/xi);
    if xi < 0.;
        phica=atan(-yi/xi);
    end
    for n=1:na;
        phi=ph(n,1);
        [thew,thewp] = corsym_geom1(2,phi);
        thw(n)=thew;
        thwp(n)=thewp;
    end
end
%
%  "wing tip" icut=4
%
if icut == 4;
    car0=tdel2*(1-talam2*cax0);
    phica=pi/2;
    if cax0 ~= 0;
        phica=atan(-car0/cax0);
    end
    for n=1:na;
        phi=ph(n,1);
        [thew,thewp] = corsym_geom1(4,phi);
        thw(n)=thew;
        thwp(n)=thewp;
    end
end
%
thwp(1)=0.;
%
%     define shock shape
%
dum=thw(1);
if icut == 2;
    [dum,thewp] = corsym_geom1(0,phisym);
end
%
%     from eq.(120)
% ths0=dum+.3*asin(1./ach); % guess theta of shock on sym. line
%
ths0=0.895*thes;
if (.226 < bet) && (bet < .961);
    ths0=interp1(bet_data,therat_data,bet)*thes;
end
if icut == 4;
 %   ths0=interp1(cax0_data,therat_data1,cax0)*thes;
 ths0min=mu+asin(1/ach);
 ths0=0.577*thes;
 ths0=max(ths0min*1.01,ths0);
end
%
phi=phcs(mc); % phi at interception of shock & sonic line
cosphi=cos(phi);
sinphi=sin(phi);
tths=tdels2/(sinphi-talam2*tdels2*cosphi);
the=atan(tths); % theta at interception of shock & sonic line
thsp(nc)=-tths*(cosphi+talam2*tdels2*sinphi)/(sinphi...
    -talam2*tdels2*cosphi)/(1.+tths^2); % 2D shock slope
%
% cubic fit for shock shape
%
B1=2*(phisym-phcs(mc));
A1=3*B1^2/4;
ths1=the;
C1=thsp(nc);
D1=A1*B1/6;
E1=A1/3;
F1=C1*B1/2+ths1-ths0;
b1=(A1*F1-C1*D1)/(D1*B1-A1*E1);
a1=-(B1*b1+C1)/A1;
for n=1:nc;
    phi=ph(n,mc);
    ths(n)=((a1*(phi-phcs(mc))+b1)*(phi-phcs(mc))+C1)*(phi-phcs(mc))+ths1;
end
%
%
for n=2:na;
    xph=-1./(phisym-phcs(mc));
    thsp(n)=.5*(ths(n+1)-ths(n-1))*ddx*xph;
end
thsp(1)=0.;
if(alam2 == 0.) && (omega == pi/2);
    %
    %     solution for 2d wedge
    %
    for n=1:nc;
        phi=ph(n,mc);
        cosphi=cos(phi);
        sinphi=sin(phi);
        tths=tdels2/sinphi;
        ths(n)=atan(tths);
        thsp(n)=-tths*(cosphi/sinphi)/(1.+tths^2);
    end
end
for n=1:nc;
    thwx(n)=thwp(n)*(phcs(1)-phisym);
    thsx(n)=thsp(n)*(phcs(mc)-phisym);
end
for m=1:mc;
    phcsy(m)=phcsth(m)*(ths(nc)-thw(nc));
end
%
for n=1:nc;
    for m=1:mc;
        phi=ph(n,m);
        the=y(m)*(ths(n)-thw(n))+thw(n);
        th(n,m)=the;
        if n == nc;
            %
            %     rhs boundary values defined from 2d solution eq.(92)
            %
            cosphi=cos(phi);
            sinphi=sin(phi);
            costhe=cos(the);
            sinthe=sin(the);
            u(n,m)=sinthe*sinphi*vv2-uu2*sinthe*cosphi+ww2*costhe;
            v(n,m)=costhe*sinphi*vv2-uu2*costhe*cosphi-ww2*sinthe;
            w(n,m)=cosphi*vv2+uu2*sinphi;
            s(n,m)=s2;
            t(n,m)=p2/d2;
            p(n,m)=log(p2);
        end
    end
end
%
%     initial shock conditions
%
uinf=ach*gf;
qmax2=2.*ga+uinf^2;
for n=1:na;
    the=th(n,mc);
    phi=ph(n,mc);
    cosphi=cos(phi);
    sinphi=sin(phi);
    costhe=cos(the);
    sinthe=sin(the);
    %
    %     direction cosines eq.(122)
    %
    enr=cosmu*costhe-sinmu*sinthe*cosphi;
    ent=-cosmu*sinthe-sinmu*costhe*cosphi;
    enp=sinmu*sinphi;
    %
    %     from eq.(126)
    %
    sqr=sqrt(1.+(thsp(n)/sinthe)^2);
    snt=1./sqr;
    snp=-thsp(n)*snt/sinthe;
    stt=snp;
    stp=-snt;
    %
    %     shock oriented velocity components eq.(127)
    %
    uwsi=-uinf*(ent*snt+enp*snp);
    wwsi=uinf*(ent*stt+enp*stp);
    vwsi=uinf*enr;
    smn=uwsi^2/gamma;
    %
    %     Rankine-Hugoniot jumps
    %
    p2p1=(gamma*smn/ge-1./gc);
    p(n,mc)=log(p2p1);
    r2r1=ge*smn/(gd*smn+1.);
    s(n,mc)=log(p2p1)-gamma*log(r2r1);
    t(n,mc)=p2p1/r2r1;
    uws2=uwsi/r2r1;
    u(n,mc)=vwsi;
    v(n,mc)=-uws2*snt+wwsi*stt;
    w(n,mc)=-uws2*snp+wwsi*stp;
end
%
%     conditions at the corner 
%
v(1,1)=0.;
w(1,1)=0.;
s(1,1)=s(1,mc);
th1=acos(-talam1/sqrt(ach^2-1.));
th2=phis-acos(-talam2/sqrt(ach^2-1.));
p1=p2;
p0=1.035*p2;
if (.226 < bet) && (bet < .961);
    p0=interp1(bet_data,prat_data,bet)*p2;
end
if icut == 4;
    p0=0.776*p2;
end
p(1,1)=log(p0);
t(1,1)=exp(s(1,1)/gamma+p(1,1)/ga);
u(1,1)=sqrt(qmax2-2.*t(1,1)*ga);
%
%     conditions along wall - except corner point
%
vcr=sqrt(v(nc,1)^2+w(nc,1)^2);
for n=2:na;
    ang=-.5*pi*sqrt(1.-x(n));
    p(n,1)=log(p0+(p2-p0)*(1.+sin(ang)));
    s(n,1)=s(nc,1);
    t(n,1)=exp(s(n,1)/gamma+p(n,1)/ga);
    thwn=thw(n);
    vx=vcr*x(n)*sqrt(x(n));
    sqr=sqrt(1.+thwp(n)^2/sin(thwn)^2);
    v(n,1)=thwp(n)*vx/(sin(thwn)*sqr);
    w(n,1)=vx/sqr;
    u(n,1)=sqrt(qmax2-vx^2-2.*t(n,1)*ga);
end
%
%     conditions along symmetry plane
%
for m=2:ma;
    p(1,m)=p(1,1)+y(m)*(p(1,mc)-p(1,1));
    s(1,m)=s(1,mc);
    t(1,m)=exp(p(1,m)/ga+s(1,m)/gamma);
    v(1,m)=y(m)*v(1,mc);
    w(1,m)=0.;
    u(1,m)=sqrt(qmax2-v(1,m)^2-2.*t(1,m)*ga);
end
%
%     initial conditions every where else
%
for n=2:na;
    wsp(n)=0.;
    ws(n)=0.;
    wsn(n)=0.;
    thsn(n)=ths(n);
    for m=2:ma;
        p(n,m)=p(n,1)+y(m)*(p(n,mc)-p(n,1));
        s(n,m)=s(n,1)+y(m)*(s(n,mc)-s(n,1));
        t(n,m)=exp(s(n,m)/gamma+p(n,m)/ga);
        v(n,m)=v(n,1)+y(m)*(v(n,mc)-v(n,1));
        w(n,m)=w(n,1)+y(m)*(w(n,mc)-w(n,1));
        u(n,m)=sqrt(qmax2-v(n,m)^2-w(n,m)^2-2.*t(n,m)*ga);
    end
end
if (alam2 == 0.) && (omega == pi/2);
    %
    %     exact solution for 2d wedge
    %
    for n=1:na;
        for m=1:ma;
            phi=ph(n,m);
            the=th(n,m);
            cosphi=cos(phi);
            sinphi=sin(phi);
            costhe=cos(the);
            sinthe=sin(the);
            u(n,m)=sinthe*sinphi*vv2-uu2*sinthe*cosphi+ww2*costhe;
            v(n,m)=costhe*sinphi*vv2-uu2*costhe*cosphi-ww2*sinthe;
            w(n,m)=cosphi*vv2+uu2*sinphi;
        end
    end
end
ws(1)=0.;
wsn(1)=0.;
ws(nc)=0.;
wsn(nc)=0.;
%
thsn(nc)=ths(nc);
wsp(1)=0.;
wsp(nc)=0.;
for n=1:nc;
    for m=1:mc;
        wn(n,m)=w(n,m);
        vn(n,m)=v(n,m);
        pn(n,m)=p(n,m);
        sn(n,m)=s(n,m);
    end
end
%
%
if iload == 1;
    load cor_save.mat tryme th ph thw thsp x y err u v w p t s ws time...
        dt dx dy ddx ddy na ma nc mc ach uinf wso phcs gamma gb ga gd ge...
        gc gf timen phisym thwx thsx wsp wsn qmax2 pn k sn...
        vn wn po so ths thwp thsn vo wo thso phcsy phcsth cax0 cay0 car0...
        phica tdel2 talam2 tdelst icut q qn
    disp(' ');
    disp('Solution loaded from file.');
end
%
%     output initial conditions
%
disp(' ');
disp('Symmetrical External Conical Corner Program (mu-version)');
disp(' ');
disp(['Run Number=' num2str(id)]);
disp(['Mach=' num2str(ach) ', gamma=' num2str(gamma)...
    ', omega=' num2str(omega/pi180) ' deg, phisym=' num2str(phisym/pi180)...
    ' deg, mu=' num2str(mu/pi180) ' deg']);
disp(['alam2=' num2str(alam2/pi180) ' deg, del2=' num2str(del2/pi180) ' deg'...
    ', corner rad=' num2str(car0) ', corner theta=' num2str(thw(1)*180/pi)...
    ' deg']);
disp([ 'na=' num2str(na) ', ma=' num2str(ma) ', klast=' num2str(klast)...
    ', PHIsym=' num2str(PHIsym*180/pi) ' deg' ', beta=' num2str(bet)...
    ', cax0=' num2str(cax0)]);
disp(' ');
%
% plot initial pressure field
%
for n=1:nc;
    for m=1:mc;
        tanth=tan(th(n,m));
        X(n,m)=tanth*cos(ph(n,m));
        Y(n,m)=tanth*sin(ph(n,m));
        pres(n,m)=exp(p(n,m));
    end
end
cosphis=cos(phisym-.5*pi);
sinphis=sin(phisym-.5*pi);
for n=1:nc;
    Xw(n)=X(n,1);
    Xsh_in(n)=X(n,mc);
    Yw(n)=Y(n,1);
    Ysh_in(n)=Y(n,mc);
    % reflect wall about symmmetry line
    Xwp=Xw(n)*cosphis+Yw(n)*sinphis;
    Ywp=Yw(n)*cosphis-Xw(n)*sinphis;
    Xwpr=-Xwp;
    Ywpr=Ywp;
    Xwr(n)=Xwpr*cosphis-Ywpr*sinphis;
    Ywr(n)=Ywpr*cosphis+Xwpr*sinphis;
end
for m=1:mc;
    pso1(m)=pres(1,m);
    pso2(m)=pres(nc,m);
    Xso1(m)=X(1,m);
    Yso1(m)=Y(1,m);
    Xso2(m)=X(nc,m);
    Yso2(m)=Y(nc,m);
end
%
% plot initial pressure field
%
nfig=nfig+1;
figure(nfig);
plot(Xw,Yw,'k-',Xwr,Ywr,'k-');
title('Initial Pressure Field');
xlabel('X');
ylabel('Y');
axis equal
hold on
plot(Xsh_in,Ysh_in,'k-');
xtop=[0,Xw(nc)];
ytop=[0,0];
xt2p=xtop(2)*cosphis;
yt2p=-xtop(2)*sinphis;
xt2pr=-xt2p;
yt2pr=yt2p;
xs2=xt2pr*cosphis-yt2pr*sinphis;
ys2=yt2pr*cosphis+xt2pr*sinphis;
xside=[0,xs2];
yside=[0,ys2];
plot(xside,yside,'k--',xtop,ytop,'k--',0,0,'k+')
plot(Xso1,Yso1,'k-',Xso2,Yso2,'k-');
[C,h]=contour(X,Y,pres);
clabel(C,h)
colorbar('vert')
hold off

disp('              av. sh. corner   min            st. off');
disp(['    k'  '  time   ' ' speed' '  press'...
    '   p-wall   xmin    dist.   thes  p-rat the-rat  s-press   cpu-t'])
for n=1:nc;
    pwall(n)=exp(p(n,1));
    pshock(n)=exp(p(n,mc));
end;
%
% plot initial pressure
%
nfig=nfig+1;
figure(nfig);
plot(x,pwall,'k-o',x,pshock,'k-s');
title('Pressure along wall & shock');
xlabel('X')
ylabel('P')
axis equal
legend('w press','s press');
hold on
%
% find minimum wall pressure
%
[pmin,nmin]=min(pwall);
if (nmin == 1) || (nmin == nc);
    xmin=0;
    if nmin == mc;
        xmin=1;
    end
else
    aa=.5*(pwall(nmin+1)+pwall(nmin-1)-2*pwall(nmin))/dx^2;
    bb=(pwall(nmin+1)-pwall(nmin))/dx-aa*(x(nmin+1)+x(nmin));
    cc=pwall(nmin)-aa*x(nmin)^2-bb*x(nmin);
    xmin=-.5*bb/aa;
    pmin=aa*xmin^2+bb*xmin+cc;
end
phmin=phisym-xmin*(phisym-phcs(1));
[thmin,dum]=corsym_geom1(icut,phmin);
Xmin=tan(thmin)*cos(phmin);
Ymin=tan(thmin)*sin(phmin);
Xc=tan(thw(1))*cos(phisym);
Yc=tan(thw(1))*sin(phisym);
dist=sqrt((Xmin-Xc)^2+(Ymin-Yc)^2);
%
% shock standoff-distance
%
Xs=tan(ths(1))*cos(phisym);
Ys=tan(ths(1))*sin(phisym);
Xc=tan(thw(1))*cos(phisym);
Yc=tan(thw(1))*sin(phisym);
ds=sqrt((Xs-Xc)^2+(Ys-Yc)^2);
%
% evaluate average shock speed
%
wsav=0;
for n=1:nc;
    wsav=wsav+abs(ws(n));
end
wsav=wsav/nc;
%
p11=exp(p(1,1));
p1mc=exp(p(1,mc));
rat1=p11/exp(p(nc,1));
rat2=ths(1)/ths(nc);
        t2=etime(clock,t0)/60;
        fprintf('%5g %7.5f %7.5f %7.5f %7.5f %7.5f %7.5f %5.3f %5.3f %6.3f %8.3f %8.3f\n'...
            ,k,time,wsav,p11,pmin,dist,ds,ths(1)*180/pi,rat1,rat2,p1mc,t2);
%
% start computation
%
for k=1:klast;
    %
    %     evaluate dt
    %
    dt=1.e+9;
    for n=1:nc;
        for m=1:mc;
            the=th(n,m);
            sinthe=sin(the);
            %
            %     evaluate jacobian,etc... eq.(131-143)
            %
            fth=-1./(ths(n)-thw(n));
            gph=1./(phisym-phcs(m));
            fx=-fth*(thwx(n)+y(m)*(thsx(n)-thwx(n)));
            gy=-x(n)*phcsy(m)*gph;
            ft=-y(m)*ws(n)*fth;
            gt=0.;
            djacob=1./(1.-fx*gy);
            xth=fth*gy*djacob;
            xph=-gph*djacob;
            xt=(ft*gy-gt)*djacob;
            yth=-fth*djacob;
            yph=fx*gph*djacob;
            yt=(fx*gt-ft)*djacob;
            xphs=xph/sinthe;
            yphs=yph/sinthe;
            ab=xt+v(n,m)*xth+w(n,m)*xphs;
            ac=yt+v(n,m)*yth+w(n,m)*yphs;
            aa=sqrt(gamma*t(n,m));
            ad=sqrt(yth^2+yphs^2);
            ae=sqrt(xth^2+xphs^2);
            dt1=dy/abs(ac+aa*ad);
            dt2=dy/abs(ac-aa*ad);
            dt3=dx/abs(ab+aa*ae);
            dt4=dx/abs(ab-aa*ae);
            dtv=[dt1,dt2,dt3,dt4,dt];
            dt=min(dtv);
        end
    end
    dt=dt*stab;
    timen=time+dt;

    corsym_points_mu;

    ko=ko+1;
    if (ko-kout) == 0; % print partial output
    %    t0 = clock;
        %
        % evaluate average shock speed
        %
        wsav=0;
        for n=1:nc;
            wsav=wsav+abs(ws(n));
        end
        wsav=wsav/nc;
        ko=0;
        for n=1:nc;
            pwall(n)=exp(p(n,1));
            pshock(n)=exp(p(n,mc));
        end
        %
        % find minimum wall pressure
        %
        [pmin,nmin]=min(pwall);
        if (nmin == 1) || (nmin == nc);
            xmin=0;
            if nmin == mc;
                xmin=1;
            end
        else
            aa=.5*(pwall(nmin+1)+pwall(nmin-1)-2*pwall(nmin))/dx^2;
            bb=(pwall(nmin+1)-pwall(nmin))/dx-aa*(x(nmin+1)+x(nmin));
            cc=pwall(nmin)-aa*x(nmin)^2-bb*x(nmin);
            xmin=-.5*bb/aa;
            pmin=aa*xmin^2+bb*xmin+cc;
        end
        phmin=phisym-xmin*(phisym-phcs(1));
        [thmin,dum]=corsym_geom1(icut,phmin);
        Xmin=tan(thmin)*cos(phmin);
        Ymin=tan(thmin)*sin(phmin);
        Xc=tan(thw(1))*cos(phisym);
        Yc=tan(thw(1))*sin(phisym);
        dist=sqrt((Xmin-Xc)^2+(Ymin-Yc)^2);
        %
        % shock standoff-distance
        %
        Xs=tan(ths(1))*cos(phisym);
        Ys=tan(ths(1))*sin(phisym);
        ds=sqrt((Xs-Xc)^2+(Ys-Yc)^2);
        %
        p11=exp(p(1,1));
        p1mc=exp(p(1,mc));
        rat1=p11/exp(p(nc,1));
        rat2=ths(1)/ths(nc);
        t2=etime(clock,t0)/60;
        fprintf('%5g %7.5f %7.5f %7.5f %7.5f %7.5f %7.5f %5.3f %5.3f %6.3f %8.3f %8.3f\n'...
            ,k,time,wsav,p11,pmin,dist,ds,ths(1)*180/pi,rat1,rat2,p1mc,t2);
        %
        % plot wall & shock pressure
        %
        figure(nfig);
        plot(x,pwall,'k-',x,pshock,'r-');
        title('Pressure along wall & shock');
        xlabel('X');
        ylabel('P');
        axis equal;
        legend('w press','s press');
    end

end % end computation loop

% save data for future run
%
save cor_save1.mat tryme th ph thw thsp x y err u v w p t s ws time...
    dt dx dy ddx ddy na ma nc mc ach uinf wso phcs gamma gb ga gd ge...
    gc gf timen phisym thwx thsx wsp wsn qmax2 pn k sn...
    vn wn po so ths thwp thsn vo wo thso phcsy phcsth cax0 cay0 car0...
    phica tdel2 talam2 tdelst icut q qn
disp(' ');%disp(['Solution saved to file.']);
%
% FINAL OUTPUT
% find minimum wall pressure location
%
for n=1:nc;
    pwall(n)=exp(p(n,1));
end
[pmin,nmin]=min(pwall);
if (nmin == 1) || (nmin == nc);
    xmin=0;
    if nmin == mc;
        xmin=1;
    end
else
    aa=.5*(pwall(nmin+1)+pwall(nmin-1)-2*pwall(nmin))/dx^2;
    bb=(pwall(nmin+1)-pwall(nmin))/dx-aa*(x(nmin+1)+x(nmin));
    xmin=-.5*bb/aa;
end
phmin=phisym-xmin*(phisym-phcs(1));
[thmin,dum]=corsym_geom1(icut,phmin);
Xmin=tan(thmin)*cos(phmin);
Ymin=tan(thmin)*sin(phmin);
%
% find q=0 location
%
xzero=0;
for n=2:nc
    if q(n)*q(n-1) < 0;
        xzero=x(n-1)-q(n-1)*dx/(q(n)-q(n-1));
        break
    end
end
phzero=phisym-xzero*(phisym-phcs(1));
[thzero,dum]=corsym_geom1(icut,phzero);
Xzero=tan(thzero)*cos(phzero);
Yzero=tan(thzero)*sin(phzero);
node_dist=sqrt((Xzero-Xc)^2+(Yzero-Yc)^2);
disp('Along the wall:');
disp(['Node distance from corner=' num2str(node_dist)]);
%
%
for n=1:nc;
    for m=1:mc;
        tanth=tan(th(n,m));
        X(n,m)=tanth*cos(ph(n,m));
        Y(n,m)=tanth*sin(ph(n,m));
        % reflect about symmmetry line
        Xp=X(n,m)*cosphis+Y(n,m)*sinphis;
        Yp=Y(n,m)*cosphis-X(n,m)*sinphis;
        Xpr=-Xp;
        Ypr=Yp;
        Xr(n,m)=Xpr*cosphis-Ypr*sinphis;
        Yr(n,m)=Ypr*cosphis+Xpr*sinphis;
        pres(n,m)=exp(p(n,m));
    end
end
for n=1:nc;
    Xsh(n)=X(n,mc);
    Ysh(n)=Y(n,mc);
    %
    % reflect about symmmetry line
    %
    Xshr(n)=Xr(n,mc);
    Yshr(n)=Yr(n,mc);
end
for m=1:mc;
    Xso1(m)=X(1,m);
    Yso1(m)=Y(1,m);
    Xso2(m)=X(nc,m);
    Yso2(m)=Y(nc,m);
    %
    % reflect about symmmetry line
    %
    Xso2r(m)=Xr(nc,m);
    Yso2r(m)=Yr(nc,m);
end
%
% plot pressure contours
%
nfig=nfig+1;
figure(nfig);
plot(Xw,Yw,'k-',Xwr,Ywr,'k-',0,0,'k+');
title('Pressure & X-flow Mach');
xlabel('X')
ylabel('Y')
axis equal
hold on
plot(Xsh,Ysh,'k-',Xshr,Yshr,'k');
plot(Xmin,Ymin,'ko',Xzero,Yzero,'k+');
plot(xside,yside,'k--',xtop,ytop,'k--')
plot(Xso1,Yso1,'k-',Xso2,Yso2,'k-',Xso2r,Yso2r,'k-');
%
[C,h]=contour(X,Y,pres);
clabel(C,h);
colorbar('vert')
%
% plot x-flow Mach number contours
%
% reflect about symmmetry line
%
Xminp=Xmin*cosphis+Ymin*sinphis;
Yminp=Ymin*cosphis-Xmin*sinphis;
Xminpr=-Xminp;
Yminpr=Yminp;
Xminr=Xminpr*cosphis-Yminpr*sinphis;
Yminr=Yminpr*cosphis+Xminpr*sinphis;
Xzerop=Xzero*cosphis+Yzero*sinphis;
Yzerop=Yzero*cosphis-Xzero*sinphis;
Xzeropr=-Xzerop;
Yzeropr=Yzerop;
Xzeror=Xzeropr*cosphis-Yzeropr*sinphis;
Yzeror=Yzeropr*cosphis+Xzeropr*sinphis;
plot(Xminr,Yminr,'ko',Xzeror,Yzeror,'k+');
for n=1:nc;
    for m=1:mc;
        xmach(n,m)=sqrt((v(n,m)^2+w(n,m)^2)/(gamma*t(n,m)));
    end
end
[C,h]=contour(Xr,Yr,xmach);
clabel(C,h)
colorbar('vert')
hold off
%
% plot entropy & temperature contours
%
nfig=nfig+1;
figure(nfig);
plot(Xw,Yw,'k-',Xwr,Ywr,'k-',0,0,'k+');
title('Entropy & Temperature');
xlabel('X')
ylabel('Y')
axis equal
hold on
plot(Xsh,Ysh,'k-',Xshr,Yshr,'k-');
plot(Xmin,Ymin,'ko',Xzero,Yzero,'k+');
plot(xside,yside,'k--',xtop,ytop,'k--')
plot(Xso1,Yso1,'k-',Xso2,Yso2,'k-',Xso2r,Yso2r,'k-');
%
[C,h]=contour(X,Y,s);
clabel(C,h)
colorbar('vert')
plot(Xminr,Yminr,'ko',Xzeror,Yzeror,'k+');
%
[C,h]=contour(Xr,Yr,t);
clabel(C,h);
colorbar('vert')
hold off
%
% plot wall & sym line pressure
%
nfig=nfig+1;
figure(nfig);
for m=1:mc;
    psym(m)=exp(p(1,m));
end
plot(x,pwall,'k-o',y,psym,'k-s');
title('Wall & Sym. Plane Pressure')
xlabel('X & Y')
ylabel('P')
legend('wall pressure', 'sym. line pressure');
grid on
%
% plot wall x-flow Mach no.
%
for n=1:nc;
    wallmach(n)=sqrt(q(n)^2/(gamma*t(n,1)));
end
nfig=nfig+1;
figure(nfig);
plot(x,wallmach,'k-o');
title('Wall X-flow Mach No.')
xlabel('X')
ylabel('Mach')
grid on
%
% shock shape comparison; initial & final
%
nfig=nfig+1;
figure(nfig);
plot(Xsh_in,Ysh_in,'r--',Xsh,Ysh,'k-');
title('Shock shape');
xlabel('X')
ylabel('Y')
legend('initial','final')
%
% plot grid
%
nfig=nfig+1;
figure(nfig);
for n=1:nc;
    for m=1:mc;
        xxm(m)=X(n,m);
        yym(m)=Y(n,m);
    end
    plot(xxm,yym,'k-');
    axis equal
    hold on
end
for m=1:mc;
    for n=1:nc;
        xxn(n)=X(n,m);
        yyn(n)=Y(n,m);
    end
    plot(xxn,yyn,'k-',0,0,'k+');
end
title('grid')
hold off
%
% carpet plot of temperature
%
nfig=nfig+1;
figure(nfig);
surf(X,Y,t);
hold on
shading flat
title('Temperature');
xlabel('x')
ylabel('y')
for n=1:nc;
    twall(n)=t(n,1);
end
plot3(Xw,Yw,twall,'k-');
hold off
%
% carpet plot of pressure
%
nfig=nfig+1;
figure(nfig);
surf(X,Y,pres);
hold on
title('Pressure');
shading flat
xlabel('x')
ylabel('y')
plot3(Xw,Yw,pwall,'k-')
hold off
%
% find  bifurcation index dq/dth, "the" being the angle from the 
%   corner to a point on the wall
%
db=sqrt(1+sqrt(Xc^2+Yc^2));
for n=1:nc;
    da=sqrt((Xw(n)-Xc)^2+(Yw(n)-Yc)^2);
    theb(n)=atan(da/db);
end
for n=1:na;
    ub=.5*(u(n+1,1)+u(n,1));
    qthu(n)=(q(n+1)-q(n))/(ub*(theb(n+1)-theb(n)));
    thebb(n)=.5*(theb(n+1)+theb(n));
end
if Xzero == Xc;
    qthuzero=qthu(1);
else
    thebzero=atan(sqrt((Xzero-Xc)^2+(Yzero-Yc)^2)/db);
    qthuzero=interp1(thebb,qthu,thebzero);
end
disp(['"(dq/dth)/u at q=0" = ' num2str(qthuzero)]);
disp(['"(dq/dth)/u at corner" = ' num2str(qthu(1))]);
for m=1:mc;
    vsym(m)=v(1,m);
end
%
% find bifurcation index dv/dth on symmetry plane
%
disp(' ')
disp('Along the symmetry plane');
%
% plot v on symmetry plane
%
nfig=nfig+1;
figure(nfig);
plot(y,vsym,'k-o')
title('v along symmetry plane')
xlabel('Y')
ylabel('v')
grid on
for m=1:ma;
    ub=.5*(u(1,m+1)+u(1,m));
    vthu(m)=(vsym(m+1)-vsym(m))/(ub*(th(1,m+1)-th(1,m)));
    thevb(m)=.5*(th(1,m+1)+th(1,m));
end
yzero=0;
for m=2:mc
    if vsym(m)*vsym(m-1) < 0;
        yzero=y(m-1)-vsym(m-1)*dy/(vsym(m)-vsym(m-1));
        break
    end
end
thzero=yzero*(ths(1)-thw(1))+thw(1);
if yzero == 0;
    vthuzero=vthu(1);
else
    vthuzero=interp1(thevb,vthu,thzero);
end
Xzero=tan(thzero)*cos(phisym);
Yzero=tan(thzero)*sin(phisym);
node_dist=sqrt((Xzero-Xc)^2+(Yzero-Yc)^2);
disp(['Node distance from corner=' num2str(node_dist)]);
disp(['"(dv/dth)/u at v=0" = ' num2str(vthuzero)]);
disp(['"(dv/dth)/u at corner" = ' num2str(vthu(1))]);
disp(' ');
%
% compare dp/dth at sonic line with theory, Roe's C.P. 1228
%
pth=(p(nc,1)-p(na,1))/(theb(nc)-theb(na));
pththeory=gamma*(sqrt(mach2d^2-1))/(gamma+1); 
disp(['2D Mach=' num2str(mach2d) ', dp/dth comp.=' num2str(pth) ...
    ', dp/dth theory=' num2str(pththeory)]);
%
% vpp @ corner
%
for m=1:mc;
    vdum(m)=v(2,m);
    thdum(m)=th(2,m);
    phdum(m)=ph(2,m);
end
for m=1:ma;
v2m = interp1(thdum,vdum,th(1,m));
ph2m = interp1(thdum,phdum,th(1,m));
vpp(m)=2*(v2m-v(1,m))/(ph2m-ph(1,m))^2;
thpp(m)=th(1,m);
end
disp(['d(dv/dphi)/dphi @ corner= ' num2str(vpp(1))]);
nfig=nfig+1;
figure(nfig);
plot(thpp,vpp,'k-o');
title('d(dv/dphi)/dphi')
grid on;
xlabel('theta')
ylabel('vpp')
%
t3=clock;
disp(['total time= ' num2str(etime(t3,t1)/60) ' min.' ]);

if istream == 1
corsym_strm % calculate and plot streamline locus
end