%cor-points_mu.m
%
%     integration of Euler eqs. using Mac Cormack's scheme
%     shock location using Kentzer's scheme
%     the radial vel. comp. is evaluated using the
%     steady state Bernoulli's eq., a proceedure valid
%     only in the steady state limit
%
% Copyright (c) 2005 -2009 by M.D.Salas;
%
global th ph thw thsp x y u v w p t s ws time dt dx ddx
global ddy na ma nc mc uinf wso phcs gamma ga gd ge gc timen
global phisym thwx thsx wsp wsn qmax2 pn sn vn wn po so
global ths thwp thsn vo wo thso phcsy phcsth 
global icut q qn sinmu cosmu
%
test=1.01;
%
%     cross flow velocity at the wall eq.(145-148)
%
for n=2:nc;
    the=thw(n);
    ant=-thwp(n)/sin(the);
    bnt=-1./sqrt(1.+ant^2);
    ant=-ant*bnt;
    q(n)=v(n,1)*ant+w(n,1)*bnt;
    if q(n) > 0.;
        s(n,1)=s(1,1);
    end
end
%
q(1)=0.;
qn(1)=0.;
qn(nc)=q(nc);
%
for kk=1:2;
    loop=kk-1;
    %
    for n=1:na;
        for m=1:mc;
            nm=n-loop;
            mm=m-loop;
            if n == 1;
                nm=1;
            end
            if m == 1;
                mm=1;
            end
            if m == mc;
                mm=ma;
            end
            np=nm+1;
            mp=mm+1;
            %
            py=(p(n,mp)-p(n,mm))*ddy;
            vy=(v(n,mp)-v(n,mm))*ddy;
            wy=(w(n,mp)-w(n,mm))*ddy;
            sy=(s(n,mp)-s(n,mm))*ddy;
            %
            if (m == mc) && (loop == 0);
                py=-(2.*p(n,m-3)-9.*p(n,m-2)+18.*p(n,m-1)...
                    -11.*p(n,m))/6.*ddy;
                vy=-(2.*v(n,m-3)-9.*v(n,m-2)+18.*v(n,m-1)...
                    -11.*v(n,m))/6.*ddy;
                wy=-(2.*w(n,m-3)-9.*w(n,m-2)+18.*w(n,m-1)...
                    -11.*w(n,m))/6.*ddy;
            end
            %
            px=(p(np,m)-p(nm,m))*ddx;
            vx=(v(np,m)-v(nm,m))*ddx;
            wx=(w(np,m)-w(nm,m))*ddx;
            sx=(s(np,m)-s(nm,m))*ddx;
            %
            if n == 1;
                %
                %     symmetry plane conditions n=1
                %
                px=0.;
                vx=0.;
                sx=0.;
                wy=0.;
                if loop == 1;
                    wx=(2.*w(2,m)-.5*w(3,m))*ddx;
                end
            end
            %
            the=th(n,m);
            costhe=cos(the);
            sinthe=sin(the);
            cot=costhe/sinthe;
            %
            %     jacobian,etc... eq.(131-143)
            %
            fth=-1./(ths(n)-thw(n));
            gph=1./(phisym-phcs(m));
            fx=-fth*(thwx(n)+y(m)*(thsx(n)-thwx(n)));
            gy=-x(n)*phcsy(m)*gph;
            ft=-y(m)*ws(n)*fth;
            gt=0.;
            djacob=1./(1.-fx*gy);
            xth=fth*gy*djacob;
            xph=-gph*djacob;
            xt=(ft*gy-gt)*djacob;
            yth=-fth*djacob;
            yph=fx*gph*djacob;
            yt=(fx*gt-ft)*djacob;
            xphs=xph/sinthe;
            yphs=yph/sinthe;
            %
            %     eq.(144)
            %
            ab=xt+v(n,m)*xth+w(n,m)*xphs;
            ac=yt+v(n,m)*yth+w(n,m)*yphs;
            %
            %     windward difference for entropy 
            %
            if ab < 0.;
                sx=(s(n+1,m)-s(n,m))*ddx;
            end
            if (n ~= 1) && (ab > 0.);
                sx=(s(n,m)-s(n-1,m))*ddx;
            end
            if(m ~= mc) && (ac < 0.);
                sy=(s(n,m+1)-s(n,m))*ddy;
            end
            if(m ~= 1) && (ac > 0.);
                sy=(s(n,m)-s(n,m-1))*ddy;
            end
            %
            pt=-(ab*px+ac*py+gamma*(xth*vx+yth*vy+wx*xphs+wy*yphs...
                +2.*u(n,m)+v(n,m)*cot));
            vt=-(ab*vx+ac*vy+t(n,m)*(yth*py+xth*px)...
                +u(n,m)*v(n,m)-w(n,m)^2*cot);
            wt=-(ab*wx+ac*wy+t(n,m)*(xphs*px+py*yphs)...
                +w(n,m)*(u(n,m)+v(n,m)*cot));
            st=-(ab*sx+ac*sy);

            dum=(thwp(n)+y(m)*(thsp(n)-thwp(n)))/sinthe;
            axp=-1./sqrt(1.+dum^2);
            axt=dum*axp;
            sqr=1./sqrt((x(n)*phcsth(m))^2+1./sinthe^2);
            ayt=sqr/sinthe;
            ayp=x(n)*phcsth(m)*sqr;
            %
            %     wall computation
            %
            if m == 1;
                %
                % limit domain of dependence for supersonic x-flow
                %
                xfmach=sqrt((w(n,m)^2+v(n,m)^2)/gamma*t(n,m));
                if (xfmach > 1.01) && (n < na-5);
                    wx=(w(n+1,m)-w(n,m))/dx;
                    vx=(v(n+1,m)-v(n,m))/dx;
                    px=(p(n+1,m)-p(n,m))/dx;
                end
                dum=(thwp(n)+y(m)*(thsp(n)-thwp(n)))/sinthe;
                axp=-1./sqrt(1.+dum^2);
                axt=dum*axp;
                sqr=1./sqrt((x(n)*phcsth(m))^2+1./sinthe^2);
                ayt=sqr/sinthe;
                ayp=x(n)*phcsth(m)*sqr;
                af=axt*xth+axp*xphs;
                qx=axt*vx+axp*wx;
                % qy=axt*vy+axp*wy, yt=0 at the wall, hence yt*qy=0
                qt=-(ab*qx+t(n,1)*af*px+u(n,1)*q(n));
                ad=xth*axp-xphs*axt;
                ae=yth*axp-yphs*axt;
                tbx=(thwp(np)-thwp(nm))*ddx/sinthe...
                    -cot*thwp(n)*thwx(n)/sinthe;
                qbn=q(n)*ad*tbx/(1.+(thwp(n)/sinthe)^2)+ae*(axp*vy-axt*wy);
                % yt*py=0
                pt=-(ab*px+gamma*(af*qx+qbn+2.*u(n,m)+v(n,m)*cot));
            end
            %
            %     entropy extrapolation at corner
            %
            if (n == 1) && (m == 1);
                if v(1,2) <= 0.;
                    s(1,1)=s(1,2);
                    so(1,1)=s(1,1);
                else
                    s(1,1)=s(2,1);
                    so(1,1)=s(1,1);
                end
            end
            %
            if m == mc;
                phi=ph(n,m);
                sinphi=sin(phi);
                cosphi=cos(phi);
                %
                %     calculation of shock location m=mc
                %
                %     direction cosine eq.(166,171)
                %
                etat=-cosmu*sinthe-sinmu*costhe*cosphi;
                etap=sinmu*sinphi;
                %
                %     unit normal to shock eq.(168)
                %
                ent=1./sqrt(1.+(thsp(n)/sinthe)^2);
                enp=-thsp(n)*ent/sinthe;
                %
                %     velocity normal to shock eq.(160)
                %
                uwinf=uinf*(etat*ent+etap*enp);
                %
                %     relative velocity eq.(157,161)
                %
                ayns=ayt*ent+ayp*enp;
                uwirel=uwinf-ws(n)*ayns;
                smrel=uwirel^2/gamma;
                %
                %     time derivatives of shock normal eq.(169,170)
                %
                entt=-ent^3*(wsp(n)-thsp(n)*ws(n)*cot)*thsp(n)/sinthe^2;
                enpt=-thsp(n)*entt/sinthe+ent*(thsp(n)*ws(n)*cot...
                    -wsp(n))/sinthe;
                %
                %     time derivative of direction cosine eq.(167,172)
                %
                pht=0.;
                etatt=(sinmu*sinthe*cosphi-cosmu*costhe)*ws(n);
                etapt=0;
                %
                %     term defined by eq.(173)
                %
                sqt=-sqr^3*(-cot*ws(n)/sinthe^2);
                aytt=-ayt*cot*ws(n)+sqt/sinthe;
                aypt=x(n)*(phcsth(m)*sqt);
                aynst=aytt*ent+aypt*enp+ayt*entt+ayp*enpt;
                aa=uinf*(entt*etat+ent*etatt+etap*enpt+etapt*enp)...
                    -ws(n)*aynst;
                %
                %     shock acceleration
                %
                dum=(gamma*smrel-gd)/(2.*uwirel);
                wst=(aa-dum*pt)/ayns;
                %
                wsn(n)=ws(n)+wst*dt;
                %
                % constrain wsn(n) to prevent expansion shock
                %
                %if smrel < test;
                %    wsn(n)=(uwirel-sqrt(test*gamma))/ayns;
                %end;
                thsn(n)=ths(n)+ws(n)*dt;
                %
                if loop == 1;
                    wsn(n)=.5*(wso(n)+wsn(n));
                %
                % constrain wsn(n) to prevent expansion shock
                %
                %if smrel < test;
                %    wsn(n)=(uwirel-sqrt(test*gamma))/ayns;
                %end;
                    thsn(n)=.5*(thso(n)+thsn(n));
                end
            end
            %
            if m ~= mc;
                pn(n,m)=p(n,m)+pt*dt;
                vn(n,m)=v(n,m)+vt*dt;
                wn(n,m)=w(n,m)+wt*dt;
                sn(n,m)=s(n,m)+st*dt;
                if (m == 1) && (n ~= 1);
                    qn(n)=q(n)+qt*dt;
                end
                %
                if loop == 1;
                    %
                    pn(n,m)=.5*(po(n,m)+pn(n,m));
                    vn(n,m)=.5*(vo(n,m)+vn(n,m));
                    wn(n,m)=.5*(wo(n,m)+wn(n,m));
                    sn(n,m)=.5*(so(n,m)+sn(n,m));
                    if(m == 1) && (n ~= 1);
                        qn(n)=.5*(qo(n)+qn(n));
                    end
                end
            end
            %
        end % for m loop
    end; % for n loop
    %
    %     shock point calculation - given new shock shape
    %
    for n=2:na;
        wsx=.5*(wsn(n+1)-wsn(n-1))*ddx;
        thsx(n)=.5*(thsn(n+1)-thsn(n-1))*ddx;
        wsp(n)=wsx/(phcs(mc)-phisym);
        thsp(n)=thsx(n)/(phcs(mc)-phisym);
    end
    thsp(1)=0.;
    thsx(1)=0.;
    wsp(1)=0.;
    %
    for n=1:na;
        phi=ph(n,mc);
        sinphi=sin(phi);
        cosphi=cos(phi);
        the=thsn(n);
        costhe=cos(the);
        sinthe=sin(the);
        ent=1./sqrt(1.+(thsp(n)/sinthe)^2);
        enp=-thsp(n)*ent/sinthe;
        tat=enp;
        tap=-ent;
        etar=cosmu*costhe-sinmu*sinthe*cosphi;
        etat=-cosmu*sinthe-sinmu*costhe*cosphi;
        etap=sinmu*sinphi;
        uwinf=uinf*(etat*ent+etap*enp);
        vwinf=uinf*etar;
        wwinf=uinf*(etat*tat+etap*tap);
        sqr=1./sqrt((x(n)*phcsth(mc))^2+1./sinthe^2);
        ayt=sqr/sinthe;
        ayp=x(n)*phcsth(mc)*sqr;
        ayns=ayt*ent+ayp*enp;
        uwirel=uwinf-wsn(n)*ayns;
        smrel=uwirel^2/gamma;
        %
        %     Rankine-Hugoniot jumps
        %
        p2p1=(gamma*smrel/ge-1./gc);
        r2r1=ge*smrel/(gd*smrel+1.);
        uw2=uwirel/r2r1+wsn(n)*ayns;
        vw2=vwinf;
        ww2=wwinf;
        pn(n,mc)=log(p2p1);
        sn(n,mc)=pn(n,mc)-gamma*log(r2r1);
        vn(n,mc)=uw2*ent+ww2*tat;
        wn(n,mc)=uw2*enp+ww2*tap;
    end
    wn(1,mc)=0.;
    %
    %     wall boundary condition
    %
    for n=2:na;
        the=thw(n);
        ant=-thwp(n)/sin(the);
        bnt=-1./sqrt(1.+ant^2);
        ant=-ant*bnt;
        vn(n,1)=qn(n)*ant;
        wn(n,1)=qn(n)*bnt;
    end
    wn(1,1)=0.;
    vn(1,1)=0.;
    %
    if icut == 3;
    % for sharp corner
    % use known functional behavior near corner to override values
    %
    %corsym_local;
    %
    end
    %
    %     update
    %
    time=timen;
    for n=1:nc;
        qo(n)=q(n);
        q(n)=qn(n);
        wso(n)=ws(n);
        ws(n)=wsn(n);
        thso(n)=ths(n);
        ths(n)=thsn(n);
        for m=1:mc;
            th(n,m)=y(m)*(ths(n)-thw(n))+thw(n);
            po(n,m)=p(n,m);
            vo(n,m)=v(n,m);
            wo(n,m)=w(n,m);
            so(n,m)=s(n,m);
            p(n,m)=pn(n,m);
            v(n,m)=vn(n,m);
            w(n,m)=wn(n,m);
            s(n,m)=sn(n,m);
            t(n,m)=exp(p(n,m)/ga+s(n,m)/gamma);
            u(n,m)=sqrt(qmax2-v(n,m)^2-w(n,m)^2-2.*t(n,m)*ga);
        end
    end
end