%corsym_strm.m
% find streamlines trajectories in the computational plane
% and maps the resdult to the physical plane
% assumes flow is steady, uses R-K scheme
%
% ZI = griddata(x,y,z,XI,YI) fits a surface of the form z = f(x,y)
% to the data in the (usually) nonuniformly spaced vectors (x,y,z).
% griddata interpolates this surface at the points specified by (XI,YI)
% to produce ZI. The surface always passes through the data points.
% XI and YI usually form a uniform grid (as produced by meshgrid).
%
% Copyright (c) 2005 - 2009 by M.D.Salas;
%
global tryme th ph thw thsp x y err u v w p t s ws time dt dx dy ddx
global ddy na ma nc mc ach uinf wso phcs gamma gb ga gd ge gc gf timen
global phisym thwx thsx wsp wsn qmax2 pn k sn vn wn po so
global ths thwp thsn vo wo thso phcsy phcsth cax0 cay0 car0 phica tdel2
global talam2 tdelst icut q qn
%
clear all
load cor_save.mat tryme th ph thw thsp x y err u v w p t s ws time...
    dt dx dy ddx ddy na ma nc mc ach uinf wso phcs gamma gb ga gd ge...
    gc gf timen phisym thwx thsx wsp wsn qmax2 pn k sn...
    vn wn po so ths thwp thsn vo wo thso phcsy phcsth cax0 cay0 car0...
    phica tdel2 talam2 tdelst icut q qn
%
t1 = clock;

disp(' ')
nfig=1;
figure(nfig)
%
% find c-variant velocity components
%
for m=1:mc;
    for n=1:nc;
        the=th(n,m);
        phi=ph(n,m);
        costhe=cos(the);
        sinthe=sin(the);
        cot=costhe/sinthe;
        fth=-1./(ths(n)-thw(n));
        gph=1./(phisym-phcs(m));
        fx=-fth*(thwx(n)+y(m)*(thsx(n)-thwx(n)));
        gy=-x(n)*phcsy(m)*gph;
        djacob=1./(1.-fx*gy);
        xth=fth*gy*djacob;
        xph=-gph*djacob;
        yth=-fth*djacob;
        yph=fx*gph*djacob;
        xphs=xph/sinthe;
        yphs=yph/sinthe;
        U(n,m)=v(n,m)*xth+w(n,m)*xphs;
        
        V(n,m)=v(n,m)*yth+w(n,m)*yphs;
        XS(n,m)=x(n);
        YS(n,m)=y(m);
        X(n,m)=tan(the)*cos(phi);
        Y(n,m)=tan(the)*sin(phi);
    end
end
%
% find q=0 location
%
xzero=0;
for n=2:nc
    if q(n)*q(n-1) < 0;
        xzero=x(n-1)-q(n-1)*dx/(q(n)-q(n-1));
        break
    end
end
disp(['xzero= ' num2str(xzero)])
phzero=phisym-xzero*(phisym-phcs(1));
[thzero,dum]=corsym_geom(icut,phzero);
Xzero=tan(thzero)*cos(phzero);
Yzero=tan(thzero)*sin(phzero);
%
% integrate using R-K scheme
%
deta=min([dx,dy]);
jc=nc;
if deta == dy;
    jc=mc;
end
%
for loop=1:2

    if loop == 1

        for n=2:6:na
            t0 = clock;
            j=1;
            xs1=x(n);
            ys1=y(mc);
            %
            % find physical X,Y corresponding to xs1, ys1
            %
            thestr = griddata(XS,YS,th,xs1,ys1);
            phistr = griddata(XS,YS,ph,xs1,ys1);
            Xphys(j)=tan(thestr)*cos(phistr);
            Yphys(j)=tan(thestr)*sin(phistr);
            %
            JT=2*jc;
            if n == 2
                JT=3*jc;
            end
            for j=1:JT
                %
                % find physical X,Y corresponding to xs1, ys1
                %
                xs(j)=xs1;
                ys(j)=ys1;
                du1 = griddata(XS,YS,U,xs1,ys1);
                dv1 = griddata(XS,YS,V,xs1,ys1);
                xs(j)=xs1;
                ys(j)=ys1;

                xs2=xs(j)+.5*deta*du1;
                ys2=ys(j)+.5*deta*dv1;
                du2 = griddata(XS,YS,U,xs2,ys2);
                dv2 = griddata(XS,YS,V,xs2,ys2);

                xs3=xs(j)+.5*deta*du2;
                ys3=ys(j)+.5*deta*dv2;
                du3 = griddata(XS,YS,U,xs3,ys3);
                dv3 = griddata(XS,YS,V,xs3,ys3);

                xs4=xs(j)+deta*du3;
                ys4=ys(j)+deta*dv3;
                du4 = griddata(XS,YS,U,xs3,ys3);
                dv4 = griddata(XS,YS,V,xs3,ys3);

                jp=j+1;
                xs(jp)=xs(j)+deta*(du1+2*(du2+du3)+du4)/6;
                ys(jp)=ys(j)+deta*(dv1+2*(dv2+dv3)+dv4)/6;
                xs1=xs(jp);
                ys1=ys(jp);
                %
                % find physical X,Y corresponding to xs1, ys1
                %
                thestr = griddata(XS,YS,th,xs1,ys1);
                phistr = griddata(XS,YS,ph,xs1,ys1);
                Xphys(jp)=tan(thestr)*cos(phistr);
                Yphys(jp)=tan(thestr)*sin(phistr);

                dist=sqrt(ys1^2+(xs1-xzero)^2);
                if dist < deta/2;
                    break
                end

            end
            Xphys(jp+1)=Xzero;
            Yphys(jp+1)=Yzero;
            t2=etime(clock,t0);
            disp(['n= ' num2str(n) ' completed, xs1= ' num2str(xs1)...
                ' ys1= ' num2str(ys1) ', elapsed time= ' num2str(t2/60) ' min.' ]);
            plot(Xphys,Yphys,'k-')
            axis equal
            hold on
            clear xs ys Xphys Yphys
        end

    else

        for m=2:6:ma
            t0 = clock;
            j=1;
            xs1=x(nc);
            ys1=y(m);
            %
            % find physical X,Y corresponding to xs1, ys1
            %
            thestr = griddata(XS,YS,th,xs1,ys1);
            phistr = griddata(XS,YS,ph,xs1,ys1);
            Xphys(j)=tan(thestr)*cos(phistr);
            Yphys(j)=tan(thestr)*sin(phistr);
            %
            for j=1:2*jc
                %
                xs(j)=xs1;
                ys(j)=ys1;
                du1 = griddata(XS,YS,U,xs1,ys1);
                dv1 = griddata(XS,YS,V,xs1,ys1);
                xs(j)=xs1;
                ys(j)=ys1;

                xs2=xs(j)+.5*deta*du1;
                ys2=ys(j)+.5*deta*dv1;
                du2 = griddata(XS,YS,U,xs2,ys2);
                dv2 = griddata(XS,YS,V,xs2,ys2);

                xs3=xs(j)+.5*deta*du2;
                ys3=ys(j)+.5*deta*dv2;
                du3 = griddata(XS,YS,U,xs3,ys3);
                dv3 = griddata(XS,YS,V,xs3,ys3);

                xs4=xs(j)+deta*du3;
                ys4=ys(j)+deta*dv3;
                du4 = griddata(XS,YS,U,xs3,ys3);
                dv4 = griddata(XS,YS,V,xs3,ys3);

                jp=j+1;
                xs(jp)=xs(j)+deta*(du1+2*(du2+du3)+du4)/6;
                ys(jp)=ys(j)+deta*(dv1+2*(dv2+dv3)+dv4)/6;
                xs1=xs(jp);
                ys1=ys(jp);
                %
                % find physical X,Y corresponding to xs1, ys1
                %
                thestr = griddata(XS,YS,th,xs1,ys1);
                phistr = griddata(XS,YS,ph,xs1,ys1);
                Xphys(jp)=tan(thestr)*cos(phistr);
                Yphys(jp)=tan(thestr)*sin(phistr);
                %
                dist=sqrt(ys1^2+(xs1-xzero)^2);
                if dist < deta/2;
                    break
                end

            end
            Xphys(jp+1)=Xzero;
            Yphys(jp+1)=Yzero;
            t2=etime(clock,t0);
            disp(['m= ' num2str(m) ' completed, xs1= ' num2str(xs1)...
                ' ys1= ' num2str(ys1) ', elapsed time= ' num2str(t2/60) ' min.' ])
            plot(Xphys,Yphys,'k-')
            clear xs ys Xphys Yphys
        end

    end

end % for loop

for n=1:nc
    Xw(n)=X(n,1);
    Xsh_in(n)=X(n,mc);
    Yw(n)=Y(n,1);
    Ysh_in(n)=Y(n,mc);
end
for m=1:mc
    Xso1(m)=X(1,m);
    Yso1(m)=Y(1,m);
    Xso2(m)=X(nc,m);
    Yso2(m)=Y(nc,m);
end
Xa=[X(1,1),0];
Ya=[Y(1,1),0];
plot(Xw,Yw,'k-',Xa,Ya,'k-.')
plot(Xsh_in,Ysh_in,'k-')
xtop=[0,Xw(nc)];
ytop=[0,0];
plot(xtop,ytop,'k--',0,0,'k+')
plot(Xso1,Yso1,'k-',Xso2,Yso2,'k-')
title('Streamlines')
xlabel('X')
ylabel('Y')
hold off
t3=clock;
disp(['total time= ' num2str(etime(t3,t1)/60) ' min.' ])