function sw_geom(lite)
% sw_geom(lite)
%
%
% Copyright (c) 2005 - 2009 by M.D.Salas
%

global hhn h2 r0z h2zn gindex cone
global yzn bzzn h2zzn bhzn mcp m y yhn bn bhn bzn hn pio2 r0 zn
global ztop1 ytop1 ytopz1 ztop2 ytop2 ytopz2 at1 bt1 ct1 dt1 xtopz1

%     this routine defines r0,r0z,hhn,h2,h2zn,hn,yhn,yzn,bn,bhn,bzn
%     by mapping elliptical cross-section to a circle
%
%     z runs from z=zstar to z=dist
%     zn=z+dz
%
%      user must define yb=yb(zn)
%                       ybz=d(yb)/dz
%                       ybzz=d(ybz)/dz
%      where yb is the minor axis
%
%      user must define xa=xa(zn)
%                       xaz=d(xa)/dz
%                       xazz=d(xaz)/dz
%      where xa is the major axis
%
%     geometry for elliptic delta wing
%
if lite == 0 % defines initial geometry parameters
    if gindex == 1  % OGIVE CYLINDER COMBINATION
        disp('OGIVE CYLINDER COMBINATION')
        % 10 degree cone up to z=2 (ztop1) followed by a circular cylinder
        % at z=15 (ztop2) between z=2 and z=15 there is a cubic fit.
        ztop1=2;
        ztop2=15;
        ytopz1=tan(10*pi/180);
        ytopz2=0;
        ytop1=ytopz1*ztop1+1;
        ytop2=2;
        sw_cubic;
    elseif gindex == 2 % ELLIPTICAL WING
        disp('ELLIPTICAL WING')
        % 10 degree angle on lee & wind-ward planes
        % 10 degree up to z=2 (ztop1)
        % between z=2 and z=15 there is a cubic fit.
        % 20 degree angle on horizontal plane
        ztop1=2;
        ztop2=15;
        ytopz1=tan(10*pi/180);
        ytopz2=0;
        ytop1=ytopz1*ztop1+1;
        ytop2=2;
        sw_cubic;
        xtopz1=tan(20*pi/180);
    elseif gindex == 3 % uses value defined for "cone"
        ytopz1=tan(cone);
    end
end

if lite == 1 % computes yb(zn) & xa(zn) from user defined geometry
    if gindex == 1  % OGIVE CYLINDER COMBINATION
        if zn < ztop1
            yb=ytopz1*zn+1;
            ybz=ytopz1;
            ybzz=0;
        elseif zn < ztop2
            yb=at1+(bt1+(ct1+dt1*zn)*zn)*zn;
            ybz=bt1+(2*ct1+3*dt1*zn)*zn;
            ybzz=2*ct1+6*dt1*zn;
        else
            yb=ytop2;
            ybz=0;
            ybzz=0;
        end
        xa=yb;
        xaz=ybz;
        xazz=ybzz;
    elseif gindex == 2 % ELLIPTICAL WING
        if zn < ztop1
            yb=ytopz1*zn+1;
            ybz=ytopz1;
            ybzz=0;
        elseif zn < ztop2
            yb=at1+(bt1+(ct1+dt1*zn)*zn)*zn;
            ybz=bt1+(2*ct1+3*dt1*zn)*zn;
            ybzz=2*ct1+6*dt1*zn;
        else
            yb=ytop2;
            ybz=0;
            ybzz=0;
        end
        xa=xtopz1*(zn+1/ytopz1);
        xaz=xtopz1;
        xazz=0;
    elseif gindex ==3 % CIRCULAR CONE, CONE ANGLE DEFINED BY "CONE"
        yb=ytopz1*zn+1;
        ybz=ytopz1;
        ybzz=0;
        xa=yb;
        xaz=ybz;
        xazz=ybzz;
    end
    
    r0=(xa+yb)/2;
    r0z=(xaz+ybz)/2;
    r0zz=(xazz+ybzz)/2;
    if abs(r0-yb) < 1.e-4
        r0=yb;
    end
    hhn=sqrt(r0*(r0-yb));
    h2=hhn^2;
    h2zn=r0z*(2*r0-yb)-r0*ybz;
    h2zzn=r0zz*(2*r0-yb)+2*r0z*(r0z-ybz)-r0*ybzz;
    
    for m=1:mcp
        yhn(m)=1;
        yzn(m)=0;
        hn(m)=y(m)-pio2;
        bhn(m)=0;
        bhzn(m)=0;
        bn(m)=r0;
        bzn(m)=r0z;
        bzzn(m)=r0zz;
    end
    
end