% sw_initial
% Find Taylor-Maccoll Solution Using 4th Order Runge-Kutta Integration Scheme
%
% Copyright (c) 2005 - 2009 by M.D.Salas
%
global h2 k nc mc mcm mcp  m gamma ga gb gc gd ge vrad ang uu vv tt
global h cz bz c b x ch bh sinthe costhe r p s w u v t xx yy
global yh yhn bn bhn bzn hn cossin guess_shk_ang body_angle nfig
global yz yzn bzz bhz bzzn bhzn chz chzn cone vin hst dx dy vl2 vl3
%
% exact for attack=0, approximate for attack ~= 0, or elliptical body
%
tolerance=1.e-8;
kf=500; % intervals between shock and body + 1
run=[1/4,1/3,1/2,1]; % Runge-Kutta coefficients

for m=2:mc
    xbod=(bn(m)+h2/bn(m))*costhe(m); % x coordinate to point on body
    ybod=(bn(m)-h2/bn(m))*sinthe(m); % y coordinate to point on body
    dbod=sqrt(xbod^2+ybod^2); % distance from origin in x,y plane to point on body
    zstar=1/tan(cone); % location on z axis of cone origin
    body_angle=atan(dbod/zstar);
    
    if m == 2 % start with guess from correlation for m=2, for m>2 use previous guess
        if guess_shk_ang == -1;
            sw_shockangle;
            tryme(1)=guess_shk_ang;
        else % user provided value
            tryme(1)=guess_shk_ang*pi/180;
        end
    end
    
    me=1;
    for kip=1:20
        if kip == 2
            % disp([ 'm= ', num2str(m),' shock angle = ', num2str(tryme(me)*180/pi) ])
            me=2;
            tryme(2)=1.05*tryme(1);
        end
        shock=tryme(me); % guess shock angle
        dth=(body_angle-shock)/(kf-1);
        k=1; % at shock from R-H
        vninf=vin*(vl2*sinthe(m)*cos(shock)-vl3*sin(shock));
        smnin=vninf^2/gamma;
        vi=vninf*(1+gd*smnin)/ge/smnin;
        av=vi;
        rs=vninf/av;
        au=vin*(vl2*sin(shock)*sinthe(m)+vl3*cos(shock));
        ps=(1-gc*rs)/(rs-gc);
        ang(1)=shock;
        rad=tan(shock);
        a2=(hst-(au^2+av^2)/2)/gb;
        uu(1)=au;
        vv(1)=av;
        tt(1)=a2/gamma;
        entr=(ga*log(tt(k))-log(ps))/gb;
        vhsave(1)=-au+(au+av/rad)/(av^2/a2-1);
        %
        % Runge-Kutta Integration
        %
        for k=2:kf
            auo=au;
            avo=av;
            for L=1:4
                vh=-au+(au+av/rad)/(av^2/a2-1);
                uh=av;
                av=avo+run(L)*vh*dth;
                au=auo+run(L)*uh*dth;
                a2=(hst-(au^2+av^2)*.5)/gb;
            end
            ang(k)=shock+(k-1)*dth;
            rad=tan(ang(k));
            tt(k)=a2/gamma;
            uu(k)=au;
            vv(k)=av;
            vhsave(k)=vh; % saved for plotting
        end % k loop
         
        err(me)=vv(kf); % vv is the velocity comp. normal to the surface
        % disp([ 'm= ', num2str(m),' kip= ', num2str(kip),' err= ',...
        % num2str(err(me)),' shock angle= ', num2str(tryme(me)*180/pi) ])
        
        if abs(err(me)) < tolerance
            break
        end
        
        if kip >= 2
            tryb=tryme(1)-err(1)*(tryme(2)-tryme(1))/(err(2)-err(1));
            tryme(1)=tryme(2);
            err(1)=err(2);
            tryme(2)=tryb;
        end
    end % for kip loop
    %
    if abs(err(me)) > tolerance
        disp(' ')
        disp('SUBROUTINE SW_INITIAL: ')
        disp('Initial circular cone solution failed to converge.')
        disp(['The error is ' num2str(err(me)) ', at m=' num2str(m)])
        disp(' ')
        % The usual problem is that the shock becomes infinitesimally weak
        % and the denumerator in vh=-au+(au+av/tan(ang(k)))/(av^2/a2-1.)
        % goes to zero.
    end
    
    if m == 2
        nfig=nfig+1; % plot dv/dtheta from Taylor Maccoll solution
        figure(nfig)
        plot(ang*180/pi,vhsave,'b-')
        xlabel('angle')
        ylabel('dv/dtheta')
        title(['Initial Taylor-Maccoll cone solution, m= ',num2str(m)])
        grid on
        nfig=nfig+1; % plot v from Taylor Maccoll solution
        figure(nfig)
        plot(ang*180/pi,vv,'b-')
        xlabel('angle')
        ylabel('v')
        title(['Initial Taylor-Maccoll cone solution, m= ',num2str(m)])
        grid on
    end

    bz(m)=bzn(m);
    b(m)=bn(m);
    dsh=zstar*tan(shock); % distance from origin (x,y plane) to point on shock
    xsh=dsh*costhe(m); % x coordinate at the shock
    ysh=dsh*sinthe(m); % y coordinate at the shock
    q=dsh^2-2*h2*cossin(m);
    csh2=.5*(q+sqrt(q^2-4*h2^2)); % follows from inverting the mapping
    c(m)=sqrt(csh2);
    cz(m)=c(m)/zstar;
    ang(kf)=body_angle;
    %
    for n=1:nc % interpolate tangent-cone data to mesh
        x(n)=dx*(n-1);
        rad=b(m)+(c(m)-b(m))*x(n);
        xpoint=(rad+h2/rad)*costhe(m); % x coordinate to a point in layer
        ypoint=(rad-h2/rad)*sinthe(m); % y coordinate to a point in layer
        dpoint=sqrt(xpoint^2+ypoint^2);
        angle=atan(dpoint/zstar);
        if n == nc
            uu1=uu(1);
            vv1=vv(1);
            t(nc,m)=tt(1);
            s(nc,m)=entr;
            p(nc,m)=ga*log(t(nc,m))-gb*s(nc,m);
            w(nc,m)=uu1*cos(angle)-vv1*sin(angle);
            vrad=uu1*sin(angle)+vv1*cos(angle);
            u(nc,m)=vrad*costhe(m);
            v(nc,m)=vrad*sinthe(m);
        else
            for k=2:kf-1
                if (ang(k)-angle)*(ang(k+1)-angle) <= 0
                    ep=(angle-ang(k+1))/(ang(k)-ang(k+1));
                    uu1=uu(k+1)+ep*(uu(k)-uu(k+1));
                    vv1=vv(k+1)+ep*(vv(k)-vv(k+1));
                    t(n,m)=tt(k+1)+ep*(tt(k)-tt(k+1));
                    s(n,m)=entr;
                    p(n,m)=ga*log(t(n,m))-gb*s(n,m);
                    w(n,m)=uu1*cos(angle)-vv1*sin(angle);
                    vrad=uu1*sin(angle)+vv1*cos(angle);
                    u(n,m)=vrad*costhe(m);
                    v(n,m)=vrad*sinthe(m);
                end % if
            end % for k loop
        end % if
    end % for n loop
    %
    %
    if m == 2
        angrat=shock/cone;
        disp(['Initial (shock angle)/(cone angle)= ',num2str(angrat)...
            ' on Windward plane'])
    end
    %
    %
    tryme(1)=shock;
end; % for m loop

c(1)=c(3);
c(mcp)=c(mcm);
for m=2:mc
    ch(m)=(c(m+1)-c(m-1))/dy;
    %    s(1,m)=s(1,2); % entropy on windward plane wets the body
    %    t(1,m)=exp(p(1,m)/ga+s(1,m)/gamma);
end

for n=1:nc
    p(n,1)=p(n,3);
    v(n,1)=v(n,3);
    w(n,1)=w(n,3);
    s(n,1)=s(n,3);
    u(n,1)=-u(n,3);
    p(n,mcp)=p(n,mcm);
    v(n,mcp)=v(n,mcm);
    w(n,mcp)=w(n,mcm);
    s(n,mcp)=s(n,mcm);
    u(n,mcp)=-u(n,mcm);
    u(n,2)=0.;
    u(n,mc)=0.;
end

ch(2)=0.;
ch(mc)=0.;
c(1)=c(3);
cz(1)=cz(3);
c(mcp)=c(mcm);
cz(mcp)=cz(mcm);
ch(mcp)=-ch(mcm);
ch(1)=-ch(3);

for m=1:mcp
    yh(m)=yhn(m);
    yz(m)=yzn(m);
    b(m)=bn(m);
    bh(m)=bhn(m);
    bz(m)=bzn(m);
    bzz(m)=bzzn(m);
    bhz(m)=bhzn(m);
    if (m > 2) && (m < mc);
        chz(m)=.5*(cz(m+1)-cz(m-1))/dy;
        chzn(m)=chz(m);
    end
    h(m)=hn(m);
    for n=1:nc
        r(n,m)=b(m)+x(n)*(c(m)-b(m));
        yy(n,m)=(r(n,m)-h2/r(n,m))*sinthe(m);
        xx(n,m)=(r(n,m)+h2/r(n,m))*costhe(m);
    end
end

chz(2)=0.;
chz(mc)=0.;
chz(1)=-chz(3);
chz(mcp)=-chz(mcm);
chzn(2)=0.;
chzn(mc)=0.;
chzn(1)=-chzn(3);
chzn(mcp)=-chzn(mcm);

% clear arrays no longer needed
clear ang tt uu vv vhsave

